﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using OuDot.ServicesOudot;

namespace OuDot.Model
{

    public partial class OU_ODVOZ_UKAZ : IDataErrorInfo
    {

        #region IDataErrorInfo Members

        string IDataErrorInfo.Error
        {
            get { throw new NotImplementedException(); }
        }

        string IDataErrorInfo.this[string columnName]
        {
            get
            {

                List<ValidacniChyba> chyby = new List<ValidacniChyba>();
                switch (columnName)
                {
                    case "NAZEV":
                        //Kontrola jmené konvence Oracle
                        string chybaNazev = Validace.JmenaKonvenceOracle(this.NAZEV);
                        if (!string.IsNullOrEmpty(chybaNazev))
                        {
                            //return chybaJmena;
                            chyby.Add(new ValidacniChyba(chybaNazev, "OU_ODVOZ_UKAZ", this, columnName, this));
                        }
                        //Kontrola existence stejného jména ou ve stejné tabulce
                        if (this.OU_TAB.OU_ODVOZ_UKAZ.Any(o => (o.NAZEV.ToUpper() == this.NAZEV.ToUpper()) && (o != this)))
                        {
                            chyby.Add(new ValidacniChyba($"Odvozený ukazatel má stejný název ({this.NAZEV}) jako jiný odvozený ukazatel ve stejné tabulce.", "OU_ODVOZ_UKAZ", this, columnName, this));
                        }
                        //Kontrola existence stejného jména OU a sloupce PK ve stejné tabulce
                        if (this.OU_TAB.OU_PRIMARNI_KLICE.Any(p => p.JMENO.ToUpper() == this.NAZEV.ToUpper()))
                        {
                            chyby.Add(new ValidacniChyba($"Odvozený ukazatel má stejný název ({this.NAZEV}) jako sloupec PK ve stejné tabulce.", "OU_ODVOZ_UKAZ", this, columnName, this));
                        }
                        break;
                    case "NAZEV_EU":
                        //Kontrola jmené konvence Oracle
                        if (!string.IsNullOrEmpty(this.NAZEV_EU))
                        {
                            string chybaNazev_EU = Validace.JmenaKonvenceOracle(this.NAZEV_EU);
                            if (!string.IsNullOrEmpty(chybaNazev_EU))
                            {
                                chyby.Add(new ValidacniChyba(chybaNazev_EU, "OU_ODVOZ_UKAZ", this, columnName, this));
                            }
                        }

                        //Kontrola existence stejného Názvu_EU ou ve stejné tabulce
                        var vsechnyNeNuloveEUNazny = this.OU_TAB.OU_ODVOZ_UKAZ.Where(o => !string.IsNullOrEmpty(o.NAZEV_EU));
                        if (!string.IsNullOrEmpty(this.NAZEV_EU) && vsechnyNeNuloveEUNazny.Any(o => (o.NAZEV_EU.ToUpper() == this.NAZEV_EU.ToUpper()) && (o != this)))
                        {
                            chyby.Add(new ValidacniChyba("Odvozený ukazatel má stejný název EU jako jiný odvozený ukazatel ve stejné tabulce: " + this.NAZEV_EU, "OU_ODVOZ_UKAZ", this, columnName, this));
                        }
                        break;
                    case "DATA_TYP":
                        if (this.DATA_TYP == 0)
                        {
                            chyby.Add(new ValidacniChyba("Není vybraný datový typ.", "OU_ODVOZ_UKAZ", this, columnName, this));
                        }
                        break;
                    case "DATA_TYP_P1":
                        if (this.DATA_TYP_P1 == null) break;//Pokud není vyplněn není potřeba kontrolovat
                        switch ((DatoveTypy)this.DATA_TYP)
                        {

                            case DatoveTypy.interval_den:
                            case DatoveTypy.interval_rok:

                                if (this.DATA_TYP_P1 > 9)
                                {
                                    chyby.Add(new ValidacniChyba("Pro vybraný datový typ je maximální hodnota parametru 9.", "OU_ODVOZ_UKAZ", this, columnName, this));
                                }
                                break;
                            case DatoveTypy.číslo:
                                if (this.DATA_TYP_P1 > 38 || this.DATA_TYP_P1 < 1)
                                {
                                    chyby.Add(new ValidacniChyba("Pro vybraný datový typ se musí parametr nacházet v rozmezí 1-38.", "OU_ODVOZ_UKAZ", this, columnName, this));
                                }
                                break;
                            case DatoveTypy.text:

                                if (this.DATA_TYP_P1 < 1)
                                {
                                    chyby.Add(new ValidacniChyba("Pro vybraný datový typ je minimální hodnota parametru 1.", "OU_ODVOZ_UKAZ", this, columnName, this));
                                }
                                break;
                            default:
                                break;
                        }
                        break;
                    case "DATA_TYP_P2":
                        if (this.DATA_TYP_P2 == null) break;//Pokud není vyplněn není potřeba kontrolovat
                        switch ((DatoveTypy)this.DATA_TYP)
                        {

                            case DatoveTypy.interval_den:
                                if (this.DATA_TYP_P2 > 9 || this.DATA_TYP_P2 < 0)
                                {
                                    chyby.Add(new ValidacniChyba("Pro vybraný datový typ se musí parametr nacházet v rozmezí 0 - 9.", "OU_ODVOZ_UKAZ", this, columnName, this));
                                }
                                break;
                            case DatoveTypy.číslo:
                                if (this.DATA_TYP_P1 == null)
                                {
                                    chyby.Add(new ValidacniChyba("Pokud je vyplněn parametr 2 musí být vyplněn i parametr 1.", "OU_ODVOZ_UKAZ", this, columnName, this));
                                }
                                if (this.DATA_TYP_P2 > 127)
                                {
                                    chyby.Add(new ValidacniChyba("Pro vybraný datový typ se musí parametr nacházet v rozmezí -84-127.", "OU_ODVOZ_UKAZ", this, columnName, this));
                                }
                                break;

                            default:
                                break;
                        }
                        break;
                    case "SPOJ":
                        if (string.IsNullOrEmpty(this.SPOJ) || (this.SPOJ.ToLower() != "česká" && this.SPOJ.ToLower() != "evropská" && this.SPOJ.ToLower() != "společná"))
                        {
                            chyby.Add(new ValidacniChyba("Povolená hodnota je pouze: \"česká\", \"evropská\", \"společná\".", "OU_ODVOZ_UKAZ", this, columnName, this));
                        }
                        break;
                    case "OU_TYP":
                        if (this.OU_TYP == null)
                        {
                            chyby.Add(new ValidacniChyba("Není vybraný typ odvozeného ukazatele.", "OU_ODVOZ_UKAZ", this, columnName, this));
                        }
                        break;
                    case "GENERACE":
                        if (this.GENERACE == null)
                        {
                            chyby.Add(new ValidacniChyba("Generace musí být vyplněna.", "OU_ODVOZ_UKAZ", this, columnName, this));

                        }

                        //Kontrola zdrojových OU:
                        foreach (var ZdrojovyOU in this.OU_ODVOZ_UKAZ1)
                        {
                            if (ZdrojovyOU.OU_TAB == this.OU_TAB)//pokud je přidávaný zdroj ze stejné tabulky, kontroluji generaci a pořadí
                            {
                                //kontrola generace + pořadí
                                if (this.GENERACE < ZdrojovyOU.GENERACE)
                                {
                                    chyby.Add(new ValidacniChyba("Generace zdrojového odvozeného ukazatele je vyšší.", "OU_ODVOZ_UKAZ", this, columnName, this));
                                }
                            }
                        }

                        //Kontrola vlastnících OU:
                        foreach (var vlastOU in this.OU_ODVOZ_UKAZ2)
                        {
                            if (vlastOU.OU_TAB == this.OU_TAB)//pokud je zdroj ze stejné tabulky, kontroluji generaci a pořadí
                            {
                                //kontrola generace
                                if (this.GENERACE > vlastOU.GENERACE)
                                {
                                    string zprava = string.Format("Generace vlastnícího odvozeného ukazatele ({0}) je nižší.", vlastOU.NAZEV);
                                    chyby.Add(new ValidacniChyba(zprava, "OU_ODVOZ_UKAZ", this, columnName, this));
                                }
                            }
                        }
                        break;
                    case "PORADI_VYPOCTU":

                        var ouStejneGeneraceVTabulce = this.OU_TAB.OU_ODVOZ_UKAZ.Where(g => (g.GENERACE == this.GENERACE) && (g != this));
                        if (ouStejneGeneraceVTabulce.Any(p => p.PORADI_VYPOCTU == this.PORADI_VYPOCTU))
                        {
                            chyby.Add(new ValidacniChyba("Pořadí výpočtu je již použito ve stejné generaci.", "OU_ODVOZ_UKAZ", this, columnName, this));
                        }

                        //Kontrola zdrojových OU
                        string chyba = Validace.KontrolaZdroju(this);
                        if (chyba != null)
                        {
                            chyby.Add(new ValidacniChyba(chyba, "OU_ODVOZ_UKAZ", this, columnName, this));
                        }

                        //STARÝ KÓD PŘI VÝPOČTU PODLE POŘADÍ TABULEK
                        //foreach (var ZdrojovyOU in this.OU_ODVOZ_UKAZ1)
                        //{
                        //    if (ZdrojovyOU.OU_TAB == this.OU_TAB)//pokud je přidávaný zdroj ze stejné tabulky, kontroluji generaci a pořadí
                        //    {
                        //        if (this.GENERACE == ZdrojovyOU.GENERACE)
                        //        {
                        //            if (this.PORADI_VYPOCTU < ZdrojovyOU.PORADI_VYPOCTU)
                        //            {
                        //                chyby.Add(new ValidacniChyba("Pořadí výpočtu zdrojového odvozeného ukazatele je vyšší.", "OU_ODVOZ_UKAZ", this, columnName, this));
                        //            }
                        //        }
                        //    }
                        //}

                        //Kontrola vlastnících OU:
                        string chyba2 = Validace.KontrolaVlastnenychOU(this);
                        if (chyba2 != null)
                        {
                            chyby.Add(new ValidacniChyba(chyba2, "OU_ODVOZ_UKAZ", this, columnName, this));
                        }
                        //STARÝ KÓD PŘI VÝPOČTU PODLE POŘADÍ TABULEK
                        //foreach (var vlastOU in this.OU_ODVOZ_UKAZ2)
                        //{
                        //    if (vlastOU.OU_TAB == this.OU_TAB)//pokud je přidávaný zdroj ze stejné tabulky, kontroluji generaci a pořadí
                        //    {
                        //        if (this.GENERACE == vlastOU.GENERACE)
                        //        {
                        //            if (this.PORADI_VYPOCTU > vlastOU.PORADI_VYPOCTU)
                        //            {
                        //                string zprava = string.Format("Pořadí výpočtu vlastnícího odvozeného ukazatele ({0}) je nižší.", vlastOU.NAZEV);
                        //                chyby.Add(new ValidacniChyba(zprava, "OU_ODVOZ_UKAZ", this, columnName, this));
                        //            }
                        //        }
                        //    }
                        //}
                        break;
                    case "POPIS":
                        if (string.IsNullOrEmpty(this.POPIS))
                        {
                            chyby.Add(new ValidacniChyba("Popis musí být vyplněn.", "OU_ODVOZ_UKAZ", this, columnName, this));
                        }

                        break;
                    case "METODIKA_ALGORITMU":
                        if (string.IsNullOrEmpty(this.METODIKA_ALGORITMU))
                        {
                            chyby.Add(new ValidacniChyba("Metodika musí být vyplněna.", "OU_ODVOZ_UKAZ", this, columnName, this));
                        }

                        break;

                    default:
                        break;
                }

                var chybyOld = Validace.ValidacniChyby.Where(v => (v.Objekt == this) && (v.ChybnaProperty == columnName)).ToList();
                Validace.ValidacniChyby.RemoveRange(chybyOld);
                Validace.ValidacniChyby.AddRange(chyby);
                if (chyby.Count > 0)
                {
                    return chyby[0].Chyba;
                }
                else
                    return null;
            }
        }

        #endregion
    }

    public partial class OU_CATEGORY : IDataErrorInfo
    {

        #region IDataErrorInfo Members

        string IDataErrorInfo.Error
        {
            get { throw new NotImplementedException(); }
        }

        string IDataErrorInfo.this[string columnName]
        {
            get
            {
                if (this.OU_ODVOZ_UKAZ == null) return null;//
                List<ValidacniChyba> chyby = new List<ValidacniChyba>();
                switch (columnName)
                {

                    case "HODNOTA":

                        if (this.OU_ODVOZ_UKAZ.OU_CATEGORY.Any(c => (c.HODNOTA == this.HODNOTA) && (c != this)))
                        {
                            chyby.Add(new ValidacniChyba("Hodnota musí být jedinečná.", "OU_CATEGORY", this, columnName, this.OU_ODVOZ_UKAZ));
                        }

                        break;
                    case "POPIS":
                        if (string.IsNullOrEmpty(this.POPIS))
                        {
                            chyby.Add(new ValidacniChyba("Pole nesmí být prázdné.", "OU_CATEGORY", this, columnName, this.OU_ODVOZ_UKAZ));
                        }
                        break;

                    default:
                        break;
                }

                var chybyOld = Validace.ValidacniChyby.Where(v => (v.Objekt == this) && (v.ChybnaProperty == columnName)).ToList();
                Validace.ValidacniChyby.RemoveRange(chybyOld);
                Validace.ValidacniChyby.AddRange(chyby);
                if (chyby.Count > 0)
                {
                    return chyby[0].Chyba;
                }
                else
                    return null;
            }
        }

        #endregion
    }


    public partial class OU_TAB : IDataErrorInfo//IValidatableObject
    {

        #region IDataErrorInfo Members

        string IDataErrorInfo.Error
        {
            get { throw new NotImplementedException(); }
        }

        string IDataErrorInfo.this[string columnName]
        {
            get
            {

                List<ValidacniChyba> chyby = new List<ValidacniChyba>();
                switch (columnName)
                {

                    case "JMENO":

                        //Kontrola jmené konvence Oracle
                        string chybaJmena = Validace.JmenaKonvenceOracle(this.JMENO);
                        if (!string.IsNullOrEmpty(chybaJmena))
                        {
                            //return chybaJmena;
                            chyby.Add(new ValidacniChyba(chybaJmena, "OU_TAB", this, "JMENO", this));
                        }
                        //Kontrola správnosti prefixu tabulky
                        if (!this.JMENO.StartsWith(Validace.PrefixJmenaTabulky(this.ULOHA_METOD_OBDOBI) + "_"))
                        {
                            //return "Jméno tabulky musí začínat prefixem: " + Validace.PrefixJmenaTabulky(this.ULOHA_METOD_OBDOBI);
                            chyby.Add(new ValidacniChyba("Jméno tabulky musí začínat prefixem: " + Validace.PrefixJmenaTabulky(this.ULOHA_METOD_OBDOBI), "OU_TAB", this, "JMENO", this));
                        }
                        //Kontrola existence stejného jména tabulky ve stejném období
                        if (this.ULOHA_METOD_OBDOBI.OU_TAB.Any(t => (t.JMENO.ToUpper() == this.JMENO.ToUpper()) && (t.ID != this.ID)))
                        {
                            //return "Vybrané období obsahuje tabulku se stejným jménem: " + this.JMENO;
                            chyby.Add(new ValidacniChyba("Vybrané období obsahuje tabulku se stejným jménem: " + this.JMENO, "OU_TAB", this, "JMENO", this));
                        }
                        break;

                    case "OU_PRIMARNI_KLICE":

                        if (this.OU_PRIMARNI_KLICE == null || this.OU_PRIMARNI_KLICE.Count <= 0)
                        {
                            chyby.Add(new ValidacniChyba("Primární klíč nesmí být prázdný.", "OU_TAB", this, "OU_PRIMARNI_KLICE", this));
                        }
                        break;

                    case "PORADI":

                        if (this.ULOHA_METOD_OBDOBI.OU_TAB.Any(t => (t.PORADI == this.PORADI) && (t != this)))
                        {
                            chyby.Add(new ValidacniChyba("Stejné pořadí pro více tabulek.", "OU_TAB", this, "PORADI", this));
                        }



                        //Kontrola zdrojových OU
                        foreach (var OU in this.OU_ODVOZ_UKAZ)
                        {
                            string chyba = Validace.KontrolaZdroju(OU);
                            if (chyba != null)
                            {
                                string zprava = string.Format("Tabulka obsahuje OU ({0}) s chybným zdrojem.{2}Chyba: {1}", OU.NAZEV, chyba, Environment.NewLine);
                                chyby.Add(new ValidacniChyba(zprava, "OU_TAB", this, columnName, this));
                            }
                        }
                        //STARÝ KÓD PŘI VÝPOČTU PODLE POŘADÍ TABULEK
                        //var zdrojoveOUzJineTabulky = this.OU_ODVOZ_UKAZ.SelectMany(z => z.OU_ODVOZ_UKAZ1).Where(z => z.OU_TAB != this);
                        //foreach (var ZdrojovyOU in zdrojoveOUzJineTabulky)
                        //{
                        //    if (ZdrojovyOU.OU_TAB.PORADI >= this.PORADI)//pokud se tabulky neshodují musí být pořadí tabulky zdrojového OU menší
                        //    {
                        //        string zprava = string.Format("Tabulka obsahuje zdrojové odvozeného ukazatele ({0}) z jiné tabulky ({1}), která má vyšší pořadí výpočtu.", ZdrojovyOU.NAZEV, ZdrojovyOU.OU_TAB.JMENO);
                        //        chyby.Add(new ValidacniChyba(zprava, "OU_TAB", this, columnName, this));
                        //    }
                        //}

                        ////Kontrola vlastnících OU:
                        foreach (var OU in this.OU_ODVOZ_UKAZ)
                        {
                            string chyba = Validace.KontrolaVlastnenychOU(OU);
                            if (chyba != null)
                            {
                                string zprava = string.Format("Tabulka obsahuje OU ({0}) vlastněný jako zdroj v jiné tabulce s následující chybou: {1}", OU.NAZEV, chyba, Environment.NewLine);
                                chyby.Add(new ValidacniChyba(zprava, "OU_TAB", this, columnName, this));
                            }
                        }
                        //STARÝ KÓD PŘI VÝPOČTU PODLE POŘADÍ TABULEK
                        //var vlastnícíOUzJineTabulky = this.OU_ODVOZ_UKAZ.SelectMany(z => z.OU_ODVOZ_UKAZ2).Where(z => z.OU_TAB != this);
                        //foreach (var vlastOU in vlastnícíOUzJineTabulky)
                        //{
                        //    if (vlastOU.OU_TAB.PORADI <= this.PORADI)//pokud se tabulky neshodují musí být pořadí tabulky zdrojového OU menší
                        //    {
                        //        string zprava = string.Format("Tabulka obsahuje odvozený ukazatel ({0}) vlastněný v jiné tabulce s nižším pořadím výpočtu ({1}) jako zdroje.", vlastOU.NAZEV, vlastOU.OU_TAB.JMENO);
                        //        chyby.Add(new ValidacniChyba(zprava, "OU_TAB", this, columnName, this));
                        //    }
                        //}
                        break;

                    case "SQL_GENEROVANI_PK":
                        if (this.OU_PRIMARNI_KLICE == null || this.OU_PRIMARNI_KLICE.Count <= 0)
                        {
                            chyby.Add(new ValidacniChyba("SQL pro generování primárního klíče nesmí být prázdné.", "OU_TAB", this, "SQL_GENEROVANI_PK", this));
                        }
                        if (this.OU_PRIMARNI_KLICE.Count > 2000)
                        {
                            chyby.Add(new ValidacniChyba("SQL pro generování primárního klíče nesmí obsahovat víc jak 2000 znaků.", "OU_TAB", this, "SQL_GENEROVANI_PK", this));
                        }
                        break;
                    default:
                        return null;



                }
                var chybyOld = Validace.ValidacniChyby.Where(v => (v.Objekt == this) && (v.ChybnaProperty == columnName)).ToList();
                Validace.ValidacniChyby.RemoveRange(chybyOld);
                Validace.ValidacniChyby.AddRange(chyby);
                if (chyby.Count > 0)
                {
                    return chyby[0].Chyba;
                }
                else
                    return null;

            }
        }

        #endregion
    }

    public partial class EXT_TABULKY : IDataErrorInfo
    {

        #region IDataErrorInfo Members

        string IDataErrorInfo.Error
        {
            get { throw new NotImplementedException(); }
        }

        string IDataErrorInfo.this[string columnName]
        {
            get
            {
                List<ValidacniChyba> chyby = new List<ValidacniChyba>();
                switch (columnName)
                {

                    case "JMENO_ID":
                        //Jméno tabulky musí začínat prefixem EXT_
                        if (!this.JMENO_ID.ToUpper().StartsWith("EXT_"))
                        {
                            chyby.Add(new ValidacniChyba("Jméno tabulky musí začínat prefixem EXT_", "EXT_TABULKY", this, columnName, this));
                        }
                        //Kontrola jmené konvence Oracle
                        string chybaNazev = Validace.JmenaKonvenceOracle(this.JMENO_ID);
                        if (!string.IsNullOrEmpty(chybaNazev))
                        {
                            //return chybaJmena;
                            chyby.Add(new ValidacniChyba(chybaNazev, "EXT_TABULKY", this, columnName, this));
                        }
                        //Kontrola existence stejného jména ou ve stejné tabulce
                        var VsechnyExtTabulky = DBContext.Instance.Entity.EXT_TABULKY.ToList();
                        if (VsechnyExtTabulky.Any(o => (o.JMENO_ID.ToUpper() == this.JMENO_ID.ToUpper()) && (o != this)))
                        {
                            chyby.Add(new ValidacniChyba(String.Format("Tabulka s názvem {0}, již existuje.", this.JMENO_ID), "EXT_TABULKY", this, columnName, this));
                        }
                        break;
                    case "POPIS":
                        if (string.IsNullOrEmpty(this.POPIS))
                        {
                            chyby.Add(new ValidacniChyba("Popis nesmí být prázdný.", "EXT_TABULKY", this, columnName, this));
                        }
                        break;
                    case "PLATNOST_OD":
                        if (this.PLATNOST_OD == null)
                        {
                            chyby.Add(new ValidacniChyba("Platnost od musí být zadána.", "EXT_TABULKY", this, columnName, this));
                        }
                        if (this.PLATNOST_OD >= this.PLATNOST_DO)
                        {
                            chyby.Add(new ValidacniChyba("Platnost do musí být vyšší než platnost od.", "EXT_TABULKY", this, columnName, this));
                        }
                        break;
                    case "PLATNOST_DO":
                        if (this.PLATNOST_OD >= this.PLATNOST_DO)
                        {
                            chyby.Add(new ValidacniChyba("Platnost do musí být vyšší než platnost od.", "EXT_TABULKY", this, columnName, this));
                        }
                        break;
                    default:
                        break;
                }

                var chybyOld = Validace.ValidacniChyby.Where(v => (v.Objekt == this) && (v.ChybnaProperty == columnName)).ToList();
                Validace.ValidacniChyby.RemoveRange(chybyOld);
                Validace.ValidacniChyby.AddRange(chyby);
                if (chyby.Count > 0)
                {
                    return chyby[0].Chyba;
                }
                else
                    return null;
            }
        }

        #endregion
    }


    public partial class EXT_SLOUPCE : IDataErrorInfo
    {

        #region IDataErrorInfo Members

        string IDataErrorInfo.Error
        {
            get { throw new NotImplementedException(); }
        }

        string IDataErrorInfo.this[string columnName]
        {
            get
            {
                List<ValidacniChyba> chyby = new List<ValidacniChyba>();
                switch (columnName)
                {

                    case "JMENO":
                        //Kontrola jmené konvence Oracle
                        string chybaNazev = Validace.JmenaKonvenceOracle(this.JMENO);
                        if (!string.IsNullOrEmpty(chybaNazev))
                        {
                            //return chybaJmena;
                            chyby.Add(new ValidacniChyba(chybaNazev, "EXT_SLOUPCE", this, columnName, this));
                        }
                        //Kontrola existence stejného jména ou ve stejné tabulce
                        if (this.EXT_TABULKY == null) break;
                        if (this.EXT_TABULKY.EXT_SLOUPCE.Any(o => (o.JMENO.ToUpper() == this.JMENO.ToUpper()) && (o != this)))
                        {
                            chyby.Add(new ValidacniChyba(String.Format("Sloupec s názvem {0} je již v tabulce obsažen.", this.JMENO), "EXT_SLOUPCE", this, columnName, this));
                        }
                        break;
                    case "DATA_TYP":
                        if (string.IsNullOrEmpty(this.DATA_TYP))
                        {
                            chyby.Add(new ValidacniChyba("Pole nesmí být prázdné.", "EXT_SLOUPCE", this, columnName, this));
                        }
                        if (!Validace.OracleDataTypes.Any(d => this.DATA_TYP.ToUpper().StartsWith(d.ToUpper())))//(this.DATA_TYP.StartsWith(Validace.OracleDataTypes))
                        {
                            chyby.Add(new ValidacniChyba("Neplatný datový typ", "EXT_SLOUPCE", this, columnName, this));
                        }
                        break;

                    default:
                        break;
                }

                var chybyOld = Validace.ValidacniChyby.Where(v => (v.Objekt == this) && (v.ChybnaProperty == columnName)).ToList();
                Validace.ValidacniChyby.RemoveRange(chybyOld);
                Validace.ValidacniChyby.AddRange(chyby);
                if (chyby.Count > 0)
                {
                    return chyby[0].Chyba;
                }
                else
                    return null;
            }
        }

        #endregion
    }

    public static class Validace
    {
        //private static readonly Validace _instance = new Validace();

        //public static Validace Instance
        //{
        //    get { return _instance; }
        //}



        public static ObservableRangeCollection<ValidacniChyba> ValidacniChyby = new ObservableRangeCollection<ValidacniChyba>();
        //public static List<ValidacniChyba> ValidacniChyby2 { get; set; }


        public static string[] OracleDataTypes = { "char", "nchar", "nvarchar2", "varchar2", "long", "raw", "long raw", "number", "numeric", "float", "dec", "decimal", "integer", "int", "smallint", "real", "double precision", "date", "timestamp", "interval year", "interval day", "bfile", "blob", "clob", "nclob", "rowid", "urowid" };


        /// <summary>
        /// Pole s nepovolenými jmény pro Oracle objekty
        /// </summary>
        public static string[] NepovolenaJmena = { "ACCESS", "ADD", "ALL", "ALTER", "AND", "ANY", "AS", "ASC", "AUDIT", "BETWEEN", "BY", "CHAR", "CHECK", "CLUSTER", "COLUMN", "COMMENT", "COMPRESS", "CONNECT", "CREATE", "CURRENT", "DECIMAL", "DEFAULT", "DELETE", "DESC", "DISTINCT", "DROP", "ELSE", "EXCLUSIVE", "EXISTS", "FILE", "FLOAT", "FOR", "FROM", "GRANT", "GROUP", "HAVING", "IDENTIFIED", "IMMEDIATE", "IN", "INCREMENT", "INDEX", "INITIAL", "INSERT", "INTEGER", "INTERSECT", "INTO", "IS", "LEVEL", "LIKE", "LOCK", "LONG", "MAXEXTENTS", "MINUS", "MLSLABEL", "MODE", "MODIFY", "NOAUDIT", "NOCOMPRESS", "NOT", "NOWAIT", "NULL", "NUMBER", "OF", "OFFLINE", "ON", "ONLINE", "OPTION", "OR", "ORDER", "PCTFREE", "PRIOR", "PRIVILEGES", "PUBLIC", "RAW", "RENAME", "RESOURCE", "REVOKE", "ROW", "ROWID", "ROWNUM", "ROWS", "SELECT", "SESSION", "SET", "SHARE", "SIZE", "SMALLINT", "START", "SUCCESSFUL", "SYNONYM", "SYSDATE", "TABLE", "THEN", "TO", "TRIGGER", "UID", "UNION", "UNIQUE", "UPDATE", "USER", "VALIDATE", "VALUES", "VARCHAR", "VARCHAR2", "VIEW", "WHENEVER", "WHERE", "WITH", "ADMIN", "AFTER", "ALLOCATE", "ANALYZE", "ARCHIVE", "ARCHIVELOG", "AUTHORIZATION", "AVG", "BACKUP", "BECOME", "BEFORE", "BEGIN", "BLOCK", "BODY", "CACHE", "CANCEL", "CASCADE", "CHANGE", "CHARACTER", "CHECKPOINT", "CLOSE", "COBOL", "COMMIT", "COMPILE", "CONSTRAINT", "CONSTRAINTS", "CONTENTS", "CONTINUE", "CONTROLFILE", "COUNT", "CURSOR", "CYCLE", "DATABASE", "DATAFILE", "DBA", "DEC", "DECLARE", "DISABLE", "DISMOUNT", "DOUBLE", "DUMP", "EACH", "ENABLE", "END", "ESCAPE", "EVENTS", "EXCEPT", "EXCEPTIONS", "EXEC", "EXECUTE", "EXPLAIN", "EXTENT", "EXTERNALLY", "FETCH", "FLUSH", "FORCE", "FOREIGN", "FORTRAN", "FOUND", "FREELIST", "FREELISTS", "FUNCTION", "GO", "GOTO", "GROUPS", "INCLUDING", "INDICATOR", "INITRANS", "INSTANCE", "INT", "KEY", "LANGUAGE", "LAYER", "LINK", "LISTS", "LOGFILE", "MANAGE", "MANUAL", "MAX", "MAXDATAFILES", "MAXINSTANCES", "MAXLOGFILES", "MAXLOGHISTORY", "MAXLOGMEMBERS", "MAXTRANS", "MAXVALUE", "MIN", "MINEXTENTS", "MINVALUE", "MODULE", "MOUNT", "NEXT", "NEW", "NOARCHIVELOG", "NOCACHE", "NOCYCLE", "NOMAXVALUE", "NOMINVALUE", "NONE", "NOORDER", "NORESETLOGS", "NORMAL", "NOSORT", "NUMERIC", "OFF", "OLD", "ONLY", "OPEN", "OPTIMAL", "OWN", "PACKAGE", "PARALLEL", "PASCAL", "PCTINCREASE", "PCTUSED", "PLAN", "PLI", "PRECISION", "PRIMARY", "PRIVATE", "PROCEDURE", "PROFILE", "QUOTA", "READ", "REAL", "RECOVER", "REFERENCES", "REFERENCING", "RESETLOGS", "RESTRICTED", "REUSE", "ROLE", "ROLES", "ROLLBACK", "SAVEPOINT", "SCHEMA", "SCN", "SECTION", "SEGMENT", "SEQUENCE", "SHARED", "SNAPSHOT", "SOME", "SORT", "SQLCODE", "SQLERROR", "STATEMENT_ID", "STATISTICS", "STOP", "STORAGE", "SUM", "SWITCH", "SYSTEM", "TABLES", "TABLESPACE", "TEMPORARY", "THREAD", "TIME", "TRACING", "TRANSACTION", "TRIGGERS", "TRUNCATE", "UNDER", "UNLIMITED", "UNTIL", "USE", "USING", "WHEN", "WRITE", "WORK", "ARRAY", "AUTHID", "ASSIGN", "BINARY_INTEGER", "BOOLEAN", "BULK", "CHAR_BASE", "COLLECT", "CONSTANT", "CURRVAL", "DATE", "DAY", "DO", "ELSIF", "EXCEPTION", "EXIT", "EXTENDS", "FALSE", "FORALL", "HEAP", "HOUR", "IF", "INTERFACE", "INTERVAL", "ISOLATION", "JAVA", "LIMITED", "LOOP", "MINUTE", "MOD", "MONTH", "NATURAL", "NATURALN", "NEXTVAL", "NOCOPY", "NUMBER_BASE", "OCIROWID", "OPAQUE", "OPERATOR", "ORGANIZATION", "OTHERS", "OUT", "PARTITION", "PLS_INTEGER", "POSITIVE", "POSITIVEN", "PRAGMA", "RAISE", "RANGE", "RECORD", "REF", "RELEASE", "RETURN", "REVERSE", "ROWLABEL", "ROWTYPE", "SECOND", "SEPARATE", "SPACE", "SQL", "SQLERRM", "STDDEV", "SUBTYPE", "TIMESTAMP", "TRUE", "TYPE", "VARIANCE", "WHILE", "YEAR", "ZONE" };


        public static string[] KlicovaOraJmena = { "CASE","MERGE","JOIN","OVER","PIVOT", "UNPIVOT", "MATCHED" };

        public static string[] FunkceOraJmena = { "ABS", "ACOS", "ADD_MONTHS", "ASCII", "ASCIISTR", "ASIN", "ATAN", "ATAN2", "AVG", "BFILENAME", "BIN_TO_NUM", "BITAND", "CARDINALITY", "CASE", "CAST", "CEIL", "CHARTOROWID", "CHR", "COALESCE", "COMPOSE", "CONCAT ", "CONVERT", "CORR", "COS", "COSH", "COUNT", "COVAR_POP", "COVAR_SAMP", "CUME_DIST", "CURRENT_DATE", "CURRENT_TIMESTAMP", "DBTIMEZONE", "DECODE", "DECOMPOSE", "DENSE_RANK", "DUMP", "EMPTY_BLOB", "EMPTY_CLOB", "EXP", "EXTRACT", "FIRST_VALUE", "FLOOR", "FROM_TZ", "GREATEST", "GROUP_ID", "HEXTORAW", "INITCAP", "INSTR", "INSTR2", "INSTR4", "INSTRB", "INSTRC", "LAG", "LAST_DAY", "LAST_VALUE", "LEAD", "LEAST", "LENGTH", "LENGTH2", "LENGTH4", "LENGTHB", "LENGTHC", "LISTAGG", "LN", "LNNVL", "LOCALTIMESTAMP", "LOG", "LOWER", "LPAD", "LTRIM", "MAX", "MEDIAN", "MIN", "MOD", "MONTHS_BETWEEN", "NANVL", "NCHR", "NEW_TIME", "NEXT_DAY", "NTH_VALUE", "NULLIF", "NUMTODSINTERVAL", "NUMTOYMINTERVAL", "NVL", "NVL2", "POWER", "RANK", "RAWTOHEX", "REGEXP_COUNT", "REGEXP_INSTR", "REGEXP_REPLACE", "REGEXP_SUBSTR", "REMAINDER", "REPLACE", "ROUND", "ROWNUM", "RPAD", "RTRIM", "SESSIONTIMEZONE", "SIGN", "SIN", "SINH", "SOUNDEX", "SQRT", "STDDEV", "SUBSTR", "SUM", "SYS_CONTEXT", "SYSDATE", "SYSTIMESTAMP", "TAN", "TANH", "TO_CHAR", "TO_CLOB", "TO_DATE", "TO_DSINTERVAL", "TO_LOB", "TO_MULTI_BYTE", "TO_NCLOB", "TO_NUMBER", "TO_SINGLE_BYTE", "TO_TIMESTAMP", "TO_TIMESTAMP_TZ", "TO_YMINTERVAL", "TRANSLATE", "TRIM", "TRUNC", "TZ_OFFSET", "UID", "UPPER", "USER", "USERENV", "VAR_POP", "VAR_SAMP", "VARIANCE", "VSIZE" };

        private static string _oraclePLSQLSlova = "(" + string.Join("|", NepovolenaJmena) + "|" + string.Join("|", KlicovaOraJmena) + "|" + string.Join("|", FunkceOraJmena) + ")";
        public static string OraclePLSQLSlova
        {
            get
            {
                return _oraclePLSQLSlova;
            }
        }


        /// <summary>
        /// Kontrola jmené konvence pro Oracle objekty
        /// </summary>
        /// <param name="Jmeno">Kotrolované jméno</param>
        /// <returns>Vrací string s chybou</returns>
        public static string JmenaKonvenceOracle(string Jmeno)
        {
            //string chyba = null;

            if (string.IsNullOrEmpty(Jmeno))
            {
                return "Jmeno musí být vyplněno";
            }

            if (Jmeno.Length > 30)
                return "Jmeno je příliš dlouhé.";
            Regex r = new Regex("^[a-zA-Z0-9_]*$");
            if (!r.IsMatch(Jmeno))
            {
                return "Jmeno smí obsahovat pouze alfanumerické znaky bez diakritiky.";
            }

            if (NepovolenaJmena.Contains(Jmeno.ToUpper()))
            {
                return "Nepovolené jméno pro Oracle.";
            }

            if (Regex.IsMatch(Jmeno, @"^\d"))
            {
                return "Jméno začíná číslicí.";
            }


            return null;
        }

        /// <summary>
        /// Vytvoří prefix pro jméno tabulky z prefixu metodické úlohy + zkrácený formát období, např.: SILC 2015q1 => ZP151
        /// </summary>
        /// <param name="Obdobi">Objetk metodiké období</param>
        /// <returns></returns>
        internal static string PrefixJmenaTabulky(ULOHA_METOD_OBDOBI Obdobi)
        {
            string[] rozdeleneObdobi = Regex.Split(Obdobi.OBDOBI, @"(q)");
            int rok;
            bool isNumeric = int.TryParse(rozdeleneObdobi[0], out rok);
            DateTime dtRok = new DateTime(rok, 1, 1);
            string ZkraceneObdobi = dtRok.ToString("yy");
            ZkraceneObdobi = ZkraceneObdobi + rozdeleneObdobi[2];

            return Obdobi.ULOHA_METOD.PREFIX + ZkraceneObdobi + "_OU";
        }

        /// <summary>
        /// Vytvoří prefix pro jméno tabulky z prefixu metodické úlohy + zkrácený formát období, např.: SILC 2015q1 => ZP151
        /// </summary>
        /// <param name="Obdobi">Objetk teréní období</param>
        /// <returns></returns>
        internal static string PrefixJmenaTabulky(ULOHA_OBDOBI Obdobi)
        {
            string[] rozdeleneObdobi = Regex.Split(Obdobi.OBDOBI, @"(q)");
            int rok;
            bool isNumeric = int.TryParse(rozdeleneObdobi[0], out rok);
            DateTime dtRok = new DateTime(rok, 1, 1);
            string ZkraceneObdobi = dtRok.ToString("yy");
            ZkraceneObdobi = ZkraceneObdobi + rozdeleneObdobi[2];
            return Obdobi.ULOHA.PREFIX_ULOHY + ZkraceneObdobi;
        }

        internal static string KontrolaZdroju(OU_ODVOZ_UKAZ OU)
        {
            foreach (var ZdrojovyOU in OU.OU_ODVOZ_UKAZ1)
            {

                if (ZdrojovyOU.OU_TAB == OU.OU_TAB)//pokud je přidávaný zdroj ze stejné tabulky, kontroluji generaci a pořadí
                {
                    //kontrola generace + pořadí
                    if (OU.GENERACE < ZdrojovyOU.GENERACE)
                    {
                        return ("Generace přidávaného odvozeného ukazatele je vyšší.");
                    }
                    if (OU.GENERACE == ZdrojovyOU.GENERACE)
                    {
                        if (OU.PORADI_VYPOCTU < ZdrojovyOU.PORADI_VYPOCTU)
                        {
                            return ("Pořadí výpočtu přidávaného odvozeného ukazatele je vyšší.");
                        }
                    }
                }
                else if (ZdrojovyOU.OU_TAB.PORADI > OU.OU_TAB.PORADI)//pokud má zdrojová tabulka vyšší pořadí výpočtu
                {
                    //kontrola generace zdroje musí být nišší
                    if (OU.GENERACE < ZdrojovyOU.GENERACE)
                    {
                        return ("Generace přidávaného odvozeného ukazatele je vyšší.");
                    }
                    //kontrola generace zdroje musí být nišší
                    if (OU.GENERACE == ZdrojovyOU.GENERACE)
                    {
                        return ("Pořadí výpočtu tabulky do které patří zdrojový odvozený ukazatele je vyšší než pořadí tabulky upravovaného odvozeného ukazatele. Generaca se nemohou rovnat.");
                    }
                    //throw new System.ComponentModel.DataAnnotations.ValidationException("Pořadí výpočtu tabulky do které patří přidávaný odvozený ukazatele je vyšší než pořadí tabulky upravovaného odvozeného ukazatele.");
                }
                else if (ZdrojovyOU.OU_TAB.PORADI < OU.OU_TAB.PORADI)//pokud má zdrojová tabulka menší pořadí výpočtu
                {
                    //kontrola generace zdroje musí být nišší
                    if (OU.GENERACE < ZdrojovyOU.GENERACE)
                    {
                        return ("Generace přidávaného odvozeného ukazatele je vyšší.");
                    }
                }
            }

            return null;

        }

        internal static string KontrolaVlastnenychOU(OU_ODVOZ_UKAZ ZdrojovyOU)
        {
            foreach (var vlastnikOU in ZdrojovyOU.OU_ODVOZ_UKAZ2)//Projíždím všechny vlastníky (ti kteří mají nastavený OU jako zdroj)
            {

                if (vlastnikOU.OU_TAB == ZdrojovyOU.OU_TAB)//pokud je přidávaný zdroj ze stejné tabulky, kontroluji generaci a pořadí
                {
                    //kontrola generace + pořadí
                    if (ZdrojovyOU.GENERACE > vlastnikOU.GENERACE)
                    {
                        return $"Generace odvozeného ukazatele ({vlastnikOU.NAZEV} z {vlastnikOU.OU_TAB.JMENO}), který je vlastníkem, je nižší než zvolená generace.";
                    }
                    if (ZdrojovyOU.GENERACE == vlastnikOU.GENERACE)
                    {
                        if (ZdrojovyOU.PORADI_VYPOCTU > vlastnikOU.PORADI_VYPOCTU)
                        {
                            return ($"Pořadí výpočtu odvozeného ukazatele ({vlastnikOU.NAZEV} z {vlastnikOU.OU_TAB.JMENO}), který je vlastníkem je nižší než zvolené.");
                        }
                    }
                }
                else if (vlastnikOU.OU_TAB.PORADI < ZdrojovyOU.OU_TAB.PORADI)//pokud má zdrojová tabulka vyšší pořadí výpočtu
                {
                    //kontrola generace zdroje musí být nišší
                    if (ZdrojovyOU.GENERACE > vlastnikOU.GENERACE)
                    {
                        return $"Generace odvozeného ukazatele ({vlastnikOU.NAZEV} z {vlastnikOU.OU_TAB.JMENO}), který je vlastníkem, je nižší než zvolená generace.";
                    }
                    //kontrola generace zdroje musí být nišší
                    if (ZdrojovyOU.GENERACE == vlastnikOU.GENERACE)
                    {
                        return ($"Pořadí výpočtu tabulky do které patří odvozený ukazatele ({vlastnikOU.NAZEV} z {vlastnikOU.OU_TAB.JMENO}), který je vlastníkem upravovaného odvozeného ukazatele, je nižší. Generaca se nemohou rovnat.");
                    }
                    //throw new System.ComponentModel.DataAnnotations.ValidationException("Pořadí výpočtu tabulky do které patří přidávaný odvozený ukazatele je vyšší než pořadí tabulky upravovaného odvozeného ukazatele.");
                }
                else if (vlastnikOU.OU_TAB.PORADI > ZdrojovyOU.OU_TAB.PORADI)//pokud má zdrojová tabulka menší pořadí výpočtu
                {
                    //kontrola generace zdroje musí být nišší
                    if (ZdrojovyOU.GENERACE > vlastnikOU.GENERACE)
                    {
                        return $"Generace odvozeného ukazatele ({vlastnikOU.NAZEV} z {vlastnikOU.OU_TAB.JMENO}), který je vlastníkem, je nižší než zvolená generace.";
                    }
                }
            }

            return null;

        }
    }
}
