﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Data;
using OuDot.Model;
using OuDot.ServicesOudot;
using System.Text.RegularExpressions;
using System.Windows.Controls;
using System.Windows;

namespace OuDot.RozsireniGrafika
{
    class Konvertory
    {
    }

    /// <summary>
    /// Podle druhu překodování vrací na základě parametru hodnotu true/false zda je možné parametr zadávat či nikoliv
    /// </summary>
    public class DruhPrekodovaniToBool : IValueConverter
    {

        #region IValueConverter Members

        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (value == null)//pokud je ComboBoxItem null nemůžu zadávat nic
                return false;

            TypPrekodovani druhPrakodovani = (TypPrekodovani)value;
            //Získám druh překódování
            string param = parameter as string; //získám parametr
            switch (druhPrakodovani)
            {
                case TypPrekodovani.NeRovno:
                case TypPrekodovani.Vetsi:
                case TypPrekodovani.Mensi:
                case TypPrekodovani.Konstanta://Pokud je druh Konstanta mohu zadávat pouze hodnotu Od
                    if (param == "Od")
                        return true;
                    else
                        return false;
                case TypPrekodovani.Interval://Pokud je druh Interval mohu zadávat Od i Do
                    if (param == "Status")
                        return false;
                    else
                        return true;
                case TypPrekodovani.Status://Pokud je druh Status mohu zadávat pouze hodnotu Status
                    if (param == "Status")
                        return true;
                    else
                        return false;
                default:
                    return false;
            }
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }

        #endregion
    }

    /// <summary>
    /// Konvertuje všechny typy překódování do textu
    /// </summary>
    public class PrekodovaniToCategoryConverter : IValueConverter
    {

        #region IValueConverter Members

        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            ObservableCollection<OU_PREKODOVANI> seznamPrekodovani = value as ObservableCollection<OU_PREKODOVANI>;
            if (seznamPrekodovani == null)
                return null;

            string Vystup = "";
            foreach (OU_PREKODOVANI prekod in seznamPrekodovani)
            {
                
                switch ((TypPrekodovani)Enum.Parse(typeof(TypPrekodovani), prekod.ZPUSOB, true))
                {
                    case TypPrekodovani.Vetsi:
                        Vystup = Vystup + "> " + prekod.OD;
                        break;
                    case TypPrekodovani.Mensi:
                        Vystup = Vystup + "< " + prekod.OD;
                        break;
                    case TypPrekodovani.Konstanta:
                        Vystup = Vystup + prekod.OD;
                        break;
                    case TypPrekodovani.Interval:
                        Vystup = Vystup + prekod.OD +" - " + prekod.DO;
                        break;
                    case TypPrekodovani.Status:
                        Vystup = Vystup + (FieldStatus)prekod.STATUS;
                        break;
                    case TypPrekodovani.Null:
                        Vystup = Vystup + TypPrekodovani.Null.ToString();
                        break;
                    case TypPrekodovani.NeNull:
                        Vystup = Vystup + "≠ " + "Null";
                        break;
                    case TypPrekodovani.NeRovno:
                        Vystup = Vystup + "≠ " + prekod.OD;
                        break;
                    default:
                        Vystup = Vystup + "Neznámí typ překódování";
                        break;
                }

                if (seznamPrekodovani.Last() != prekod)//pokud není poslední doplním čárku
                {
                    Vystup = Vystup + "; ";
                }
            }

            return Vystup;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }

        #endregion
    }

    /// <summary>
    /// Konvertuje datový typ odvozeného ukazatele na visibilitu parametru P1 a P2
    /// </summary>
    public class MujDataTypToVisibilityConverter : IValueConverter
    {

        #region IValueConverter Members

        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (value == null)
            {
                return System.Windows.Visibility.Collapsed;
            }
            DatoveTypy novyTyp = (DatoveTypy)value;

            switch (novyTyp)
            {
                case DatoveTypy.text:
                case DatoveTypy.interval_rok:
                    //ukazat P1
                    if (parameter.ToString() == "P1")
                        return System.Windows.Visibility.Visible;
                    else
                        return System.Windows.Visibility.Collapsed;
                case DatoveTypy.číslo:
                case DatoveTypy.interval_den:
                    return System.Windows.Visibility.Visible;
                default:
                    return System.Windows.Visibility.Collapsed;


            }
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }

        #endregion
    }

    /// <summary>
    /// Konvertuje datový typ odvozeného ukazatele na popisek P1 a P2
    /// </summary>
    public class MujDataTypToParameterTextConverter : IValueConverter
    {

        #region IValueConverter Members

        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (value==null)
            {
                return "CHYBA!";
            }
            DatoveTypy novyTyp = (DatoveTypy)value;
            

            switch (novyTyp)
            {
                case DatoveTypy.text:
                    return "Max počet znaků:";
                case DatoveTypy.interval_rok:
                    return "Max číslic v letech:";
                case DatoveTypy.číslo:
                    if (parameter.ToString() == "P1")
                    {
                        return string.Format("Max počet číslic{0}před des. čárkou:",Environment.NewLine);
                    }
                    else if (parameter.ToString() == "P2")
                    {
                        return string.Format("Max počet číslic{0}za des. čárkou:", Environment.NewLine);
                    }
                    return "CHYBA!";
                case DatoveTypy.interval_den:
                    if (parameter.ToString() == "P1")
                    {
                        return "Max číslic ve dnech:";
                    }
                    else if (parameter.ToString() == "P2")
                    {
                        return string.Format("Max číslic zlomků{0}sekund:", Environment.NewLine);
                    }
                    return "CHYBA!";
                default:
                    return "CHYBA!";


            }
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }

        #endregion
    }

    /// <summary>
    /// Konvertuje stringove hodnoty ze dvou či 3 zdrojů do jednoho textoveho sloupce v gridu
    /// </summary>
    public class MyColumnStringConverter : IMultiValueConverter
    {

        #region IMultiValueConverter Members

        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            string navrat = "";
            foreach (object value in values)
            {
                navrat = string.Format("{0}{1}",navrat, value as string);
            }
            //string navrat = string.Format("{0}{1}", values[0] as string, values[1] as string, values[3] as string, values[4] as string);
            if (values.Count() == 3 && values[2] as string != null)
            {
                navrat=navrat + values[2] as string;
            }

            return navrat;
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }

        #endregion
    }

    /// <summary>
    /// Získá view kolekce (ObservableCollection), přidá sortiong a sortuje dle parametru + nastaví na Live sorting (při přidání nového prvku sortuje bez refreshe)
    /// </summary>
    public class MySortByNameCollectionConverter : IValueConverter
    {

        #region IValueConverter Members

        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            IList collection = value as IList;
            ICollectionView view = System.Windows.Data.CollectionViewSource.GetDefaultView(collection);
            SortDescription sort = new SortDescription(parameter.ToString(), ListSortDirection.Ascending);
            view.SortDescriptions.Add(sort);

            ICollectionViewLiveShaping viewLV = view as ICollectionViewLiveShaping;
            if (viewLV.CanChangeLiveSorting)
            {
                viewLV.IsLiveSorting = true;
            }

            return viewLV;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }

        #endregion
    }


    [ValueConversion(typeof(bool), typeof(bool))]
    public class InverseBooleanConverter : IValueConverter
    {
        #region IValueConverter Members

        public object Convert(object value, Type targetType, object parameter,
            System.Globalization.CultureInfo culture)
        {
            if (targetType != typeof(bool))
                throw new InvalidOperationException("The target must be a boolean");

            return !(bool)value;
        }

        public object ConvertBack(object value, Type targetType, object parameter,
            System.Globalization.CultureInfo culture)
        {
            throw new NotSupportedException();
        }

        #endregion
    }

    /// <summary>
    /// Konvertuje enumerátor DatoveTypy na int a naopak
    /// </summary>
    [ValueConversion(typeof(DatoveTypy), typeof(int))]
    public class EnumToIntConverter : IValueConverter
    {
        #region IValueConverter Members

        public object Convert(object value, Type targetType, object parameter,
            System.Globalization.CultureInfo culture)
        {
            
            if (value == null) return 0;

            return (DatoveTypy)value;
            
        }

        public object ConvertBack(object value, Type targetType, object parameter,
            System.Globalization.CultureInfo culture)
        {
            if (value == null) return 0;

            return (int)value;
        }
        
        #endregion
    }


    /// <summary>
    /// Konvertuje enumerátor Spoj na string a naopak
    /// </summary>
    [ValueConversion(typeof(SpojEnum), typeof(int))]
    public class SpojEnumToStringConverter : IValueConverter
    {
        #region IValueConverter Members

        public object Convert(object value, Type targetType, object parameter,
            System.Globalization.CultureInfo culture)
        {

            if (value == null) return null;

            return (SpojEnum)Enum.Parse(typeof(SpojEnum), value.ToString(), true); 

        }

        public object ConvertBack(object value, Type targetType, object parameter,
            System.Globalization.CultureInfo culture)
        {
            if (value == null) return null;

            return value.ToString();
        }

        #endregion
    }

    /// <summary>
    /// Konvertuje téma na nejnadrazenější téma
    /// </summary>
    public class OUTemaNadrazeneToStringConverter : IMultiValueConverter
    {
       
        #region IMultiValueConverter Members

        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            OU_TEMATA tema = values[0] as OU_TEMATA;
            ObservableCollection<OU_TEMATA> temata = values[1] as ObservableCollection<OU_TEMATA>;

            string temaString = "";
            if (tema != null)
            {
                //Vyhledání nadřazené kategorie:
                string kodNad = tema.KOD.Split('.').FirstOrDefault();
                var hlTema = temata.Where(t => t.KOD == kodNad).FirstOrDefault();

                temaString = hlTema.POPISEK;
            }

            return temaString;
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }

        #endregion
    }

    /// <summary>
    /// Konvertuje 2 hodnoty: 1.Odvozený Ukazatel, 2.List OU. Vrací index z Listu podle daného OU o jedna vyšší převedený na string
    /// </summary>
    public class ListIndexToIntConverter : IMultiValueConverter
    {
        public object Convert(object[] values, Type targetType, object parameter, CultureInfo culture)
        {
            OU_ODVOZ_UKAZ ou = values[0] as OU_ODVOZ_UKAZ;
            List<OU_ODVOZ_UKAZ> ListOU = values[1] as List<OU_ODVOZ_UKAZ>;

            return (ListOU.IndexOf(ou) + 1).ToString();
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    /// <summary>
    /// Konvertuje string na bold, pokud string není prázdný
    /// </summary>
    [ValueConversion(typeof(string), typeof(FontWeight))]
    public class StringToFontWeightConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return (string.IsNullOrEmpty((value as string)?.Trim())) ? FontWeights.Normal : FontWeights.Bold;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    /// <summary>
    /// Konvertuje kolekci na bold, pokud obsahuje itemy
    /// </summary>
    [ValueConversion(typeof(ICollection), typeof(FontWeight))]
    public class EnumerableToFontWeightConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return ((value as ICollection)?.Count > 0) ? FontWeights.Bold : FontWeights.Normal;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    /// <summary>
    /// Kovertuje bool na sedivou nebo bílou barvu, true = bíla, false = šedivá
    /// </summary>
    [ValueConversion(typeof(bool),typeof(System.Windows.Media.SolidColorBrush))]
    public class PovolitZmenuToGrayColor : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return (bool)value ? new System.Windows.Media.SolidColorBrush(System.Windows.Media.Colors.White) : new System.Windows.Media.SolidColorBrush(System.Windows.Media.Colors.LightGray);
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
