﻿using OuDot.Model;
using OuDot.ViewModel;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using Xceed.Wpf.AvalonDock;
using Xceed.Wpf.AvalonDock.Layout;

namespace OuDot.RozsireniGrafika
{
    
    public static class Rozsireni
    {
        /// <summary>
        /// Otevře nové OU okno
        /// </summary>
        /// <param name="OdvozUkaz">Odvozený ukazetel, který se  v okně zobrazí</param>
        /// <param name="DockMan">Hlavní DockingManager, který spravuje všechna používaná okna</param>
        internal static void OtevriOUOkno(OU_ODVOZ_UKAZ OdvozUkaz, DockingManager DockMan)
        {
           
            //Vyhledání zda okno s ou již existuje
            var vsechnyLD = DockMan.Layout.Descendents().OfType<LayoutDocument>();
            LayoutContent OknoVybOU = vsechnyLD.FirstOrDefault(p => (p.Content.GetType() == typeof(OdvozUkazDetailsWindow)) && ((p.Content as OdvozUkazDetailsWindow).DataContext as OdvozUkazDetailsViewModel).OdvozUkaz == OdvozUkaz);

            if (OknoVybOU == null)//Vytvářím nové okno
            {
                //Vytvořím a nastavím okno pro detaili tabulky
                OdvozUkazDetailsWindow oudw = new OdvozUkazDetailsWindow(OdvozUkaz);
                //(oudw.DataContext as OdvozUkazDetailsViewModel).OdvozUkaz = OdvozUkaz;
                //Vložím vytvořené okno jako LeyoutDocument a s aktivním ho
                LayoutDocument ld = new LayoutDocument
                {
                    Content = oudw
                };

                //Nabinduju titul okna s nazvem ou
                Binding titulBinding = new Binding();
                titulBinding.Source = oudw.DataContext;
                titulBinding.Path = new PropertyPath("OdvozUkaz.NAZEV");
                BindingOperations.SetBinding(ld, LayoutDocument.TitleProperty, titulBinding);

                //Nabinduji IsBusy property z hlavního okna:
                MainWindow mw = ((MainWindow)System.Windows.Application.Current.MainWindow);
                Binding isBusy = new Binding("IsBusy");
                isBusy.Source = mw.DataContext;
                isBusy.Converter = new RozsireniGrafika.InverseBooleanConverter();
                BindingOperations.SetBinding(oudw, TabulkaDetailsWindow.IsEnabledProperty, isBusy);

                //Vyhledání LayoutDocumentPane
                LayoutDocumentPane LDP = DockMan.Layout.Descendents().OfType<LayoutDocumentPane>().FirstOrDefault();
                LDP.Children.Add(ld);
                ld.IsActive = true;
            }
            else//aktivuji již existující
            {
                OknoVybOU.IsActive = true;
            }
        }

        /// <summary>
        /// Otevře okno s metadaty z CAPIdotu
        /// </summary>
        /// <param name="Sloupec">Vybraný zdroj z BDBConverteru</param>
        /// <param name="DockMan">Hlavní DockingManager, který spravuje všechna používaná okna</param>
        internal static void ZobrazCapiMetaData(BDBC_SLOUPCE Sloupec, DockingManager DockMan)
        {

            if (Sloupec != null)//(BdbcZdroj.GetType().BaseType.Name == "BDBC_SLOUPCE")
            {
                ULOHA_OBDOBI obdobiMetaData = Sloupec.BDBCPRIRAZENE_FIELDY.BDBCTABULKY.ULOHA_OBDOBI;
                string variable = Sloupec.BDBCPRIRAZENE_FIELDY.NAZEV_FIELDU;

                BDBC_POPISY metaData = Sloupec.BDBCPRIRAZENE_FIELDY.BDBCTABULKY.ULOHA_OBDOBI.BDBC_POPISY.Where(p => p.VARIABLE?.ToUpper() == variable.ToUpper()).FirstOrDefault();

                if (metaData == null)
                {
                    MessageBox.Show("Meta datanebyla nalezena.", "Meta data", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }

                var vsechnyLD = DockMan.Layout.Descendents().OfType<LayoutDocument>();

                //Vyhledání zda okno s tabulkou již existuje
                //LayoutContent OknoVybOU = LDP.Children.FirstOrDefault(p => p.Title == odvozUkaz.NAZEV);
                LayoutContent OknoMetaData = vsechnyLD.FirstOrDefault(p => (p.Content.GetType() == typeof(MetaDataWindow)) && ((p.Content as MetaDataWindow).DataContext as MetaDataViewModel).MetaData == metaData);

                if (OknoMetaData == null)//Vytvářím nové okno
                {
                    //Vytvořím a nastavím okno pro detaili tabulky
                    MetaDataWindow mdw = new MetaDataWindow();
                    (mdw.DataContext as MetaDataViewModel).MetaData = metaData;
                    //Vložím vytvořené okno jako LeyoutDocument a s aktivním ho
                    LayoutDocument ld = new LayoutDocument { Content = mdw, Title = Sloupec.JMENO };

                    LayoutDocumentPane LDP = DockMan.Layout.Descendents().OfType<LayoutDocumentPane>().FirstOrDefault();
                    LDP.Children.Add(ld);
                    ld.IsActive = true;
                }
                else//aktivuji již existující
                {
                    OknoMetaData.IsActive = true;
                }
                
            }
        }
    }

    public class CustomDataGrid : DataGrid
    {

        public CustomDataGrid()
        {
            this.SelectionChanged += CustomDataGrid_SelectionChanged;
        }

        void CustomDataGrid_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            this.SelectedItemsList = this.SelectedItems;
        }
        #region SelectedItemsList

        public IList SelectedItemsList
        {
            get { return (IList)GetValue(SelectedItemsListProperty); }
            set { SetValue(SelectedItemsListProperty, value); }
        }

        public static readonly DependencyProperty SelectedItemsListProperty =
                DependencyProperty.Register("SelectedItemsList", typeof(IList), typeof(CustomDataGrid), new PropertyMetadata(null));

        #endregion
    }

    public static class ComboUtil
    {
        private static readonly CommandBinding DeleteCommandBinding = new CommandBinding(ApplicationCommands.Delete, HandleExecuteDeleteCommand);

        private static void HandleExecuteDeleteCommand(object sender, ExecutedRoutedEventArgs e)
        {
            var combo = e.Source as ComboBox;
            if (combo != null)
                combo.SelectedIndex = -1;
        }

        #region AllowNull Property

        public static bool GetAllowNull(ComboBox combo)
        {
            if (combo == null)
                throw new ArgumentNullException("combo");

            return (bool)combo.GetValue(AllowNullProperty);
        }

        public static void SetAllowNull(ComboBox combo, bool value)
        {
            if (combo == null)
                throw new ArgumentNullException("combo");

            combo.SetValue(AllowNullProperty, value);
        }

        public static readonly DependencyProperty AllowNullProperty =
            DependencyProperty.RegisterAttached(
                "AllowNull",
                typeof(bool),
                typeof(ComboUtil),
                new UIPropertyMetadata(HandleAllowNullPropertyChanged));

        private static void HandleAllowNullPropertyChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
            var combo = (ComboBox)o;
            if (true.Equals(e.NewValue))
            {
                if (!combo.CommandBindings.Contains(DeleteCommandBinding))
                    combo.CommandBindings.Add(DeleteCommandBinding);
            }
            else
            {
                combo.CommandBindings.Remove(DeleteCommandBinding);
            }
        }
        #endregion
    }
}
