﻿using OuDot.Model;
using OuDot.ServicesOudot;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Threading;

namespace OuDot.RozsireniGrafika
{
    public class SyntaxHighlightRichTextBox : Xceed.Wpf.Toolkit.RichTextBox
    {
        
        private DispatcherTimer _tmrFormat = new DispatcherTimer();// na událost tick spouštím formátování kódu

        /// <summary>
        /// flag k oznaceni spusteneho formatovani
        /// </summary>
        private bool beziFormatovani = false;

        /// <summary>
        /// Struktura slozici k ulozeni inlineu a pozice jeho začátku v indexu paragraphu
        /// </summary>
        struct ParagraphStruktura
        {
            public Run Inline;
            public int indexStart;
        }

        /// <summary>
        /// Slouží k definování textového řetězce, který budu formátovat
        /// </summary>
        struct TagFormat
        {
            public TextRange Text;
            public FormatPLSQLEnum DruhFormatu;
            public string PlainText;
        }

        /// <summary>
        /// Konstruktor
        /// </summary>
        public SyntaxHighlightRichTextBox()
        {
            //Registrace události vložení do RichTextBoxu
            DataObject.AddPastingHandler(this, new DataObjectPastingEventHandler(SyntaxHighlightRichTextBox_Pasting));
            //naplnění timeru na formátování
            _tmrFormat.Tick += _tmrFormat_Tick;
            _tmrFormat.Interval = new TimeSpan(0, 0, 1);
        }

        /// <summary>
        /// Řeší problé odstranění veškerého formátování při vkládání textu
        /// https://social.msdn.microsoft.com/Forums/vstudio/en-US/0d672c70-d49d-4ebf-871d-420cc164f7d8/c-wpf-richtextbox-remove-formatting-and-line-spaces?forum=wpf
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SyntaxHighlightRichTextBox_Pasting(object sender, DataObjectPastingEventArgs e)
        {
            e.FormatToApply = DataFormats.Text;
        }



        /// <summary>
        /// Na událost TextChanged
        /// </summary>
        /// <param name="e"></param>
        protected override void OnTextChanged(TextChangedEventArgs e)
        {
            //Pokud je spuštěno při zvýrazňování testu událost upouštím
            if (beziFormatovani) return;
            
            base.OnTextChanged(e);//provedena událost
            
            //Zastavím timer a spouští odpočet odznovu
            _tmrFormat.Stop();
            _tmrFormat.Start();
        }

        #region PLSQL highlighting
        /// <summary>
        /// Spouští zvýraznění textu sekundu po jeho změně
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void _tmrFormat_Tick(object sender, EventArgs e)
        {
            _tmrFormat.Stop();//timer zastavím
            beziFormatovani = true;

            //http://blogs.microsoft.co.il/tamir/2006/12/14/richtextbox-syntax-highlighting/
            //Převzato z návodu výše
            if (this.Document == null)
                return;

            TextRange documentRange = new TextRange(this.Document.ContentStart, this.Document.ContentEnd);
            //http://stackoverflow.com/questions/12065616/how-to-select-richtextbox-text-given-an-index-and-length
            this.BeginChange();//Združí všechny provedené změny do jednoho undo kroku, dokud není spouštěna funkce EndChange

            //Nejdríve odstraním veškerý původní formát
            documentRange.ClearAllProperties();
            //documentRange.ApplyPropertyValue(TextElement.FontFamilyProperty, new FontFamily("Segoe UI"));
            //documentRange.ApplyPropertyValue(TextElement.FontSizeProperty, 12.00);
            

            ZvyrazneniSyntaxePLSQL();//Nastavení zvýraznění syntaxe
            this.EndChange();//Ukončím sdružování změň

            beziFormatovani = false;
        }

        /// <summary>
        /// Zvýraznění syntaxe PLSQL
        /// </summary>
        private void ZvyrazneniSyntaxePLSQL()
        {
            List<TagFormat> tagy = new List<TagFormat>();//List všech formátovaných částí textu
            TextPointer startGlobKoment = null;//Začátek globálního komentáře
            foreach (Paragraph p in this.Document.Blocks)
            {

                // Run r = (Run)navigator.Parent;
                TextRange trParagraph = new TextRange(p.ContentStart, p.ContentEnd);

                List<ParagraphStruktura> inlines = new List<ParagraphStruktura>();
                var sb = new StringBuilder();
                foreach (Run r in p.Inlines)
                {
                    inlines.Add(new ParagraphStruktura { Inline = r, indexStart = sb.Length });
                    sb.Append(r.Text);
                }

                //Vyhledání v textu v RUN veskerych shod textu obsaženého v <>
                var vsechnyBold = Regex.Matches(trParagraph.Text, "<(.*?)>");
                foreach (Match bold in vsechnyBold)
                {
                    VytvorPridejTagNaRadku(tagy, inlines, bold, FormatPLSQLEnum.Bold);
                }

                //Formátovní pro všechna Oracle slova
                //http://phoboslab.org/log/2007/08/generic-syntax-highlighting-with-regular-expressions
                var vsechnyShody = Regex.Matches(trParagraph.Text, "(?:(?<!\\w|\\$|\\%|\\@|>)" + Validace.OraclePLSQLSlova + "(?!\\w))", RegexOptions.IgnoreCase);//Vyhledání všech shod se slovem v daném textu
                foreach (Match shoda in vsechnyShody)
                {
                    VytvorPridejTagNaRadku(tagy, inlines, shoda, FormatPLSQLEnum.Modre);
                }

                //Formátování číslic
                var cisla = Regex.Matches(trParagraph.Text, "(?:(?<!\\w|\\$|\\%|\\@)-*\\d+(?!\\w))", RegexOptions.IgnoreCase);
                foreach (Match c in cisla)
                {
                    VytvorPridejTagNaRadku(tagy, inlines, c, FormatPLSQLEnum.Oranz);
                }

                //Formátování textu v jednoduchých uvozovkách //bez uvozovek (?<![^']|^)(.*?)(?![^']|$) (nefunguje pro dalsi uvozovky v rade)
                var shodyUvozovky = Regex.Matches(trParagraph.Text, "'(.*?)'", RegexOptions.IgnoreCase);
                foreach (Match su in shodyUvozovky)
                {
                    VytvorPridejTagNaRadku(tagy, inlines, su, FormatPLSQLEnum.SvetleModr);
                }

                //zakomentované radky DODĚLAT PŘI PROCHÁZENÍ CELÉHO TEXTU KOMENTÁŘ PRO 
                var shodyKomenty = Regex.Matches(trParagraph.Text, "--", RegexOptions.IgnoreCase);
                foreach (Match su in shodyKomenty)
                {
                    //Vyhledám ve struktůře pragarafu začátek máho Tagu
                    ParagraphStruktura startStruc = inlines.Where(i => su.Index >= i.indexStart).OrderByDescending(i => i.indexStart).First();
                    TextPointer start = startStruc.Inline.ContentStart.GetPositionAtOffset(su.Index - startStruc.indexStart, LogicalDirection.Forward);

                    TextRange tr = new TextRange(start, p.ContentEnd);//začátek řádku nebo pokud je null konec dokumentu
                    tagy.Add(new TagFormat { Text = tr, DruhFormatu = FormatPLSQLEnum.Zelene, PlainText = su.Value });

                }

                //Globální komentář /*
                //1.Vyhledání gobálních komentářů na jednom řádku
                var shodyGlobKomentNaRadku = Regex.Matches(trParagraph.Text, "/\\*(.*?)\\*/", RegexOptions.IgnoreCase);
                foreach (Match sgknr in shodyGlobKomentNaRadku)
                {
                    VytvorPridejTagNaRadku(tagy, inlines, sgknr, FormatPLSQLEnum.Zelene);
                }
                //2.Označení začátku a konce gobálního komentáře přes více řádků
                var zacaGlobKoment = Regex.Match(trParagraph.Text, @"\/\*((?!\*\/).)*$");//Vyhledání na řádku počátečního globálního komentáře, který není na řádku ukončen
                var konecGlobKoment = Regex.Match(trParagraph.Text, "\\*/");//Vyhledání 1. koncové značky globálního komentáře
                //Pokud existuje TextPointer na začátek globálního komentáře a zároveň byla nalezena ukončovací zančka
                if (startGlobKoment != null && konecGlobKoment.Success)
                {
                    //Dohledám TextPointer na ukončovací značku a vytvořím Tag
                    ParagraphStruktura startStruc = inlines.Where(i => i.indexStart <= konecGlobKoment.Index).OrderByDescending(i => i.indexStart).First();
                    TextPointer end = startStruc.Inline.ContentStart.GetPositionAtOffset(konecGlobKoment.Index + konecGlobKoment.Length - startStruc.indexStart, LogicalDirection.Backward);
                    TextRange tr = new TextRange(startGlobKoment, end);//začátek řádku nebo pokud je null konec dokumentu
                    tagy.Add(new TagFormat { Text = tr, DruhFormatu = FormatPLSQLEnum.Zelene, PlainText = tr.Text });
                    //Nastavím TextPointer začátku globálního komentáře na prázdnou hodnotu
                    startGlobKoment = null;
                }
                //Pokud neexistuje TextPointer na začátek globálního komentáře a byl na řádku nalezen neukončený, tak ho vyhledám a vložím do proměnné
                if (startGlobKoment == null && zacaGlobKoment.Success)
                {
                    ParagraphStruktura startStruc = inlines.Where(i => i.indexStart <= zacaGlobKoment.Index).OrderByDescending(i => i.indexStart).First();
                    startGlobKoment = startStruc.Inline.ContentStart.GetPositionAtOffset(zacaGlobKoment.Index - startStruc.indexStart, LogicalDirection.Forward);
                }


            }
            //Po projetí celého textu zjistím zda exituje TextPointer na začátek globálního komentáře a pokud ano vytvořím Tag do konce celého textu
            if (startGlobKoment != null)
            {
                TextRange tr = new TextRange(startGlobKoment, startGlobKoment.DocumentEnd);
                tagy.Add(new TagFormat { Text = tr, DruhFormatu = FormatPLSQLEnum.Zelene, PlainText = tr.Text });
                startGlobKoment = null;
            }

            //Nastavení formátů
            Format(tagy);
        }

        /// <summary>
        /// Vytvoří podle daného Tagu TextRange a přidá záznam do daného listu Tagů
        /// </summary>
        /// <param name="Tagy">List všech tagů k formátu</param>
        /// <param name="InLines">všechny Inliny Paragrafu ze kterého je Tag</param>
        /// <param name="Tag">Text, který budu formátovat</param>
        /// <param name="TypFormatu">Druh formátu</param>
        private void VytvorPridejTagNaRadku(List<TagFormat> Tagy, List<ParagraphStruktura> InLines, Match Tag, FormatPLSQLEnum TypFormatu)
        {
            //Nejdříve vždy ve struktuře paragraphu je nutné vybrat správný Run (inline) na kterém náš Tag začíná a na kterém končí.
            //Poté na daném Run (inline) vyhledám spravný TextPointer začátku a konce a z toho vytvořím TextRange
            //Vyhledám ve struktůře pragarafu začátek máho Tagu
            ParagraphStruktura startStruc = InLines.Where(i => Tag.Index >= i.indexStart).OrderByDescending(i => i.indexStart).First();
            TextPointer start = startStruc.Inline.ContentStart.GetPositionAtOffset(Tag.Index - startStruc.indexStart, LogicalDirection.Forward);
            //Vyhládám konec Tagu
            ParagraphStruktura x = InLines.Where(i => (Tag.Index + Tag.Value.Length) >= i.indexStart).OrderByDescending(i => i.indexStart).First();
            TextPointer end = x.Inline.ContentStart.GetPositionAtOffset(Tag.Index + Tag.Value.Length - x.indexStart, LogicalDirection.Backward);
            Tagy.Add(new TagFormat { Text = new TextRange(start, end), DruhFormatu = TypFormatu, PlainText = Tag.Value });
        }

        /// <summary>
        /// Nastaví formát dle definic v daném listu
        /// </summary>
        /// <param name="TagList">List tagů k naformátování</param>
        void Format(List<TagFormat> TagList)
        {
            //Odeberu událost na změnu textu a po té nastavuji všechny formáty, které podle typu formátu aplikuji na dané TextRange
            TagList = TagList.OrderBy(t => t.DruhFormatu.ToString()).ToList();
            foreach (TagFormat tf in TagList)
            {
                switch (tf.DruhFormatu)
                {
                    case FormatPLSQLEnum.Bold:
                        tf.Text.ApplyPropertyValue(TextElement.FontWeightProperty, FontWeights.Bold);
                        break;
                    case FormatPLSQLEnum.Modre:
                        tf.Text.ApplyPropertyValue(TextElement.ForegroundProperty, new SolidColorBrush(Colors.Blue));
                        break;
                    case FormatPLSQLEnum.Zelene:
                        tf.Text.ApplyPropertyValue(TextElement.ForegroundProperty, new SolidColorBrush(Colors.Green));
                        break;
                    case FormatPLSQLEnum.Oranz:
                        tf.Text.ApplyPropertyValue(TextElement.ForegroundProperty, new SolidColorBrush(Colors.Chocolate));
                        break;
                    case FormatPLSQLEnum.SvetleModr:
                        tf.Text.ApplyPropertyValue(TextElement.ForegroundProperty, new SolidColorBrush(Colors.BlueViolet));
                        break;
                    default:
                        throw new NotImplementedException("Neznámí druh textového formátu.");
                }

            }
        }
        #endregion PLSQL highlighting

        protected override void OnPreviewKeyDown(System.Windows.Input.KeyEventArgs e)
        {
            base.OnPreviewKeyDown(e);
            //Oprava při stisku  Ctrl + z pkud je to možné prouvedou se dva kroky na zpátek a to zdůvodu, že jeden krok zpět je sdružené zvýraznění textu
            if (e.Key == Key.Z && Keyboard.Modifiers == ModifierKeys.Control)
            {
                if (this.CanUndo)
                    this.Undo();
            }

            
        }

        protected override void OnPreviewKeyUp(System.Windows.Input.KeyEventArgs e)
        {
            if (e.Key == Key.Tab)
            {
                //Oprava chování RichTextBoxu při vkládání tabulátoru na začátek řádku, standardně RichTextBox vkládá pouze indention (odsazení) což se neukládá do textu. Proto ruším nastavení TextIndent a vkládám na pozici tabulátor
                if (this.CaretPosition.IsAtLineStartPosition)
                {
                    this.CaretPosition.InsertTextInRun("\t");
                    this.CaretPosition.GetLineStartPosition(0).Paragraph.TextIndent = 0;
                }

            }

            base.OnPreviewKeyUp(e);
        }
    }
}
