﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using Oracle.ManagedDataAccess.Client;
using OuDot.Model;
using OuDot.Properties;
using OuDot.ViewModel;
using System.Data.Entity;

namespace OuDot.ServicesOudot
{
    /// <summary>
    /// Singleton
    /// </summary>
    public sealed class DBContext
    {
        private static readonly DBContext _instance = new DBContext();
        /// <summary>
        /// Konstruktor
        /// </summary>
        private DBContext()
        {
            Entity = new Entities();
            var objsm = ((System.Data.Entity.Infrastructure.IObjectContextAdapter)Entity).ObjectContext.ObjectStateManager;
            objsm.ObjectStateManagerChanged += objsm_ObjectStateManagerChanged;

           
            OUdotConn = VytvorServisniPripojeni();
        }

        

        /// <summary>
        /// Vrací svojí vlastní instanci
        /// </summary>
        public static DBContext Instance
        {
            get
            {
                return _instance;
            }
        }
        /// <summary>
        /// Entity z modelu z Ora DB
        /// </summary>
        public Entities Entity { get; set; }
        /// <summary>
        /// Servisní OUdot připojení k Oracle databázy (slouží např. k vytváření tabulek)
        /// </summary>
        public OracleConnection OUdotConn { get; set; }

        /// <summary>
        /// Připojení do Ora DB
        /// </summary>
        /// <param name="Pass"></param>
        /// <param name="PrihlasJm"></param>
        /// <returns></returns>
        internal bool PripojitKdb(string Pass, string PrihlasJm)
        {
            try
            {
                OracleConnectionStringBuilder MyConnectionString = new OracleConnectionStringBuilder(@"DATA SOURCE=APOLO;PERSIST SECURITY INFO=True");
                MyConnectionString.UserID = PrihlasJm;
                MyConnectionString.Password = Pass;
                MyConnectionString.ConnectionTimeout = 30;
                Entity.Database.Connection.ConnectionString = MyConnectionString.ToString();
                Entity.Database.Connection.Open();
                return true;


            }
            catch (Exception ex)
            {
                string zprava = ex.Message;
                if (ex.InnerException != null)
                    zprava = ex.InnerException.Message;
                MessageBox.Show(zprava, "Chyba Přihlášení do DB", MessageBoxButton.OK, MessageBoxImage.Error);
                return false;
            }

        }

        /// <summary>
        /// Událost změna statusu Entity v DBcontextu
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void objsm_ObjectStateManagerChanged(object sender, System.ComponentModel.CollectionChangeEventArgs e)
        {
            try
            {
                if (e.Action == System.ComponentModel.CollectionChangeAction.Remove)
                {
                    var chybnaEntyta = Validace.ValidacniChyby.Where(v => v.Objekt == e.Element).ToList();
                    Validace.ValidacniChyby.RemoveRange(chybnaEntyta);
                }
            }
            catch (Exception ex)
            {
                string zprava = string.Format("Chyba při odebrání entity z listu chyb.", Environment.NewLine, ex.Message);
                MessageBox.Show(zprava, "DB Context", MessageBoxButton.OK, MessageBoxImage.Error);
            }

        }

        /// <summary>
        /// Vytvoří servisní OUdot připojení
        /// </summary>
        /// <returns></returns>
        private OracleConnection VytvorServisniPripojeni()
        {
            try
            {
                //Načtení z mého logovacího souboru
                //Krypt.Heslo h = new Krypt.Heslo();
                //byte[] k = Encoding.Unicode.GetBytes(h.NactiHeslo(@"d:\Dokumenty\Visual Studio 2013\Projects\OuDot\k.csdb"));
                byte[] k = Encoding.Unicode.GetBytes(Settings.Default.key);
                //byte[] iv = Encoding.Unicode.GetBytes(h.NactiHeslo(@"d:\Dokumenty\Visual Studio 2013\Projects\OuDot\iv.csdb"));
                byte[] iv = Encoding.Unicode.GetBytes(Settings.Default.iv);
                Krypt.VerejnaSifra vs = new Krypt.VerejnaSifra();
                string MyConnectionString = vs.NactiSifrovanyText(System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "ServiceCS2.csdb"), k, iv);//h.NactiHeslo(System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "ServiceCS2.csdb"));
                vs.Dispose();//h.Dispose();

                OracleConnection con = new OracleConnection(MyConnectionString);
                //con.Open();
                MyConnectionString = null;
                return con;
            }
            catch (Exception ex)
            {

                throw new Exception("Chyba načtení servisního připojení", ex);
            }

        }



        /// <summary>
        /// http://stackoverflow.com/questions/16437083/dbcontext-discard-changes-without-disposing
        /// </summary>
        public void RejectChanges()
        {
            foreach (var entry in Instance.Entity.ChangeTracker.Entries())
            {
                switch (entry.State)
                {
                    case EntityState.Modified:
                    case EntityState.Deleted:
                        entry.State = EntityState.Modified; //Revert changes made to deleted entity.
                        entry.State = EntityState.Unchanged;
                        break;
                    case EntityState.Added:
                        entry.State = EntityState.Detached;
                        break;
                }
            }
        }


    }
}
