﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Markup;

namespace OuDot.ServicesOudot
{
    /// <summary>
    /// Typ parametrů pro generované funkce
    /// </summary>
    public enum ParamFceEnum
    {
        konstanta,
        ignorovatSloupec
    }

    /// <summary>
    /// Spoj OU
    /// </summary>
    public enum SpojEnum
    {
        česká,
        evropská,
        společná
    }

    /// <summary>
    /// Statusy fieldů z Blaisu
    /// </summary>
    public enum FieldStatus 
    { 
        Response = 0, 
        Unprocessed = 1,
        DontKnow = 2,
        Refusal = 3 
    }

    /// <summary>
    /// Typy zdrojů, které může odovozený ukazatel používat
    /// </summary>
    public enum TypZdroj
    {
        BDBC_SLOUPCE,
        OU_ODVOZ_UKAZ,
        BDBC_PRIMARNI_KLICE,
        EXT_SLOUPCE,
        CISEL_SEZNAM
    };

    /// <summary>
    /// Datový typ odvozeného ukazatele
    /// </summary>
    [TypeConverter(typeof(EnumDescriptionTypeConverter))]
    public enum DatoveTypy
    {
        [Description("VYBER")]
        NotSet = 0,
        text = 1,
        číslo = 2,
        kategorie = 3,
        čas = 4,
        [Description("datum + čas")]
        datumAčas = 5,
        číselník = 6,
        klasifikace = 7,
        [Description("interval den")]
        interval_den = 8,
        [Description("interval rok")]
        interval_rok = 9,
        boolean = 10,
        [Description("plovoucí des. čárka")]
        PlovCarka = 11
    }

    /// <summary>
    /// Možné druhy překódování pro definovanou kategorii
    /// </summary>
    [TypeConverter(typeof(EnumDescriptionTypeConverter))]
    public enum TypPrekodovani
    {
        [Description("VYBER")]
        NotSet = 0,
        Konstanta = 1,
        Interval = 2,
        Status = 3,
        [Description("Větší než")]
        Vetsi = 4,
        [Description("Menší než")]
        Mensi = 5,
        [Description("Nerovná se")]
        NeRovno = 7,
        [Description("Prázdná hodnota")]
        Null = 6,
        [Description("Ne Prázdná hodnota")]
        NeNull = 8
    }

    /// <summary>
    /// Typy formátů PL/SQL kódu
    /// </summary>
    public enum FormatPLSQLEnum
    {
        Bold,
        Modre,
        Zelene,
        Oranz,
        SvetleModr
    }

    #region Rozšíření enumerátorů pro grafiku

    public class EnumBindingSourceExtension : MarkupExtension
    {
        private Type _enumType;
        public Type EnumType
        {
            get { return this._enumType; }
            set
            {
                if (value != this._enumType)
                {
                    if (null != value)
                    {
                        Type enumType = Nullable.GetUnderlyingType(value) ?? value;

                        if (!enumType.IsEnum)
                            throw new ArgumentException("Type must be for an Enum.");
                    }

                    this._enumType = value;
                }
            }
        }

        public EnumBindingSourceExtension() { }

        public EnumBindingSourceExtension(Type enumType)
        {
            this.EnumType = enumType;
        }

        public override object ProvideValue(IServiceProvider serviceProvider)
        {
            if (null == this._enumType)
                throw new InvalidOperationException("The EnumType must be specified.");

            Type actualEnumType = Nullable.GetUnderlyingType(this._enumType) ?? this._enumType;
            Array enumValues = Enum.GetValues(actualEnumType);

            if (actualEnumType == this._enumType)
                return enumValues;

            Array tempArray = Array.CreateInstance(actualEnumType, enumValues.Length + 1);
            enumValues.CopyTo(tempArray, 1);
            return tempArray;
        }
    }


    public class EnumDescriptionTypeConverter : EnumConverter
    {
        public EnumDescriptionTypeConverter(Type type)
            : base(type)
        {
        }

        public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
        {
            if (destinationType == typeof(string))
            {
                if (value != null)
                {
                    FieldInfo fi = value.GetType().GetField(value.ToString());
                    if (fi != null)
                    {
                        var attributes = (DescriptionAttribute[])fi.GetCustomAttributes(typeof(DescriptionAttribute), false);
                        return ((attributes.Length > 0) && (!String.IsNullOrEmpty(attributes[0].Description))) ? attributes[0].Description : value.ToString();
                    }
                }

                return string.Empty;
            }

            return base.ConvertTo(context, culture, value, destinationType);
        }
    }
    #endregion Rozšíření enumerátorů pro grafiku
}
