﻿using OuDot.Model;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OuDot.ServicesOudot
{
    public static class ExtensionMethods
    {
        public static int Remove<T>(
            this ObservableCollection<T> coll, Func<T, bool> condition)
        {
            var itemsToRemove = coll.Where(condition).ToList();

            foreach (var itemToRemove in itemsToRemove)
            {
                coll.Remove(itemToRemove);
            }

            return itemsToRemove.Count;
        }

        /// <summary>
        /// Při použítí v kolekci pro klasifikace v metodě sort, seřadí klasifikace podle hierarchie.
        /// Zdroj: https://social.msdn.microsoft.com/Forums/silverlight/en-US/f1fcfc50-2907-4d00-9193-52adf43f046e/sorting-numbers-such-as-111-112-1110-etc?forum=silverlightstart
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <returns></returns>
        internal static int RazeniKlasifikace(string x, string y)
        {
            int ret = 0;
            var xsplit = x.Split(".".ToCharArray()).Select(z => int.Parse(z.Trim())).ToList();
            var ysplit = y.Split(".".ToCharArray()).Select(z => int.Parse(z.Trim())).ToList();
            for (int i = 0; i < Math.Max(xsplit.Count, ysplit.Count); i++)
            {

                if (xsplit.Count - 1 < i)
                {
                    ret = -1;
                    return ret;
                }
                else if (ysplit.Count - 1 < i)
                {
                    ret = 1;
                    return ret;
                }
                else
                {
                    ret = xsplit[i] - ysplit[i];
                    if (ret != 0)
                        return ret;
                }
            }
            return ret;
        }
    }

    /// <summary>
    /// Seřadí odvozené ukazatele podle kódů přiřazených témat a pokud se kód u OU shoduje řadí OU dále podle jejich názvů
    /// //http://stackoverflow.com/questions/24147585/collectionviewsource-with-custom-sort
    /// </summary>
    public class SortOUpodleTema : IComparer
    {
        public int Compare(object x, object y)
        {
            string kodX = (x as OU_ODVOZ_UKAZ)?.OU_TEMATA?.KOD;
            string kodY = (y as OU_ODVOZ_UKAZ)?.OU_TEMATA?.KOD;
            if (kodX == null)
            {
                kodX = "0";
            }
            if (kodY == null)
            {
                kodY = "0";
            }
            int ret = 0;
            var xsplit = kodX.Split(".".ToCharArray()).Select(z => int.Parse(z.Trim())).ToList();
            var ysplit = kodY.Split(".".ToCharArray()).Select(z => int.Parse(z.Trim())).ToList();
            for (int i = 0; i < Math.Max(xsplit.Count, ysplit.Count); i++)
            {

                if (xsplit.Count - 1 < i)
                {
                    ret = -1;
                    return ret;
                }
                else if (ysplit.Count - 1 < i)
                {
                    ret = 1;
                    return ret;
                }
                else
                {
                    ret = xsplit[i] - ysplit[i];
                    if (ret != 0)
                        return ret;
                }
            }
            return string.Compare((x as OU_ODVOZ_UKAZ).NAZEV, (y as OU_ODVOZ_UKAZ).NAZEV);
            //return ret;
        }
    }


}
