﻿using System;
using System.Collections.Generic;
using System.Data.Entity.Core.Objects;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using OuDot.Model;

namespace OuDot.ServicesOudot
{
    public static class GenerovaniSQL
    {
        ///// <summary>
        ///// Pokud je true, uživatel nezadává hodnoty např. pro where nebo konstantu, ale je vyhledána v původním kódu. Slouží pro přegenerování všech SQL kódů.
        ///// </summary>
        //internal static bool AutoPregenerovaniSQL = false;

        /// <summary>
        /// Vygeneruje sql kód podle typu funkce a přiřazených zdrojů
        /// </summary>
        /// <param name="OdvozUkaz">Odvozený ukazatel pro který generuje sql kód</param>
        /// <param name="Typ">funkce pro kterou chci sql kód vygenerovat</param>
        /// <returns>sql kód</returns>
        public static string GenerujSQL(OU_ODVOZ_UKAZ OdvozUkaz, OU_FUNKCE Typ)
        {
            switch (Typ.ID)
            {
                case 1: // Vlastní funkce
                    return "1";
                case 2: //Prosté prevzetí
                    return ProstePrevzetiPodleZdroje(OdvozUkaz);
                case 3: //Nadřízené převzetí
                    return NadrizenePrevzetiPodleZdroje(OdvozUkaz);
                case 4: //Podřízené převzetí
                    return PodrizenePrevzetiPodleZdroje(OdvozUkaz);
                case 5: //Konstanta
                    return GenerujKonstanta(OdvozUkaz);
                default:
                    throw new Exception("Nedefinovaná funkce.");
            }
        }

        /// <summary>
        /// Vygeneruje sql pro prosté převzetí podle typu přiřazeného zdroje
        /// </summary>
        /// <param name="OdvozUkaz">Odvozený ukazatel pro který generuje sql kód</param>
        /// <returns>sql kód</returns>
        private static string ProstePrevzetiPodleZdroje(OU_ODVOZ_UKAZ OdvozUkaz)
        {
            switch (ZjistiTypZdroje(OdvozUkaz, 1))//zjistím typ zdroje pro který se vygeneruje sql
            {
                case TypZdroj.BDBC_SLOUPCE:
                    string tabPuv = OdvozUkaz.BDBC_SLOUPCE[0].BDBCPRIRAZENE_FIELDY.BDBCTABULKY.JMENO; //Jméno tabulky z bdbc;
                    if (OdvozUkaz.BDBC_SLOUPCE[0].BDBCPRIRAZENE_FIELDY.BDBCTABULKY.BDBC_SLUC1 != null) //Pokud se tabulka slucovala nastavím jméno tabulky na jmeno sloučení
                        tabPuv = OdvozUkaz.BDBC_SLOUPCE[0].BDBCPRIRAZENE_FIELDY.BDBCTABULKY.BDBC_SLUC1.JMENO;

                    return GenerujProstePrevzeti(OdvozUkaz, OdvozUkaz.BDBC_SLOUPCE[0].JMENO, tabPuv + "_D", ZjistiJmenaPKZdroje(OdvozUkaz, TypZdroj.BDBC_SLOUPCE));
                case TypZdroj.OU_ODVOZ_UKAZ:
                    if (OdvozUkaz.OU_CATEGORY.SelectMany(c => c.OU_PREKODOVANI).Any(p => p.STATUS != null))
                    {
                        throw new NotImplementedException("Nelze generovat překódování STATUSU pro zdroj typu odvozeného ukazatele.");
                    }
                    return GenerujProstePrevzeti(OdvozUkaz, OdvozUkaz.OU_ODVOZ_UKAZ1[0].NAZEV, OdvozUkaz.OU_ODVOZ_UKAZ1[0].OU_TAB.JMENO, ZjistiJmenaPKZdroje(OdvozUkaz, TypZdroj.OU_ODVOZ_UKAZ));
                case TypZdroj.BDBC_PRIMARNI_KLICE:
                    if (OdvozUkaz.OU_CATEGORY.SelectMany(c => c.OU_PREKODOVANI).Any(p => p.STATUS != null))
                    {
                        throw new NotImplementedException("Nelze generovat překódování STATUSU pro zdroj, který je součást primárního klíče (typu BDBC_PRIMARNI_KLICE).");
                    }
                    return GenerujProstePrevzeti(OdvozUkaz, OdvozUkaz.BDBC_PRIMARNI_KLICE[0].JMENO, OdvozUkaz.BDBC_PRIMARNI_KLICE[0].BDBCTABULKY.JMENO + "_D", ZjistiJmenaPKZdroje(OdvozUkaz, TypZdroj.BDBC_PRIMARNI_KLICE));
                case TypZdroj.EXT_SLOUPCE:
                    if (OdvozUkaz.OU_CATEGORY.SelectMany(c => c.OU_PREKODOVANI).Any(p => p.STATUS != null))
                    {
                        throw new NotImplementedException("Nelze generovat překódování STATUSU pro Externí zdroj.");
                    }
                    return GenerujProstePrevzeti(OdvozUkaz, OdvozUkaz.EXT_SLOUPCE[0].JMENO, OdvozUkaz.EXT_SLOUPCE[0].EXT_TABULKY.JMENO_ID, ZjistiJmenaPKZdroje(OdvozUkaz, TypZdroj.EXT_SLOUPCE));
                default:
                    throw new NotImplementedException("Pro daný typ zdroje nelze generovat sql.");
            }
        }

        /// <summary>
        /// Vygeneruje sql pro převzetí z nadřízené tabulky podle typu přiřazeného zdroje
        /// </summary>
        /// <param name="OdvozUkaz">Odvozený ukazatel pro který generuje sql kód</param>
        /// <returns>sql kód</returns>
        private static string NadrizenePrevzetiPodleZdroje(OU_ODVOZ_UKAZ OdvozUkaz)
        {
            switch (ZjistiTypZdroje(OdvozUkaz, 1))//zjistím typ zdroje pro který se vygeneruje sql
            {
                case TypZdroj.BDBC_SLOUPCE:
                    string tabPuv = OdvozUkaz.BDBC_SLOUPCE[0].BDBCPRIRAZENE_FIELDY.BDBCTABULKY.JMENO; //Jméno tabulky s bdbc daty;
                    if (OdvozUkaz.BDBC_SLOUPCE[0].BDBCPRIRAZENE_FIELDY.BDBCTABULKY.BDBC_SLUC1 != null) //Pokud se tabulka slucovala nastavím jméno tabulky s daty na jmeno sloučení
                        tabPuv = OdvozUkaz.BDBC_SLOUPCE[0].BDBCPRIRAZENE_FIELDY.BDBCTABULKY.BDBC_SLUC1.JMENO;

                    return GenerujPrevzetiZNadrizene(OdvozUkaz, OdvozUkaz.BDBC_SLOUPCE[0].JMENO, tabPuv + "_D", ZjistiJmenaPKZdroje(OdvozUkaz, TypZdroj.BDBC_SLOUPCE));
                case TypZdroj.BDBC_PRIMARNI_KLICE:
                    if (OdvozUkaz.OU_CATEGORY.SelectMany(c => c.OU_PREKODOVANI).Any(p => p.STATUS != null))
                    {
                        throw new NotImplementedException("Nelze generovat překódování STATUSU pro zdroj, který je součást primárního klíče (typu BDBC_PRIMARNI_KLICE).");
                    }
                    return GenerujPrevzetiZNadrizene(OdvozUkaz, OdvozUkaz.BDBC_PRIMARNI_KLICE[0].JMENO, OdvozUkaz.BDBC_PRIMARNI_KLICE[0].BDBCTABULKY.JMENO + "_D", ZjistiJmenaPKZdroje(OdvozUkaz, TypZdroj.BDBC_PRIMARNI_KLICE));
                case TypZdroj.EXT_SLOUPCE:
                    if (OdvozUkaz.OU_CATEGORY.SelectMany(c => c.OU_PREKODOVANI).Any(p => p.STATUS != null))
                    {
                        throw new NotImplementedException("Nelze generovat překódování STATUSU pro Externí zdroj.");
                    }
                    return GenerujPrevzetiZNadrizene(OdvozUkaz, OdvozUkaz.EXT_SLOUPCE[0].JMENO, OdvozUkaz.EXT_SLOUPCE[0].EXT_TABULKY.JMENO_ID, ZjistiJmenaPKZdroje(OdvozUkaz, TypZdroj.EXT_SLOUPCE));
                case TypZdroj.OU_ODVOZ_UKAZ:
                    if (OdvozUkaz.OU_CATEGORY.SelectMany(c => c.OU_PREKODOVANI).Any(p => p.STATUS != null))
                    {
                        throw new InvalidOperationException("Nelze generovat překódování STATUSU pro zdroj typu Odvozeného ukazatele.");
                    }
                    return GenerujPrevzetiZNadrizene(OdvozUkaz, OdvozUkaz.OU_ODVOZ_UKAZ1[0].NAZEV, OdvozUkaz.OU_ODVOZ_UKAZ1[0].OU_TAB.JMENO, ZjistiJmenaPKZdroje(OdvozUkaz, TypZdroj.OU_ODVOZ_UKAZ));
                default:
                    throw new NotImplementedException("Pro daný typ zdroje nelze generovat sql.");
            }
        }

        /// <summary>
        /// Vygeneruje sql pro převzetí z podřízené tabulky podle typu přiřazeného zdroje
        /// </summary>
        /// <param name="OdvozUkaz">Odvozený ukazatel pro který generuje sql kód</param>
        /// <returns>sql kód</returns>
        private static string PodrizenePrevzetiPodleZdroje(OU_ODVOZ_UKAZ OdvozUkaz)
        {
            TypZdroj typZdroje = ZjistiTypZdroje(OdvozUkaz, 1);
            switch (typZdroje)//zjistím typ zdroje pro který se vygeneruje sql
            {
                case TypZdroj.BDBC_SLOUPCE:
                    string tabPuv = OdvozUkaz.BDBC_SLOUPCE[0].BDBCPRIRAZENE_FIELDY.BDBCTABULKY.JMENO; //Jméno tabulky s bdbc daty;
                    if (OdvozUkaz.BDBC_SLOUPCE[0].BDBCPRIRAZENE_FIELDY.BDBCTABULKY.BDBC_SLUC1 != null) //Pokud se tabulka slucovala nastavím jméno tabulky s daty na jmeno sloučení
                        tabPuv = OdvozUkaz.BDBC_SLOUPCE[0].BDBCPRIRAZENE_FIELDY.BDBCTABULKY.BDBC_SLUC1.JMENO;

                    return GenerujPrevzetiZPodrizene(OdvozUkaz, OdvozUkaz.BDBC_SLOUPCE[0].JMENO, tabPuv + "_D", ZjistiJmenaPKZdroje(OdvozUkaz, typZdroje));
                case TypZdroj.BDBC_PRIMARNI_KLICE:
                    if (OdvozUkaz.OU_CATEGORY.SelectMany(c => c.OU_PREKODOVANI).Any(p => p.STATUS != null))
                    {
                        throw new NotImplementedException("Nelze generovat překódování STATUSU pro zdroj, který je součást primárního klíče (typu BDBC_PRIMARNI_KLICE).");
                    }
                    return GenerujPrevzetiZPodrizene(OdvozUkaz, OdvozUkaz.BDBC_PRIMARNI_KLICE[0].JMENO, OdvozUkaz.BDBC_PRIMARNI_KLICE[0].BDBCTABULKY.JMENO + "_D", ZjistiJmenaPKZdroje(OdvozUkaz, typZdroje));
                case TypZdroj.OU_ODVOZ_UKAZ:
                    throw new NotImplementedException();
                default:
                    throw new Exception("Pro daný typ zdroje nelze generovat sql.");
            }
        }

        /// <summary>
        /// Zjistí podle počtu zdrojů jaké zdroje má odvozený ukazatel nadefinované
        /// </summary>
        /// <param name="OdvozUkaz">Odvozený ukazatel pro který generuje sql kód</param>
        /// <param name="PocetZdroju">počet povolených zdrojů</param>
        /// <returns>typ definovaných zdrojů</returns>
        internal static TypZdroj ZjistiTypZdroje(OU_ODVOZ_UKAZ OdvozUkaz, int PocetZdroju)
        {
            if (OdvozUkaz.SoucetVsechZdroju != PocetZdroju)//Zda odpovídá součet všech definovaných zdrojů odvozeného ukazatele povolenému počtu pro generovanou funkci
            {
                throw new Exception("Počet zdrojů neodpovídá.");
            }

            //Slovník každý typ má přiřazený svůj definovaný počet zdrojů
            Dictionary<TypZdroj, int> slovnikZdrojePocet = new Dictionary<TypZdroj, int> 
            {
                {TypZdroj.BDBC_SLOUPCE, OdvozUkaz.BDBC_SLOUPCE.Count},
                {TypZdroj.OU_ODVOZ_UKAZ, OdvozUkaz.OU_ODVOZ_UKAZ1.Count},
                {TypZdroj.BDBC_PRIMARNI_KLICE, OdvozUkaz.BDBC_PRIMARNI_KLICE.Count},
                {TypZdroj.EXT_SLOUPCE, OdvozUkaz.EXT_SLOUPCE.Count},
                {TypZdroj.CISEL_SEZNAM, OdvozUkaz.CISEL_SEZNAM.Count}

            };

            //dotaz, který typ odpovídá povolenému počtu
            var vysledek = slovnikZdrojePocet.Where(z => z.Value == PocetZdroju).FirstOrDefault();

            //Pokud žádný typ neodpovídá hledanému počtu zdrojů
            if (vysledek.Equals(default(KeyValuePair<TypZdroj, int>)))//(vysledek == null)
            {
                throw new Exception("Chybně definované zdroje.");
            }

            return vysledek.Key;
        }

        /// <summary>
        /// Podle typu zdroje vrátí všechna jména primárních klíčů prvního nadefinovaného zdroje
        /// </summary>
        /// <param name="OdvozUkaz">Odvozený ukazatel daného zdroje</param>
        /// <param name="TypZdroje">Typ zdroje pro který chci zjistit jejich jména</param>
        /// <returns>List se jmény primárních klíčů</returns>
        public static List<string> ZjistiJmenaPKZdroje(OU_ODVOZ_UKAZ OdvozUkaz, TypZdroj TypZdroje)
        {
            switch (TypZdroje)
            {
                case TypZdroj.BDBC_SLOUPCE:
                    return OdvozUkaz.BDBC_SLOUPCE[0].BDBCPRIRAZENE_FIELDY.BDBCTABULKY.BDBC_PRIMARNI_KLICE.Select(p => p.JMENO).ToList();
                case TypZdroj.OU_ODVOZ_UKAZ:
                    return OdvozUkaz.OU_ODVOZ_UKAZ1[0].OU_TAB.OU_PRIMARNI_KLICE.Select(p => p.JMENO).ToList();
                case TypZdroj.BDBC_PRIMARNI_KLICE:
                    return OdvozUkaz.BDBC_PRIMARNI_KLICE[0].BDBCTABULKY.BDBC_PRIMARNI_KLICE.Select(p => p.JMENO).ToList();
                case TypZdroj.EXT_SLOUPCE:
                    return OdvozUkaz.EXT_SLOUPCE[0].EXT_TABULKY.EXT_SLOUPCE.Where(e=>e.PRIMARNI_KLIC).Select(p => p.JMENO).ToList();
                default:
                    throw new Exception("Pro daný typ zdroje nelze najít jeho primární klíče.");
            }
        }

        #region Generování SQL

        /// <summary>
        /// Vygeneruje sql kod pro typ BDBC_SLOUPCE, prevzetí z podřízené tabulky. Nutné zadat zadat konstantu pro chybějící primární klíč.
        /// </summary>
        /// <returns>sql kód</returns>
        private static string GenerujPrevzetiZPodrizene(OU_ODVOZ_UKAZ OdvozUkaz, string PuvodniSloupec, string PuvodniTab, List<string> PrimarniKliceZdroje)
        {
            


            //zdrojova tabulka musí mít o jeden primární klíč navíc
            if (PrimarniKliceZdroje.Count != OdvozUkaz.OU_TAB.OU_PRIMARNI_KLICE.Count + 1)
                throw new Exception("Zdroj musí mít o jeden primární klíč navíc než je nastaveno pro tabulku.");

            
            string bdbcPrimKlice = "";//primární klíče za zdrojové tabulky odděléné čárkami (HD, IDOKRES, IDPAGINA,)
            string propojeniPK = "";//propojení primárních klíčů zdrojové tabulky s výstupní oudotovou (O.HD = B.HD AND O.IDOKRES = B.IDOKRES AND O.IDPAGINA = B.IDPAGINA)
            string ouDotPrimKlice = "";//primární klíče ouDotové tabulky (O.IDOKRES, O.IDPAGINA, O.HD,)
            string hodnotyB = "";//sloupce do kterých se vkládají hodnoty z ouDotPrimKlice (O.IDOKRES, O.IDPAGINA, O.HD,)
            string konstantniPK = "";//název sloupce, kterému je nutné přiřadit konstantní hodnotu

            //Kontrola zda jsou všechny sloupce primárního klíče tabulky OU obsaženy v primárním klíči zdrojové tabulky
            foreach (var sl in OdvozUkaz.OU_TAB.OU_PRIMARNI_KLICE)
            {
                if (!PrimarniKliceZdroje.Any(o => o.ToUpper() == sl.JMENO.ToUpper()))
                {
                    string zprava = string.Format("Nebyl nalezen odpovídající primární klíč v tabulce zdroje.{0}{0}Sloupec {1} neexistuje v tabulce zdroje {2}.", Environment.NewLine, sl.JMENO, PuvodniTab);
                    throw new Exception(zprava);
                }
            }

            foreach (string pkBDBC in PrimarniKliceZdroje)
            {

                OU_PRIMARNI_KLICE pkOU = OdvozUkaz.OU_TAB.OU_PRIMARNI_KLICE.Where(o => o.JMENO.ToUpper() == pkBDBC.ToUpper()).FirstOrDefault();//podle názvu vyberu odpovídající primární klíč z ouDot tabulky


                bdbcPrimKlice = string.Format("{0}{1}, ", bdbcPrimKlice, pkBDBC);
                if (pkOU != null)//byl nalezen odpovídajíc primární klíč v ouDot tabulce
                {
                    hodnotyB = string.Format("{0}B.{1}, ", hodnotyB, pkBDBC);
                    propojeniPK = string.Format("{2}O.{0} = B.{1} AND ", pkOU.JMENO, pkBDBC, propojeniPK);
                    ouDotPrimKlice = string.Format("{0}O.{1}, ", ouDotPrimKlice, pkOU.JMENO);
                }
                else//klíč nebyl nalezen, je to název sloupce, kterému musíme přiřadit konstantu
                {
                    konstantniPK = pkBDBC;
                }
            }

            propojeniPK = Regex.Replace(propojeniPK, " AND $", "");//Odstraní poslední AND

            string klausuleWhere = "";
            //Pokud NENÍ nastaven parametr na ignorování přebívajícího sloupce PK zdroje, nastavím klausuli where 
            if (!OdvozUkaz.OU_PARAM_FCE.Any(p => p.TYP_PARAM == ParamFceEnum.ignorovatSloupec.ToString()))
            {
                string konstantaWhere = OdvozUkaz.OU_PARAM_FCE.Where(p => p.TYP_PARAM == ParamFceEnum.konstanta.ToString()).FirstOrDefault().HODNOTA;
                klausuleWhere = $"where {konstantniPK}= '{konstantaWhere}'";
            }

            
            //Zjistí konstantu pro klausuli where, pokud je nastaveno AutoPřegenerovaniSQL a nalezne shodu ve vyhledávacím paternu v dříve nastaveném SQL kódu, převezmu tuto hodnotu z původního kódu. Jinak se vždy dotážu uživatele na její hodnotu.
            //if (AutoPregenerovaniSQL && Regex.IsMatch(OdvozUkaz.SQL_DOTAZ, "where " + konstantniPK + "='\\d+'"))
            //{
            //    //Vyhledám konstantu v původním kódu
            //    konstantaWhere = Regex.Match(OdvozUkaz.SQL_DOTAZ, "where " + konstantniPK + "='\\d+'").Value;
            //    konstantaWhere = Regex.Match(konstantaWhere, "\\d+").Value;

            //}
            //else
            //{
            //    //Zobrazím uživateli okno pro zadání konstanty
            //    OuDot.ViewModel.PomocnaOkna.NastavHodnotuWindow nhw = new ViewModel.PomocnaOkna.NastavHodnotuWindow();
            //    nhw.Title = OdvozUkaz.NAZEV;
            //    nhw.ShowDialog();
            //    konstantaWhere = nhw.tbKonstanta.Text;
            //}


            string sloupecStatus = "";
            string joinStatus = "";
            string vkladanaHodnota = "B." + PuvodniSloupec;
            //Pokud bylo u Odvozeného ukazatele nastaveno nějaké překódování, vygeneruji pro něj příslušné doplňující SQL kódy:
            if (OdvozUkaz.OU_CATEGORY.SelectMany(c => c.OU_PREKODOVANI).Any())
            {
                string[] doplneniPrekodovani = GenerovaniPrekodovani(OdvozUkaz, PuvodniTab, PuvodniSloupec, bdbcPrimKlice);
                sloupecStatus = doplneniPrekodovani[0];
                joinStatus = doplneniPrekodovani[1];
                vkladanaHodnota = doplneniPrekodovani[2];
            }

            if (PrimarniKliceZdroje.Contains(PuvodniSloupec))//Pokud je zdrojovy sloupec primárním klíčem tabulky => odstraním poslední čárku v bdbcPrimKlice a PuvodniSloupec nastavym na null
            {
                bdbcPrimKlice = Regex.Replace(bdbcPrimKlice, ", $", "");
                PuvodniSloupec = null;
            }
            else//Pokud zdroj není součástí PK doplním před sloupec D. => pochází z tabulek z daty
            {
                PuvodniSloupec = "D." + PuvodniSloupec;
            }

            //Vytvoření finálního sql
            string sql = string.Format(
@"MERGE INTO {0} O
USING (SELECT {1}{2}{9} FROM {3} D{10} {8}) B
ON ({4})
WHEN MATCHED THEN UPDATE SET O.{5} = {11}" 
//WHEN NOT MATCHED THEN INSERT ({6}O.{5})
//VALUES ({7}{12})"
,
                                        OdvozUkaz.OU_TAB.JMENO, //0
                                        bdbcPrimKlice, //1
                                        PuvodniSloupec, //2
                                        PuvodniTab, //3
                                        propojeniPK, //4
                                        OdvozUkaz.NAZEV, //5
                                        ouDotPrimKlice, //6
                                        hodnotyB, //7
                                        klausuleWhere, //8
                                        sloupecStatus, //9
                                        joinStatus, //10
                                        vkladanaHodnota //11
                                        );
            return sql;
        }

        /// <summary>
        /// Vygeneruje sql kod pro typ BDBC_SLOUPCE, prevzetí z nadřízené tabulky.
        /// </summary>
        /// <returns>sql kód</returns>
        private static string GenerujPrevzetiZNadrizene(OU_ODVOZ_UKAZ OdvozUkaz, string PuvodniSloupec, string PuvodniTab, List<string> PrimarniKliceZdroje)
        {

            //zdrojova tabulka musí mít o jeden primární klíč méně
            if (PrimarniKliceZdroje.Count >= OdvozUkaz.OU_TAB.OU_PRIMARNI_KLICE.Count)
                throw new Exception("Zdroj musí mít menší počet sloupců primárního klíče než je nastaveno pro tabulku.");

            
            string bdbcPrimKlice = "";//primární klíče za zdrojové tabulky odděléné čárkami (HD, IDOKRES, IDPAGINA,)
            string propojeniPK = "";//propojení primárních klíčů zdrojové tabulky s výstupní oudotovou (O.HD = B.HD AND O.IDOKRES = B.IDOKRES AND O.IDPAGINA = B.IDPAGINA)
            string ouDotPrimKlice = "";//primární klíče ouDotové tabulky (O.IDOKRES, O.IDPAGINA, O.HD,)
            string hodnotyB = "";//sloupce do kterých se vkládají hodnoty z ouDotPrimKlice (O.IDOKRES, O.IDPAGINA, O.HD,)

           
            //Kontrola zda jsou všechny zdrojové sloupce primárního klíče obsaženy v primárním klíči tabulky OU
            foreach (var sl in PrimarniKliceZdroje)
            {
                if (!OdvozUkaz.OU_TAB.OU_PRIMARNI_KLICE.Any(o => o.JMENO.ToUpper() == sl.ToUpper()))
                {
                    string zprava = string.Format("Nebyl nalezen odpovídající primární klíč v tabulce zdroje.{0}{0}Sloupec {1} neexistuje v tabulce zdroje {2}.", Environment.NewLine, sl, OdvozUkaz.OU_TAB.JMENO);
                    throw new Exception(zprava);
                }
            }

            foreach (OU_PRIMARNI_KLICE pkOU in OdvozUkaz.OU_TAB.OU_PRIMARNI_KLICE)
            {
                ouDotPrimKlice = string.Format("{0}O.{1}, ", ouDotPrimKlice, pkOU.JMENO);

                string pkBDBC = PrimarniKliceZdroje.Where(b => b.ToUpper() == pkOU.JMENO.ToUpper()).FirstOrDefault();//podle názvu vyberu odpovídající primární klíč ze zdrojové tabulky


                if (pkBDBC != null)//pokud jsem našel zdrojový primární klíč
                {
                    bdbcPrimKlice = string.Format("{0}{1}, ", bdbcPrimKlice, pkBDBC);

                    propojeniPK = string.Format("{2}O.{0} = B.{1} AND ", pkOU.JMENO, pkBDBC, propojeniPK);

                    hodnotyB = string.Format("{0}B.{1}, ", hodnotyB, pkBDBC);
                }
                else//klíč nebyl nalezen
                {
                    hodnotyB = string.Format("{0}1, ", hodnotyB);//vložena 1 do primárního při nalezení chybějících hodnot v tabulce, protože nemůže být null
                }
            }

            propojeniPK = Regex.Replace(propojeniPK, " AND $", "");//Odstraní poslední AND

            string sloupecStatus = "";
            string joinStatus = "";
            string vkladanaHodnota = "B." + PuvodniSloupec;
            //Pokud bylo u Odvozeného ukazatele nastaveno nějaké překódování, vygeneruji pro něj příslušné doplňující SQL kódy:
            if (OdvozUkaz.OU_CATEGORY.SelectMany(c => c.OU_PREKODOVANI).Any())
            {
                string[] doplneniPrekodovani = GenerovaniPrekodovani(OdvozUkaz, PuvodniTab, PuvodniSloupec, bdbcPrimKlice);
                sloupecStatus = doplneniPrekodovani[0];
                joinStatus = doplneniPrekodovani[1];
                vkladanaHodnota = doplneniPrekodovani[2];
            }

            if (PrimarniKliceZdroje.Contains(PuvodniSloupec))//Pokud je zdrojovy sloupec primárním klíčem tabulky => odstraním poslední čárku v bdbcPrimKlice a PuvodniSloupec nastavym na null
            {
                bdbcPrimKlice = Regex.Replace(bdbcPrimKlice, ", $", "");
                PuvodniSloupec = null;
            }
            else//Pokud zdroj není součástí PK doplním před sloupec D. => pochází z tabulek z daty
            {
                PuvodniSloupec = "D." + PuvodniSloupec;
            }

            //Vytvoření finálního sql
            string sql = string.Format(
@"MERGE INTO {0} O
USING (SELECT {1}{2}{8} FROM {3} D{9}) B
ON ({4})
WHEN MATCHED THEN UPDATE SET O.{5} = {10}",
//WHEN NOT MATCHED THEN INSERT ({6}O.{5})
//VALUES ({7}{10})",
                                        OdvozUkaz.OU_TAB.JMENO, //0
                                        bdbcPrimKlice, //1
                                        PuvodniSloupec, //2
                                        PuvodniTab, //3
                                        propojeniPK, //4
                                        OdvozUkaz.NAZEV, //5
                                        ouDotPrimKlice, //6
                                        hodnotyB, //7
                                        sloupecStatus, //8
                                        joinStatus, //9
                                        vkladanaHodnota //10
                                        );
            return sql;
        }

        private static string GenerujProstePrevzeti(OU_ODVOZ_UKAZ OdvozUkaz, string PuvodniSloupec, string PuvodniTab, List<string> PrimarniKliceZdroje)
        {
            //počet primárních klíčů musí být stejný
            if (PrimarniKliceZdroje.Count != OdvozUkaz.OU_TAB.OU_PRIMARNI_KLICE.Count)
                throw new Exception("Zdroj musí mít shodné primární klíče s tabulkou odvozeného ukazatele.");



            string bdbcPrimKlice = "";//primární klíče za zdrojové tabulky odděléné čárkami (HD, IDOKRES, IDPAGINA,)
            string propojeniPK = "";//propojení primárních klíčů zdrojové tabulky s výstupní oudotovou (O.HD = B.HD AND O.IDOKRES = B.IDOKRES AND O.IDPAGINA = B.IDPAGINA)
            string ouDotPrimKlice = "";//primární klíče ouDotové tabulky (O.IDOKRES, O.IDPAGINA, O.HD,)
            string hodnotyB = "";//sloupce do kterých se vkládají hodnoty z ouDotPrimKlice (O.IDOKRES, O.IDPAGINA, O.HD,)


            foreach (OU_PRIMARNI_KLICE pkOU in OdvozUkaz.OU_TAB.OU_PRIMARNI_KLICE)
            {
                string pkBDBC = PrimarniKliceZdroje.Where(b => b.ToUpper() == pkOU.JMENO.ToUpper()).FirstOrDefault();//podle názvu vyberu odpovídající primární klíč ze zdrojové tabulky
                if (pkBDBC == null)//Kontrola zda byl nalezen sloupec primárního klíče v tabulce zdroje
                {
                    string zprava = string.Format("Nebyl nalezen odpovídající primární klíč v tabulce zdroje.{0}{0}Sloupec {1} neexistuje v tabulce zdroje {2}.", Environment.NewLine, pkOU.JMENO, PuvodniTab);
                    throw new Exception(zprava);
                }


                ouDotPrimKlice = string.Format("{0}O.{1}, ", ouDotPrimKlice, pkOU.JMENO);
                bdbcPrimKlice = string.Format("{0}{1}, ", bdbcPrimKlice, pkBDBC);

                propojeniPK = string.Format("{2}O.{0} = B.{1} AND ", pkOU.JMENO, pkBDBC, propojeniPK);

                hodnotyB = string.Format("{0}B.{1}, ", hodnotyB, pkBDBC);
            }

            propojeniPK = Regex.Replace(propojeniPK, " AND $", "");//Odstraní poslední AND

            string sloupecStatus = "";
            string joinStatus = "";
            string vkladanaHodnota = "B." + PuvodniSloupec;

            //Pokud bylo u Odvozeného ukazatele nastaveno nějaké překódování, vygeneruji pro něj příslušné doplňující SQL kódy:
            if (OdvozUkaz.OU_CATEGORY.SelectMany(c => c.OU_PREKODOVANI).Any())
            {
                string[] doplneniPrekodovani = GenerovaniPrekodovani(OdvozUkaz, PuvodniTab, PuvodniSloupec, bdbcPrimKlice);
                sloupecStatus = doplneniPrekodovani[0];
                joinStatus = doplneniPrekodovani[1];
                vkladanaHodnota = doplneniPrekodovani[2];
            }

            if (PrimarniKliceZdroje.Contains(PuvodniSloupec))//Pokud je zdrojovy sloupec primárním klíčem tabulky => odstraním poslední čárku v bdbcPrimKlice a PuvodniSloupec nastavym na null
            {
                bdbcPrimKlice = Regex.Replace(bdbcPrimKlice, ", $", "");
                PuvodniSloupec = null;
            }
            else//Pokud zdroj není součástí PK doplním před sloupec D. => pochází z tabulek z daty
            {
                PuvodniSloupec = "D." + PuvodniSloupec;
            }


            string sql = string.Format(
@"MERGE INTO {1} O
USING (SELECT {2}{3}{9} FROM {4} D{10}) B
ON ({5})
WHEN MATCHED THEN UPDATE SET O.{6} = {11}",
//WHEN NOT MATCHED THEN INSERT ({7}O.{6})
//VALUES ({8}{11})"
                                        Environment.NewLine, //0
                                        OdvozUkaz.OU_TAB.JMENO, //1
                                        bdbcPrimKlice, //2
                                        PuvodniSloupec, //3
                                        PuvodniTab,//4
                                        propojeniPK, //5
                                        OdvozUkaz.NAZEV, //6
                                        ouDotPrimKlice, //7
                                        hodnotyB, //8
                                        sloupecStatus, //9
                                        joinStatus, //10
                                        vkladanaHodnota //11
                                        );
            return sql;
        }

        /// <summary>
        /// Generuje sql pro zadanou konstantu
        /// </summary>
        /// <param name="OdvozUkaz"></param>
        /// <returns></returns>
        private static string GenerujKonstanta(OU_ODVOZ_UKAZ OdvozUkaz)
        {
            
            //počet primárních klíčů musí být stejný
            if (OdvozUkaz.SoucetVsechZdroju != 0)
                throw new Exception("Nesmí mít žádné zdroje.");

           
            //NEPOUŽÍVÁ SE NASTAVENO POMOCÍ PARAMETRU FUNKCE
            //Zjistí hodnotu konstanty, pokud je nastaveno AutoPřegenerovaniSQL a nalezne shodu ve vyhledávacím paternu v dříve nastaveném SQL kódu, převezmu tuto hodnotu z původního kódu. Jinak se vždy dotážu uživatele na její hodnotu.
            string konstantaWhere = OdvozUkaz.OU_PARAM_FCE.Where(p => p.TYP_PARAM == ParamFceEnum.konstanta.ToString()).FirstOrDefault().HODNOTA;
            //if (AutoPregenerovaniSQL && Regex.IsMatch(OdvozUkaz.SQL_DOTAZ, string.Format("SET {0} = '{1}'", OdvozUkaz.NAZEV, "\\d+")))
            //{
            //    //Vyhledám konstantu v původním kódu
            //    konstantaWhere = Regex.Match(OdvozUkaz.SQL_DOTAZ, string.Format("SET {0} = '{1}'", OdvozUkaz.NAZEV, "\\d+")).Value;
            //    konstantaWhere = Regex.Match(konstantaWhere, "\\d+").Value;

            //}
            //else
            //{
            //    //Zobrazím uživateli okno pro zadání konstanty
            //    OuDot.ViewModel.PomocnaOkna.NastavHodnotuWindow nhw = new ViewModel.PomocnaOkna.NastavHodnotuWindow();
            //    nhw.Title = OdvozUkaz.NAZEV;
            //    nhw.tblHodnotaLabel.Text = "Konstanta: ";
            //    nhw.ShowDialog();
            //    konstantaWhere = nhw.tbKonstanta.Text;
            //}

            

            string sql = string.Format(
@"UPDATE {1} 
SET {2} = '{3}'"
                                        , Environment.NewLine, OdvozUkaz.OU_TAB.JMENO, OdvozUkaz.NAZEV, konstantaWhere);
            return sql;
        }

        /// <summary>
        /// Generuje 3 doplňující SQL kódy pro překódování
        /// </summary>
        /// <param name="OdvozUkaz">Odvozený ukazatel pro který je kód generován</param>
        /// <param name="PuvodniTab">Jméno tabulky z BDBConverteru</param>
        /// <param name="PuvodniSloupec">Jméno sloupce ze kterého získávám data</param>
        /// <param name="BdbcPrimarniKlice"></param>
        /// <param name="PrimKlicKonstanty"></param>
        /// <param name="Konstanta"></param>
        /// <returns>stringové pole doplnějící SQL: 1.sloupec se statuse, 2.join tabulky se statusy, 3.překódování původních hodnot</returns>
        private static string[] GenerovaniPrekodovani(OU_ODVOZ_UKAZ OdvozUkaz, string PuvodniTab, string PuvodniSloupec, string BdbcPrimarniKlice)
        {
            string[] doplneniPrekodovani = new string[3];

            bool neRovnoHotovo = false;
            //case s dekódováním:
            string dekodovaniHodnoty = "";
            string dekodovaniStatusy = "";
            foreach (OU_CATEGORY kat in OdvozUkaz.OU_CATEGORY)
            {
                foreach (OU_PREKODOVANI prekod in kat.OU_PREKODOVANI)
                {
                    if (prekod.ZPUSOB == TypPrekodovani.Konstanta.ToString())//(prekod.OD != null && prekod.DO == null) //Konstanta
                    {
                        //dekodovaniHodnoty = string.Format("{3},{0}{4}{1}, {2}", Environment.NewLine, prekod.OD, kat.HODNOTA, dekodovaniHodnoty, new String(' ', 23));
                        dekodovaniHodnoty = string.Format("{3}{0}{4}WHEN B.{5} = {1} THEN {2}", Environment.NewLine, prekod.OD, kat.HODNOTA, dekodovaniHodnoty, new String(' ', 14), PuvodniSloupec);
                        //např.: when B.bhDluh_naj = 4 then 11
                    }
                    else if (prekod.ZPUSOB == TypPrekodovani.Interval.ToString())//(prekod.OD != null && prekod.DO != null)//interval:
                    {
                        dekodovaniHodnoty = string.Format("{3}{0}{4}WHEN B.{5} BETWEEN {1} AND {6} THEN {2}", Environment.NewLine, prekod.OD, kat.HODNOTA, dekodovaniHodnoty, new String(' ', 14), PuvodniSloupec, prekod.DO);
                        //např.: when B.bhDluh_naj between 1 and 4 then 11
                    }
                    else if (prekod.ZPUSOB == TypPrekodovani.Vetsi.ToString())//Větší než
                    {
                        dekodovaniHodnoty = string.Format("{3}{0}{4}WHEN B.{5} > {1} THEN {2}", Environment.NewLine, prekod.OD, kat.HODNOTA, dekodovaniHodnoty, new String(' ', 14), PuvodniSloupec);
                    }
                    else if (prekod.ZPUSOB == TypPrekodovani.Mensi.ToString())//Větší než
                    {
                        dekodovaniHodnoty = string.Format("{3}{0}{4}WHEN B.{5} < {1} THEN {2}", Environment.NewLine, prekod.OD, kat.HODNOTA, dekodovaniHodnoty, new String(' ', 14), PuvodniSloupec);
                    }
                    else if (prekod.ZPUSOB == TypPrekodovani.Null.ToString())//Prázdná hodnota
                    {
                        dekodovaniHodnoty = string.Format("{2}{0}{3}WHEN B.{4} IS NULL THEN {1}", Environment.NewLine, kat.HODNOTA, dekodovaniHodnoty, new String(' ', 14), PuvodniSloupec);
                    }
                    else if (prekod.ZPUSOB== TypPrekodovani.NeNull.ToString())//Nerovná se prázdné hodnotě
                    {
                        dekodovaniHodnoty = string.Format("{2}{0}{3}WHEN B.{4} IS NOT NULL THEN {1}", Environment.NewLine, kat.HODNOTA, dekodovaniHodnoty, new String(' ', 14), PuvodniSloupec);
                    }
                    else if (prekod.ZPUSOB == TypPrekodovani.NeRovno.ToString() && !neRovnoHotovo)//Nerovná se
                    {

                        //Všechny Ne rovná se musím zapsat na jeden řádek
                        var listNeRovno = kat.OU_PREKODOVANI.Where(p => p.ZPUSOB == TypPrekodovani.NeRovno.ToString());
                        if (listNeRovno.Count() > 1)
                        {
                            string vsechnyHodnoty = "(";
                            foreach (var neRovno in listNeRovno)
                            {
                                vsechnyHodnoty = vsechnyHodnoty + neRovno.OD + ",";
                            }
                            vsechnyHodnoty = Regex.Replace(vsechnyHodnoty, ",$", ")");

                            dekodovaniHodnoty = string.Format("{3}{0}{4}WHEN B.{5} NOT IN {1} THEN {2}", Environment.NewLine, vsechnyHodnoty, kat.HODNOTA, dekodovaniHodnoty, new String(' ', 14), PuvodniSloupec);
                        }
                        else
                        {
                            dekodovaniHodnoty = string.Format("{3}{0}{4}WHEN B.{5} <> {1} THEN {2}", Environment.NewLine, prekod.OD, kat.HODNOTA, dekodovaniHodnoty, new String(' ', 14), PuvodniSloupec);
                        }
                        //Ne rovná se se zapíší všechny najednou, označím si že již byly zapsány a již je znova nepřidávám
                        neRovnoHotovo = true;
                    }
                    else if (prekod.ZPUSOB == TypPrekodovani.Status.ToString())//(prekod.STATUS != null)//Status
                    {
                        dekodovaniStatusy = string.Format("{3},{0}{4}{1}, {2}", Environment.NewLine, prekod.STATUS, kat.HODNOTA, dekodovaniStatusy, new String(' ', 23));
                        //např.: ,
                        //       5, 3
                    }
                    
                }
            }
            //Pokud OU není kategoriální typ, přidám ze překódování klausuli ELSE a do ní vložím zdroj (a pokud již obsahuje nějaké předchozí kategorie)
            if (!string.IsNullOrEmpty(dekodovaniHodnoty) && OdvozUkaz.OU_TYP.NAZEV != "kategoriální")
            {
                dekodovaniHodnoty = string.Format("{2}{0}{3}ELSE{0}{3}  B.{1}", Environment.NewLine, PuvodniSloupec, dekodovaniHodnoty, new String(' ', 14));
            }

            //vložení nastaveného dekódování pokud jsou součástí statusy
            doplneniPrekodovani[2] = string.Format(
@"
CASE
    WHEN B.status = 0 THEN
        CASE{1}
        END
    ELSE
        DECODE(nvl(B.status,1){2})
END",
                                     PuvodniSloupec,
                                     dekodovaniHodnoty,
                                     dekodovaniStatusy);

            //Pokud nebylo nastaveno žádné překódování pro statusy:
            if (string.IsNullOrEmpty(dekodovaniStatusy))
            {
                doplneniPrekodovani[2] = string.Format("{0}CASE{1}{0}END", Environment.NewLine, dekodovaniHodnoty);
            }
            else//Překódování obsahuje statusy
            {

                //join tabulky se statusy
                doplneniPrekodovani[1] = Regex.Replace(BdbcPrimarniKlice, ", $", "");//zrušení poslední čárky
                string puvodniTabStatusy;
                if (Regex.IsMatch(PuvodniTab, "_D$"))//Pokud nazev obsahuje _D nahradím jeje za _S (tabulku se statusy)
                {
                    puvodniTabStatusy = Regex.Replace(PuvodniTab, "_D$", "_S");//změna tabulky s daty na tabulky se statusy pro data
                }
                else
                {
                    puvodniTabStatusy = PuvodniTab + "_S";
                }
                doplneniPrekodovani[1] = string.Format(
    @"
JOIN {0} S USING ({1})",
                                         puvodniTabStatusy,
                                         doplneniPrekodovani[1]);

                //sloupec s statusem
                doplneniPrekodovani[0] = string.Format(", S.{0} status", PuvodniSloupec);


                if (string.IsNullOrEmpty(dekodovaniHodnoty)) //dekódují se pouze statusy. hodnoty se přenášejí:
                {
                    doplneniPrekodovani[2] = string.Format(
    @"
CASE
    WHEN B.status = 0 THEN
        B.{0}
    ELSE
        DECODE(nvl(B.status,1){1})
END",
                                         PuvodniSloupec,
                                         dekodovaniStatusy);
                }
            }



            return doplneniPrekodovani;
        }

        #endregion Generování SQL

        #region SQL založení primárních klíčů tabulky

        internal static string ZalozeniPK(OU_TAB NovaTabulka,BDBCTABULKY VzorovaTabulka)
        {
            if (NovaTabulka.OU_PRIMARNI_KLICE == null || NovaTabulka.OU_PRIMARNI_KLICE.Count==0)
                throw new Exception("Chybý primarní klíč.");
            
            string bdbcPrimKlice = "";//primární klíče za zdrojové tabulky odděléné čárkami (HD, IDOKRES, IDPAGINA,)
            string propojeniPK = "";//propojení primárních klíčů zdrojové tabulky s výstupní oudotovou (O.HD = B.HD AND O.IDOKRES = B.IDOKRES AND O.IDPAGINA = B.IDPAGINA)
            string ouDotPrimKlice = "";//primární klíče ouDotové tabulky (O.IDOKRES, O.IDPAGINA, O.HD,)
            string hodnotyB = "";//sloupce do kterých se vkládají hodnoty z ouDotPrimKlice (B.IDOKRES, B.IDPAGINA, B.HD,)


            foreach (OU_PRIMARNI_KLICE pkOU in NovaTabulka.OU_PRIMARNI_KLICE)
            {
                var pkBDBC = VzorovaTabulka.BDBC_PRIMARNI_KLICE.Where(b => b.JMENO == pkOU.JMENO).FirstOrDefault();//podle názvu vyberu odpovídající primární klíč ze zdrojové tabulky
                if (pkBDBC == null)//Kontrola zda byl nalezen sloupec primárního klíče v tabulce zdroje
                {
                    string zprava = string.Format("Nebyl nalezen odpovídající primární klíč v tabulce zdroje.{0}{0}Sloupec {1} neexistuje v tabulce zdroje {2}.", Environment.NewLine, pkOU.JMENO, VzorovaTabulka.JMENO);
                    throw new Exception(zprava);
                }


                ouDotPrimKlice = string.Format("{0}O.{1}, ", ouDotPrimKlice, pkOU.JMENO);
                bdbcPrimKlice = string.Format("{0}{1}, ", bdbcPrimKlice, pkBDBC.JMENO);

                propojeniPK = string.Format("{2}O.{0} = B.{1} AND ", pkOU.JMENO, pkBDBC.JMENO, propojeniPK);

                hodnotyB = string.Format("{0}B.{1}, ", hodnotyB, pkBDBC.JMENO);
            }

            propojeniPK = Regex.Replace(propojeniPK, " AND $", "");//Odstraní poslední AND
            bdbcPrimKlice = Regex.Replace(bdbcPrimKlice, ", $", "");
            ouDotPrimKlice = Regex.Replace(ouDotPrimKlice, ", $", "");
            hodnotyB = Regex.Replace(hodnotyB, ", $", "");

            string sql = string.Format(
@"MERGE INTO {0} O
USING (SELECT {1} FROM {2}_D D) B
ON ({3})
WHEN NOT MATCHED THEN INSERT ({4})
VALUES ({5})"
                                        ,NovaTabulka.JMENO, //0
                                        bdbcPrimKlice, //1
                                        VzorovaTabulka.JMENO,//2
                                        propojeniPK, //3
                                        ouDotPrimKlice, //4
                                        hodnotyB //5
                                        );
            return sql;
        
        }

        #endregion SQL založení primárních klíčů tabulky

        #region Pomocné funkce
        /// <summary>
        /// Nalezne právě jeden neshodný sloupec primárních klíčů zdroje a tabulky OUdot
        /// </summary>
        /// <param name="PKTabOUdot">Názvy sloupců PK tabulky OUdot</param>
        /// <param name="ZdrojPK">Názvy sloupců PK tabulky zdroje</param>
        /// <returns></returns>
        public static string NeshodnySloupecPK(List<string> PKTabOUdot, List<string> ZdrojPK)
        {
            if (PKTabOUdot.Count + 1 != ZdrojPK.Count)
            {
                throw new InvalidOperationException("Zdroj musí mít o jeden sloupec primárního klíče navíc.");
            }
            //Najdu všechny sloupce, které nejsou obsaženy v PK OUdot tabulky
            var rozdilneSloupce = ZdrojPK.FindAll(o => !PKTabOUdot.Contains(o, StringComparer.OrdinalIgnoreCase)).ToList();


            if (rozdilneSloupce.Count > 1)
            {
                throw new InvalidOperationException("Více jak 1 sloupce primárního klíče zdroje neodpovídá primárnímu klíči tabulky odvozeného ukazatele.");
            }

            return rozdilneSloupce[0];
        }

        #endregion Pomocné funkce
    }
}
