﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Data.Common;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using OuDot.Model;
using OuDot.Properties;
using Oracle.ManagedDataAccess.Client;

namespace OuDot.ServicesOudot
{

    public static class Services
    {



        internal static void SpocitejOdvozUkaz(ULOHA_METOD_OBDOBI Obdobi, System.Threading.CancellationToken cancellationToken)
        {
            Oracle.ManagedDataAccess.Client.OracleConnection con = DBContext.Instance.OUdotConn;



            try
            {

                //Vymažu všechny tabulky v období a pak je založím znovu, v připadě že se změnili definice:
                //DropTables(Obdobi.OU_TAB, con);

                KontrolaAVytvoreniOracleTabulek(con, Obdobi.OU_TAB, Obdobi.OU_TAB);
                //Pokud není otevřéné spojení do oracle otevře se

                if (con.State != ConnectionState.Open)
                    con.Open();




                int? maxGenerace = Obdobi.OU_TAB.SelectMany(t => t.OU_ODVOZ_UKAZ).Max(o => o.GENERACE);
                for (int g = 0; g <= maxGenerace; g++)
                {
                    //seřazení tabulkek podle pořadí:
                    var tabSerazene = Obdobi.OU_TAB.OrderBy(t => t.PORADI);
                    foreach (OU_TAB tab in tabSerazene)
                    {
                        //Podle nastavené genereace založení PK
                        if (tab.GEN_ZALOZ_PK == g)
                        {//Založení hodnot PK pro tabulku, probíhá v každé generaci, protože se v kódu mohu odkazovat na vypočtené OU
                            try
                            {
                                SpusteneniOraKodu(tab.SQL_GENEROVANI_PK, con);
                            }
                            catch (Exception ex)
                            {
                                string zprava = $"{ex.Message}{Environment.NewLine}{Environment.NewLine}Chyba založení hodnot PK u tabulky {tab.JMENO}.";
                                throw new Exception(ex.Message);
                            }
                        }

                        //Spočítám OU, pokud již byl založen v tabulce PK (má řádky, jinak nespouštím)
                        if (tab.GEN_ZALOZ_PK <= g)
                        {
                            var serazeneOU = tab.OU_ODVOZ_UKAZ.Where(o => o.STAV == true).Where(o => o.GENERACE == g).OrderBy(o => o.PORADI_VYPOCTU);
                            foreach (OU_ODVOZ_UKAZ ou in serazeneOU)
                            {
                                SpusteneniOraKodu(ou, con);
                                cancellationToken.ThrowIfCancellationRequested();//Vyhodí chybu pokud je akce přerušena
                            }
                        }
                        

                        //spuštění opravného kódu tabulky po 1.generaci:
                        if (g == 1)
                        {
                            try
                            {
                                SpusteneniOraKodu(tab.SQL_OPRAVA_DAT, con);
                            }
                            catch (Exception ex)
                            {
                                string zprava = $"{ex.Message}{Environment.NewLine}{Environment.NewLine}Chyba při zpracování opravného kódu pro 1.generaci u tabulky  {tab.JMENO}.";
                                throw new Exception(ex.Message);
                            }
                        }
                    }
                }


                //Po vypočtení všech OU sputím opravné kódy pro tabulky
                //foreach (var tab in Obdobi.OU_TAB.OrderBy(t => t.PORADI))
                //{
                //    try
                //    {
                //        SpusteneniOraKodu(tab.SQL_OPRAVA_DAT, con);
                //    }
                //    catch (Exception ex)
                //    {
                //        string zprava = $"{ex.Message}{Environment.NewLine}{Environment.NewLine}Chyba při zpracování opravného kódu u tabulky  {tab.JMENO}.";
                //        throw new Exception(ex.Message);
                //    }
                //}



                //dropnutí pomocný tabulek:
                var tabPomocne = Obdobi.OU_TAB.Where(t => t.POMOCNA == true);
                foreach (var tab in tabPomocne)
                {
                    //Vytvoření sql commandu na vytvoření tabulky v oracle
                    DbCommand cmd = con.CreateCommand();

                    cmd.CommandText = "DROP TABLE " + tab.JMENO;

                    cmd.ExecuteNonQuery();
                }
            }
            finally
            {
                con.Close();
            }




        }

        internal static void PocitejTabulku(Oracle.ManagedDataAccess.Client.OracleConnection MyConnection, OU_TAB Tabulka, System.Threading.CancellationToken cancellationToken)
        {
            try
            {
                //Pokud není otevřéné spojení do oracle otevře se
                if (MyConnection.State != ConnectionState.Open)
                    MyConnection.Open();
                try
                {
                    SpusteneniOraKodu(Tabulka.SQL_GENEROVANI_PK, MyConnection);
                }
                catch (Exception ex)
                {
                    string zprava = $"{ex.Message}{Environment.NewLine}{Environment.NewLine}Chyba založení hodnot PK u tabulky {Tabulka.JMENO}.";
                    throw new Exception(ex.Message);
                }
                //Spuštění sql pro založení hodnot PK



                // seřazení podle genrece
                var serazeneOU = Tabulka.OU_ODVOZ_UKAZ.Where(o => o.STAV == true).OrderBy(o => o.GENERACE);
                serazeneOU = serazeneOU.ThenBy(o => o.PORADI_VYPOCTU);
                foreach (OU_ODVOZ_UKAZ ou in serazeneOU)
                {

                    SpusteneniOraKodu(ou, MyConnection);
                    cancellationToken.ThrowIfCancellationRequested();//Vyhodí chybu pokud je akce přerušena
                }
                
                //Spuštění opravného kódu tabulky
                try
                {
                    SpusteneniOraKodu(Tabulka.SQL_OPRAVA_DAT, MyConnection);
                }
                catch (Exception ex)
                {
                    string zprava = $"{ex.Message}{Environment.NewLine}{Environment.NewLine}Chyba při zpracování opravného kódu u tabulky {Tabulka.JMENO}.";
                    throw new Exception(ex.Message);
                }
            }
            finally
            {
                MyConnection.Close();
            }

        }

        /// <summary>
        /// Kontroluje existenci tabulek v Oraclu a pokud neexistují vytvoří je.
        /// </summary>
        /// <param name="db">Oracle db</param>
        internal static void KontrolaAVytvoreniOracleTabulek(Oracle.ManagedDataAccess.Client.OracleConnection MyConnection, ObservableCollection<OU_TAB> tabulky, IEnumerable<OU_TAB> TabulkyDrop)
        {
            //Kontrola existence tabulky v oraclu pokud neexistuje uloži se do listu
            List<OU_TAB> NeexistujeVOracle = new List<OU_TAB>();
            foreach (OU_TAB tab in tabulky)
            {
                //sql které hledá podle jména tabulku v oracle
                string sql = string.Format(@"SELECT TABLE_NAME FROM ALL_TABLES WHERE TABLE_NAME='{0}'", tab.JMENO.ToUpper());

                //Pokud není otevřéné spojení do oracle otevře se
                if (MyConnection.State != ConnectionState.Open)
                    MyConnection.Open();

                //Vytvoření commandu pro oracle
                DbCommand cmd = MyConnection.CreateCommand();
                cmd.CommandText = sql;
                //Spuštění sql commandu pokud nebyla nalezena vrací null
                object vysledek = cmd.ExecuteScalar();
                if (vysledek == null)
                    NeexistujeVOracle.Add(tab);
                else if (TabulkyDrop.Contains(tab))
                {
                    //Vytvoření sql commandu na vytvoření tabulky v oracle
                    cmd = MyConnection.CreateCommand();

                    cmd.CommandText = "DROP TABLE " + tab.JMENO;

                    cmd.ExecuteNonQuery();

                    NeexistujeVOracle.Add(tab);
                }




            }
            //Popkud nějáká tabulka neexistuje vytvoří se
            if (NeexistujeVOracle.Count > 0)
                VytvoreniOracleTabulek(NeexistujeVOracle, MyConnection);

            //Zkopírování načtených dat do oracle
            //try
            //{
            //    DataDoOracle(db.Connection.ConnectionString, ds);
            //}
            //catch (Exception ex)
            //{ throw ex; }
            //finally
            //{
            //    //Vždy je třeba zapsat zbylá data
            //    //Pokud se nepodaří zapsat vše jsou uložena jen ta dat, která nebyla vložena do Oracle
            //    if (File.Exists(xml))
            //        File.Delete(xml);
            //    ds.WriteXml(xml, XmlWriteMode.WriteSchema);

            //}



        }


        /// <summary>
        /// Vytváří tabulky v oracle
        /// </summary>
        /// <param name="NeexistujiciTabulkyVOraclu">List jmen tabulek, které budu vytvářet</param>
        /// <param name="db">Oracle DB</param>
        private static void VytvoreniOracleTabulek(List<OU_TAB> NeexistujiciTabulkyVOraclu, Oracle.ManagedDataAccess.Client.OracleConnection MyConnection)
        {   //Pro všechny neexistující jména tabulek
            foreach (OU_TAB tab in NeexistujiciTabulkyVOraclu)
            {

                //Vytvoření sql commandu na vytvoření tabulky v oracle
                DbCommand cmd = MyConnection.CreateCommand();
                cmd.CommandText = string.Format("CREATE TABLE {0} (", tab.JMENO.ToUpper());
                //Vložit sloupce PK podle 1. odvozeného ukazatele
                foreach (OU_PRIMARNI_KLICE pk in tab.OU_PRIMARNI_KLICE)
                {
                    cmd.CommandText = cmd.CommandText + string.Format("{0} {1},", pk.JMENO.ToUpper(), "NUMBER");
                }

                foreach (OU_ODVOZ_UKAZ ou in tab.OU_ODVOZ_UKAZ)
                {   //přidání jména sloupce a datový typ pro oracle
                    cmd.CommandText = cmd.CommandText + string.Format("{0} {1},", ou.NAZEV.ToUpper(), OracleDBColumnType((DatoveTypy)ou.DATA_TYP, ou.DATA_TYP_P1, ou.DATA_TYP_P2));
                }

                //Vytvoření prim klíče pro tabulku
                cmd.CommandText = cmd.CommandText + string.Format("CONSTRAINT {0}_PK PRIMARY KEY (", tab.JMENO.ToUpper());
                //1.základní primarní klíče
                foreach (OU_PRIMARNI_KLICE ou in tab.OU_PRIMARNI_KLICE)
                {
                    cmd.CommandText = cmd.CommandText + string.Format("{0},", ou.JMENO.ToUpper());
                }
                //2. přidané primarní klíče
                var pridanePK = from o in tab.OU_ODVOZ_UKAZ
                                where o.PRIMARNI_KLIC
                                select o;
                foreach (OU_ODVOZ_UKAZ ou in pridanePK)
                {
                    cmd.CommandText = cmd.CommandText + string.Format("{0},", ou.NAZEV.ToUpper());
                }


                //Za poslednim sloupcem v prim. klíči se odstraní čárka a nahradí koncem závorky
                if (cmd.CommandText.EndsWith(","))
                    cmd.CommandText = cmd.CommandText.Substring(0, cmd.CommandText.Length - 1) + ")";

                //Po přídání všech  sloupců se přidá konec závorky a spustí se command
                cmd.CommandText = cmd.CommandText + string.Format(")");

                cmd.ExecuteNonQuery();


                //Přířazení práva na tabulku rolím:
                cmd = MyConnection.CreateCommand();
                cmd.CommandText = "GRANT SELECT ON " + tab.JMENO.ToUpper() + " TO ROLE_CENTRALA_S";
                cmd.ExecuteNonQuery();

                cmd = MyConnection.CreateCommand();
                cmd.CommandText = "GRANT select, insert, update, delete ON " + tab.JMENO.ToUpper() + " TO ROLE_CENTRALA_SUDI";
                cmd.ExecuteNonQuery();

                cmd = MyConnection.CreateCommand();
                string role = string.Format("ROLE_{0}_METODIK_M", tab.ULOHA_METOD_OBDOBI.ULOHA_METOD.NAZEV_ID.ToUpper());
                cmd.CommandText = "GRANT select, insert, update, delete ON " + tab.JMENO.ToUpper() + " TO " + role;
                cmd.ExecuteNonQuery();
            }
        }


        /// <summary>
        /// Spustí definované kódy na odvozeném ukazately
        /// </summary>
        /// <param name="OdvozUkaz"></param>
        /// <param name="MyConnection"></param>
        internal static void SpusteneniOraKodu(OU_ODVOZ_UKAZ OdvozUkaz, Oracle.ManagedDataAccess.Client.OracleConnection MyConnection)
        {
            //Výpočet:
            if (!string.IsNullOrEmpty(OdvozUkaz.SQL_DOTAZ?.Trim())) // Pokud kod pro vypocet není prázdný
            {
                DbCommand cmd = MyConnection.CreateCommand();

                cmd.CommandText = OdvozUkaz.SQL_DOTAZ;
                try
                {
                    cmd.ExecuteNonQuery();
                }
                catch (Exception ex)
                {
                    string zprava = string.Format("{1}{0}{0}Chyba nastala při zpracování {2} v tabulce {3}", Environment.NewLine, ex.Message, OdvozUkaz.NAZEV, OdvozUkaz.OU_TAB.JMENO);
                    throw new Exception(zprava);
                }
            }

            //Opravný kód
            if (!string.IsNullOrEmpty(OdvozUkaz.OPRAVNY_KOD?.Trim())) // Pokud opravný kod není prázdný
            {
                DbCommand cmd = MyConnection.CreateCommand();

                cmd.CommandText = OdvozUkaz.OPRAVNY_KOD;
                try
                {
                    cmd.ExecuteNonQuery();
                }
                catch (Exception ex)
                {
                    string zprava = string.Format("{1}{0}{0}Chyba při zpracování opravného kódu {2} v tabulce {3}", Environment.NewLine, ex.Message, OdvozUkaz.NAZEV, OdvozUkaz.OU_TAB.JMENO);
                    throw new Exception(zprava);
                }
            }

        }

        internal static void SpusteneniOraKodu(String OraKod, OracleConnection MyConnection)
        {

            //Spuštění opravného kódu
            if (!string.IsNullOrEmpty(OraKod?.Trim())) // Pokud opravný kod není prázdný
            {
                DbCommand cmd = MyConnection.CreateCommand();

                cmd.CommandText = OraKod;

                cmd.ExecuteNonQuery();

            }
        }

        /// <summary>
        /// Podle dotvého typu vrací datový typ pro oracle
        /// </summary>
        /// <param name="Type">datový typ, který chci převést</param>
        /// <returns>strig s datový typem pro oracle</returns>
        internal static string OracleDBColumnType(DatoveTypy Type, int? P1, int? P2)
        {
            //Nastaveni parametrů datového typu
            //MatchCollection x = Regex.Matches(Type, @"\[(.*?)\]");
            //string p1 = "";
            //string p2 = "";
            //if (x != null)
            //{
            //    if (x.Count >= 1)
            //        p1 = x[0].Groups[1].Value;
            //    if (x.Count == 2)
            //        p2 = x[1].Groups[1].Value;
            //}
            ////datový typ bez parametru
            //string typBezParam = Regex.Match(Type, @"^(.*?)(\[|$)").Groups[1].Value;
            //Vrátí podle typu a parametrů odpovídající definici pro Oracle
            switch (Type)
            {
                case DatoveTypy.text:
                    if (P1 == null || P1 > 2000)
                        return "CLOB";
                    else
                        return string.Format("NVARCHAR2({0})", P1);
                case DatoveTypy.číslo:
                    if (P1 == null)
                        return "NUMBER";
                    else if (P2 == null)
                        return string.Format("NUMBER({0})", P1);
                    else
                        return string.Format("NUMBER({0},{1})", P1, P2);
                case DatoveTypy.datumAčas:
                    return "DATE";
                case DatoveTypy.čas:
                    return "TIMESTAMP(7)";
                case DatoveTypy.kategorie:
                    return "NUMBER(3)";
                case DatoveTypy.číselník:
                case DatoveTypy.klasifikace:
                    return "VARCHAR(255)";
                case DatoveTypy.interval_den:
                    return string.Format("INTERVAL DAY({0}) TO SECOND({1})", P1, P2);
                case DatoveTypy.interval_rok:
                    return string.Format("INTERVAL YEAR({0}) TO MONTH", P1);
                case DatoveTypy.boolean:
                    return "NUMBER(1)";
                case DatoveTypy.PlovCarka:
                    return "FLOAT";
                default:
                    throw new Exception("Nedefinovaný datový typ pro novou tabulku v Oracle.");

            }
        }

        #region Vytvoreni View
        /// <summary>
        /// Vygenereuje View do Oracle podle dané definice
        /// </summary>
        /// <param name="JmenoView">Pojmenování vystupního view</param>
        /// <param name="OUVyber">Všechny OU, které patří do výstupního view</param>
        /// <param name="EUVystup">True v případě že jako názvy sloupců se mají použít EU názvy</param>
        /// <param name="Tabulky">Slovník všech používaných tabulek a klíčem je zástupný znak za tabulku</param>
        /// <param name="PrimarniKlic">Společný PK, jednotlivé sloupce PK oddělené čárkami</param>
        /// <param name="Obdobi">Období pro které tvořím výstup</param>
        public static void VytvorOraView(string JmenoView, List<OU_ODVOZ_UKAZ> OUVyber, bool EUVystup, Dictionary<char, OU_TAB> Tabulky, string PrimarniKlic, ULOHA_METOD_OBDOBI Obdobi)
        {
            OracleConnection MyConnection = DBContext.Instance.OUdotConn;//Připojení k DB

            try
            {

                //1.Ověření existence View
                string sql = string.Format(@"SELECT VIEW_NAME FROM ALL_VIEWS WHERE VIEW_NAME='{0}'", JmenoView.ToUpper());

                //Pokud není otevřéné spojení do oracle otevře se
                if (MyConnection.State != ConnectionState.Open)
                    MyConnection.Open();

                //Vytvoření commandu pro oracle
                DbCommand cmd = MyConnection.CreateCommand();
                cmd.CommandText = sql;
                //Spuštění sql commandu, pokud název View nebylo nalezena vrací null
                object vysledek = cmd.ExecuteScalar();
                if (vysledek != null)
                {
                    //Pokud View již existuje zeptám se jestli pokračovat, pokud ne ukončuji metodu
                    var resulat = System.Windows.MessageBox.Show($"View {JmenoView} s tímto názvem již v Oracle databázi existuej.{Environment.NewLine}Přejete si pokračovat vymazáním původního a vytvořením nového?", "Vytvořní view", System.Windows.MessageBoxButton.YesNo, System.Windows.MessageBoxImage.Question);
                    if (resulat == System.Windows.MessageBoxResult.No)
                    {
                        return;
                    }
                    //else
                    //{
                    //    cmd = MyConnection.CreateCommand();
                    //    cmd.CommandText = "DROP VIEW " + JmenoView;
                    //    cmd.ExecuteNonQuery();
                    //}
                }



                //2. Vytvoření View
                //Vytvoření stringu sloupců, názvy oddělené čárkami, pokud EU výstup, jsou za názvy přidány EU aliasy
                string sloupce = "";
                if (EUVystup)
                {

                    var x = OUVyber.Select(o => Tabulky.Where(t => t.Value == o.OU_TAB).FirstOrDefault().Key + "." + o.NAZEV + " as " + o.NAZEV_EU);
                    sloupce = string.Join(", ", x);
                }
                else
                {
                    sloupce = string.Join(", ", OUVyber.Select(o => Tabulky.Where(t => t.Value == o.OU_TAB).FirstOrDefault().Key + "." + o.NAZEV));
                }

                //Vytvořím text commandu na vytvoření view pro Oracle DB
                cmd = MyConnection.CreateCommand();
                cmd.CommandText = $"create or replace view {JmenoView} as select {sloupce} from {Tabulky['A'].JMENO} A";
                Tabulky.Remove('A');

                //Pokud jsou výstupu z více tabulek přidám je joinem do textu commandu
                foreach (var tab in Tabulky)
                {
                    cmd.CommandText = $"{cmd.CommandText}{Environment.NewLine}join {tab.Value.JMENO} {tab.Key} using ({PrimarniKlic})";
                }
                //Supstím command
                cmd.ExecuteNonQuery();


                //Přířazení práva na View rolím:
                cmd = MyConnection.CreateCommand();
                cmd.CommandText = "GRANT SELECT ON " + JmenoView.ToUpper() + " TO ROLE_CENTRALA_S";
                cmd.ExecuteNonQuery();

                cmd = MyConnection.CreateCommand();
                cmd.CommandText = "GRANT select, insert, update, delete ON " + JmenoView.ToUpper() + " TO ROLE_CENTRALA_SUDI";
                cmd.ExecuteNonQuery();

                cmd = MyConnection.CreateCommand();
                string role = string.Format("ROLE_{0}_METODIK_M", Obdobi.ULOHA_METOD.NAZEV_ID.ToUpper());
                cmd.CommandText = "GRANT select, insert, update, delete ON " + JmenoView.ToUpper() + " TO " + role;
                cmd.ExecuteNonQuery();
            }
            finally
            {

                MyConnection.Close();//Uzavřu spojení na DB
            }




        }

        #endregion Vytvoreni View

        #region Exception helper metods

        /// <summary>
        /// Rekurzivní funkce, která vrací všechny zprávy z InnerException
        /// </summary>
        /// <param name="Ex">Vyjímka ze které získám všechny error zprávy.</param>
        /// <returns>Všechny error zprávy oddělené entrem ze všech InnerException</returns>
        public static string ExceptionsText(Exception Ex)
        {
            string errorText = Ex.Message;
            if (Ex.InnerException != null)
            {
                errorText = string.Format("{1}{0}{2}", Environment.NewLine, errorText, ExceptionsText(Ex.InnerException));
            }

            return errorText;
        }

        #endregion Exeption helper metods


        #region Externí zdroje import

        public static void ImportXlsToOracle(EXT_TABULKY ExterniTabulka, DataTable TabulkaSDaty, Oracle.ManagedDataAccess.Client.OracleConnection Mycon)
        {


            using (Oracle.DataAccess.Client.OracleBulkCopy orbc = new Oracle.DataAccess.Client.OracleBulkCopy(Mycon.ConnectionString, Oracle.DataAccess.Client.OracleBulkCopyOptions.UseInternalTransaction))
            {

                //Nastavení jména tabulky do kterého se bude kopírovat a zkopírování datatable
                orbc.DestinationTableName = ExterniTabulka.JMENO_ID.ToUpper();
                orbc.BatchSize = TabulkaSDaty.Rows.Count;//nutné nastavit velikost dávky aby kopírování proběhlo jako transkace

                foreach (EXT_SLOUPCE extSl in ExterniTabulka.EXT_SLOUPCE)
                {   //Mapování sloupců      Zdrojový dataTable  Oracle vše velkými píslemny
                    orbc.ColumnMappings.Add(extSl.dtJmeno, extSl.JMENO.ToUpper());
                }

                orbc.WriteToServer(TabulkaSDaty);
            }

        }


        public static void VytvoreniExterniTabulky(EXT_TABULKY ExterniTabulka, Oracle.ManagedDataAccess.Client.OracleConnection Mycon)
        {
            //sql které hledá podle jména tabulku v oracle
            string sql = string.Format(@"SELECT TABLE_NAME FROM ALL_TABLES WHERE TABLE_NAME='{0}'", ExterniTabulka.JMENO_ID.ToUpper());

            //Pokud není otevřéné spojení do oracle otevře se
            if (Mycon.State != ConnectionState.Open)
                Mycon.Open();

            //Vytvoření commandu pro oracle
            DbCommand cmd = Mycon.CreateCommand();
            cmd.CommandText = sql;
            //Spuštění sql commandu pokud nebyla nalezena vrací null
            object vysledek = cmd.ExecuteScalar();
            if (vysledek == null)
            {
                //Vytvoření sql commandu na vytvoření tabulky v oracle
                cmd = Mycon.CreateCommand();
                cmd.CommandText = string.Format("CREATE TABLE {0} (", ExterniTabulka.JMENO_ID.ToUpper());



                foreach (EXT_SLOUPCE extSl in ExterniTabulka.EXT_SLOUPCE)
                {   //přidání jména sloupce a datový typ pro oracle
                    cmd.CommandText = cmd.CommandText + string.Format("{0} {1},", extSl.JMENO.ToUpper(), extSl.DATA_TYP);
                }

                //2. přidané primarní klíče
                var pridanePK = from e in ExterniTabulka.EXT_SLOUPCE
                                where e.PRIMARNI_KLIC == true
                                select e;
                if (pridanePK.Count() != 0)
                {
                    //Vytvoření prim klíče pro tabulku
                    cmd.CommandText = cmd.CommandText + string.Format("CONSTRAINT {0}_PK PRIMARY KEY (", ExterniTabulka.JMENO_ID.ToUpper());


                    foreach (EXT_SLOUPCE extPK in pridanePK)
                    {
                        cmd.CommandText = cmd.CommandText + string.Format("{0},", extPK.JMENO.ToUpper());
                    }
                }


                //Za poslednim sloupcem v prim. klíči se odstraní čárka a nahradí koncem závorky
                if (cmd.CommandText.EndsWith(","))
                    cmd.CommandText = cmd.CommandText.Substring(0, cmd.CommandText.Length - 1) + ")";


                //Pokud byli přidány PK doplním konec závorky
                if (pridanePK.Count() != 0)
                    cmd.CommandText = cmd.CommandText + string.Format(")");

                cmd.ExecuteNonQuery();


                //Přířazení práva na tabulku rolím:
                cmd = Mycon.CreateCommand();
                cmd.CommandText = "GRANT SELECT ON " + ExterniTabulka.JMENO_ID.ToUpper() + " TO ROLE_CENTRALA_S";
                cmd.ExecuteNonQuery();

                cmd = Mycon.CreateCommand();
                cmd.CommandText = "GRANT select, insert, update, delete ON " + ExterniTabulka.JMENO_ID.ToUpper() + " TO ROLE_CENTRALA_SUDI";
                cmd.ExecuteNonQuery();

                cmd = Mycon.CreateCommand();
                string role = string.Format("ROLE_{0}_METODIK_M", DBContext.Instance.Entity.ULOHA_METOD_OBDOBI.Find(Settings.Default.IdMetodObdobi).ULOHA_METOD.NAZEV_ID.ToUpper());
                cmd.CommandText = "GRANT select, insert, update, delete ON " + ExterniTabulka.JMENO_ID.ToUpper() + " TO " + role;
                cmd.ExecuteNonQuery();

                cmd = Mycon.CreateCommand();
                string roleT = string.Format("ROLE_{0}_METODIK_T", DBContext.Instance.Entity.ULOHA_OBDOBI.Find(Settings.Default.IdTerenUlohaObd).ULOHA.ZKRATKA_ULOHY.ToUpper());
                cmd.CommandText = "GRANT select, insert, update, delete ON " + ExterniTabulka.JMENO_ID.ToUpper() + " TO " + roleT;
                cmd.ExecuteNonQuery();

                //Práva pro uživatele, není nutné
                //Oracle.ManagedDataAccess.Client.OracleConnectionStringBuilder sb = new Oracle.ManagedDataAccess.Client.OracleConnectionStringBuilder(DBContext.Instance.Entity.Database.Connection.ConnectionString);
                //cmd.CommandText = "GRANT select, insert, update, delete ON " + ExterniTabulka.JMENO_ID.ToUpper() + " TO " + sb.UserID;
            }
            else
            {
                throw new Exception("Tabulka s tímto názvem již v Oracle existuje.");
            }

        }

        /// <summary>
        /// Podle dotvého typu vrací datový typ pro oracle
        /// </summary>
        /// <param name="Type">datový typ, který chci převést</param>
        /// <returns>strig s datový typem pro oracle</returns>
        internal static string OracleColumnTypeFromDC(Type Type)
        {

            switch (Type.Name)
            {
                case "String":
                    return "NVARCHAR2(2000)";
                case "Int64":
                    return "NUMBER(18,0)";
                case "Int32":
                    return "NUMBER(9,0)";
                case "Decimal":
                    return "NUMBER";
                case "DateTime":
                    return "DATE";
                case "TimeSpan":
                    return "TIMESTAMP(7)";
                case "Double":
                    return "FLOAT";
                case "Boolean":
                    return "NUMBER(1)";
                default:
                    throw new Exception("Nedefinovaný datový typ pro novou tabulku v Oracle.");

            }
        }
        #endregion Externí zdroje import
    }
}
