﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using GalaSoft.MvvmLight.CommandWpf;
using OuDot.Model;
using OuDot.ServicesOudot;

namespace OuDot.ViewModel.IporterWizzard
{
    public sealed class ImporterViewModel : ViewModelBase
    {

        private static readonly ImporterViewModel _instance = new ImporterViewModel();
        /// <summary>
        /// Vrací svojí vlastní instanci
        /// </summary>
        public static ImporterViewModel Instance
        {
            get
            {
                return _instance;
            }
        }

        private string _cestaXls;

        public string CestaXls
        {
            get { return _cestaXls; }
            set
            {
                if (_cestaXls != value)
                {
                    _cestaXls = value;
                    OnPropertyChanged();
                }
            }
        }


        private DataTable _xlsTabulka;

        public DataTable XlsTabulka
        {
            get { return _xlsTabulka; }
            set
            {
                if (_xlsTabulka != value)
                {
                    _xlsTabulka = value;
                    OnPropertyChanged();
                }
            }
        }



        private EXT_TABULKY _extTabulka;

        public EXT_TABULKY ExtTabulka
        {
            get { return _extTabulka; }
            set
            {
                if (_extTabulka != value)
                {
                    _extTabulka = value;
                    OnPropertyChanged();
                }
            }
        }

        private bool _isBusy;
        /// <summary>
        /// Značí, že probíhá akce na pozadí programu
        /// </summary>
        public bool IsBusy
        {
            get { return _isBusy; }
            set
            {
                if (_isBusy != value)
                {
                    _isBusy = value;
                    OnPropertyChanged();
                }
            }
        }

        public event EventHandler ImportExtZdroje;

        public RelayCommand VyberXlsCommand { get; private set; }
        public RelayCommand NactiXlsCommand { get; private set; }
        public RelayCommand ImportCommand { get; private set; }

        private ImporterViewModel()
        {
            if (DesignerProperties.GetIsInDesignMode(new System.Windows.DependencyObject()))
            {

                // Code runs in Blend --> create design time data.
            }
            else
            {
                // Code runs "for real"

                VyberXlsCommand = new RelayCommand(() => VyberXls());
                NactiXlsCommand = new RelayCommand(() => NactiXls());
                ImportCommand = new RelayCommand(() => Import());
            }
        }

        internal async void Import()
        {
            try
            {
                if (Validace.ValidacniChyby.Count > 0)
                {
                    throw new Exception("Byly zjištěny chyby při validaci.");
                }
                IsBusy = true;
                if (ExtTabulka == null || XlsTabulka == null)
                    throw new Exception("Nebyla načtena žádná data.");

                await Task.Run(() =>
                {
                    Services.VytvoreniExterniTabulky(ExtTabulka, DBContext.Instance.OUdotConn);
                    Services.ImportXlsToOracle(ExtTabulka, XlsTabulka, DBContext.Instance.OUdotConn);
                });

                
                DBContext.Instance.Entity.SaveChanges();
                
                //Spustím událost o naimportovaní externích zdrojů
                if(ImportExtZdroje != null)
                 ImportExtZdroje.Invoke(ExtTabulka, null);

                MessageBox.Show("Data byla úspěšně naimportována", "Import Ext zdroje", MessageBoxButton.OK, MessageBoxImage.Information);

            }
            catch (Exception ex)
            {
                string zprava = string.Format("Nastala chyba při importu tabulky:{0}{1}", Environment.NewLine, ex.Message);
                MessageBox.Show(zprava, "Import Ext zdroje", MessageBoxButton.OK, MessageBoxImage.Error);
            }
            finally
            {
                ExtTabulka = null;
                DBContext.Instance.OUdotConn.Close();
                IsBusy = false;
            }
        }

        public void VyberXls()
        {
            CestaXls = SouboryPomocne.VyberSoubor("", "Excel Files|*.xls;*.xlsx|All Files (*.*)|*.*");
        }

        public async void NactiXls()
        {
            
            if (!File.Exists(CestaXls))
            {
                string zprava = "Nebyl naleze vybranný Excelovský soubor.";
                MessageBox.Show(zprava, "Import Ext zdroje", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }
            IsBusy = true;
            try
            {
                 
                //Pokud již je něco načteno odstraním to z kontextu
                if (ExtTabulka != null)
                {
                    DBContext.Instance.Entity.EXT_TABULKY.Remove(ExtTabulka);
                    ExtTabulka = null;
                }

                string excelConnectionString = string.Format("Provider=Microsoft.ACE.OLEDB.12.0;Data Source={0};Extended Properties=Excel 8.0", CestaXls);
                System.Data.OleDb.OleDbConnection connection = new System.Data.OleDb.OleDbConnection();
                connection.ConnectionString = excelConnectionString;

                await Task.Run(() => { connection.Open(); });

                DataTable dtSchema = connection.GetOleDbSchemaTable(System.Data.OleDb.OleDbSchemaGuid.Tables, new object[] { null, null, null, "TABLE" });
                string sheet1 = dtSchema.Rows[0].Field<string>("TABLE_NAME");
                System.Data.OleDb.OleDbCommand command = new System.Data.OleDb.OleDbCommand(string.Format("select * from [{0}]", sheet1), connection);


                System.Data.OleDb.OleDbDataAdapter daexcel = new System.Data.OleDb.OleDbDataAdapter(command);

                XlsTabulka = new DataTable(Path.GetFileNameWithoutExtension(CestaXls));
                await Task.Run(() => { daexcel.Fill(XlsTabulka); });

                VytvorPopisneObjekty(XlsTabulka);
            }
            catch (Exception ex)
            {
                string zprava = string.Format("Nepodařili se načíst data z Excelovského souboru: {1}.{0}{0}Chyba:{2}", Environment.NewLine, Path.GetFileName(CestaXls), ex.Message);
                MessageBox.Show(zprava, "Import Ext zdroje", MessageBoxButton.OK, MessageBoxImage.Error);
            }
            IsBusy = false;
        }

        internal void VytvorPopisneObjekty(DataTable dtImport)
        {
            ExtTabulka = new EXT_TABULKY { JMENO_ID = "EXT_" + dtImport.TableName, PLATNOST_OD = DateTime.Now };
            DBContext.Instance.Entity.EXT_TABULKY.Add(ExtTabulka);
            foreach (DataColumn dc in dtImport.Columns)
            {

                DBContext.Instance.Entity.EXT_SLOUPCE.Add(new EXT_SLOUPCE { JMENO = dc.ColumnName, dtJmeno = dc.ColumnName, DATA_TYP = Services.OracleColumnTypeFromDC(dc.DataType), EXT_TABULKY = ExtTabulka });
            }

            
        }

    }
}
