﻿using OuDot.Model;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace OuDot.ViewModel.Knihovnik
{
    /// <summary>
    /// Interaction logic for VystupyDefinicePoradiWindow.xaml
    /// </summary>
    public partial class VystupyDefinicePoradiWindow : Window
    {
        public List<OU_ODVOZ_UKAZ> VybraneOU { get; set; }
        public VystupyDefinicePoradiWindow(List<OU_ODVOZ_UKAZ> ListOU)
        {
            InitializeComponent();
            VybraneOU = ListOU;
            dgPoradiOU.ItemsSource = VybraneOU;
        }

        #region Řazení

        private void bNazev_Click(object sender, RoutedEventArgs e)
        {
            VybraneOU = VybraneOU.OrderBy(o => o.NAZEV).ToList();
            dgPoradiOU.ItemsSource = VybraneOU;
        }

        private void bNezevEU_Click(object sender, RoutedEventArgs e)
        {
            VybraneOU = VybraneOU.OrderBy(o => o.NAZEV_EU).ToList();
            dgPoradiOU.ItemsSource = VybraneOU;
        }

        private void bGenPor_Click(object sender, RoutedEventArgs e)
        {
            VybraneOU = VybraneOU.OrderBy(o => o.GENERACE).ThenBy(o => o.PORADI_VYPOCTU).ToList();
            dgPoradiOU.ItemsSource = VybraneOU;
        }

        #endregion Řazení

        //Zdroj http://www.c-sharpcorner.com/uploadfile/raj1979/drag-and-drop-datagrid-row-in-wpf/
        #region Drag&Drop
        private List<OU_ODVOZ_UKAZ> _vybraneRadky = new List<OU_ODVOZ_UKAZ>();//vybrané řádky v DataGridu
        private void dgPoradiOU_MouseMove(object sender, MouseEventArgs e)
        {
            //base.OnMouseMove(e);
            //pokud je zmáčknuté levé tlačítko myši
            if (e.LeftButton == MouseButtonState.Pressed)
            {
                
                if (this.dgPoradiOU.SelectedItems == null || this.dgPoradiOU.SelectedItems.Count == 0)
                    return;

                //získá prvek na který jsme klikli
                Point p = e.GetPosition((IInputElement)e.Source);
                HitTestResult result = VisualTreeHelper.HitTest((Visual)e.Source, p);
                if (result == null)
                    return;
                var obj = result.VisualHit;

                //hlédá jestli je visuální objekt DataGridRow, pokud není, posouvá se na rodiče tohoto obejtktu až dokud žádný rodič není tedy null
                while (VisualTreeHelper.GetParent(obj) != null && !(obj is DataGridRow))
                {
                    obj = VisualTreeHelper.GetParent(obj);
                }
                //pokud je výsledek null opouštíme událost
                if (obj == null) return;

                // uložíme objekt jako DataGridRow a pokud je null opuštíme událost
                var row = obj as DataGridRow;
                if (row == null) return;
                // pokud označené řádky obsahuje nalezený DataRow
                if (this.dgPoradiOU.SelectedItems.Contains(row.DataContext))
                {   //označuje korektní odchycení události a unmožnuje posun dat dále po trase událostí
                    e.Handled = true;
                    //zjistím jestli řádek na kterém byl spuštěn přesun je obsažen v předchozích vybraných řádcích, pokud ano přidám předchozí vybrané řádky jako vybrané a tedy součástí přesunu, je to z důvodu že pokud jsem klikl na nově vybraný řádek a ten rovnou přesouval, přidali se tam i původně vybrané, což je špatně
                    if (_vybraneRadky.Contains(dgPoradiOU.SelectedItems[0]))
                    {
                        //potom co jsem klikl a sputil přesun označil se pouze řádek na kterém jsem přesun zahájil, označím opět původně vybrané řádky jako selected
                        foreach (var radek in _vybraneRadky)
                        {
                            if (!dgPoradiOU.SelectedItems.Contains(radek))
                            {
                                dgPoradiOU.SelectedItems.Add(radek);
                            }
                        }
                    }
                    
                    // vytvoření objektu s přesouvanými datky
                    DataObject data = new DataObject();
                    //vložení dat plus jméno formátu v kterém jsou
                    data.SetData("myFormat", dgPoradiOU.SelectedItems);
                    //inicializuje drag-and-drop operaci
                    DragDrop.DoDragDrop((DependencyObject)e.Source, data, DragDropEffects.Move);
                }

            }
        }

        private void dgPoradiOU_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            //ukládá označené řádky do listu
            if (dgPoradiOU.SelectedItems != null && dgPoradiOU.SelectedItems.Count > 0)
            {
                _vybraneRadky.Clear();
                _vybraneRadky.AddRange(dgPoradiOU.SelectedItems.Cast<OU_ODVOZ_UKAZ>());
            }

        }

        private void dgPoradiOU_DragEnter(object sender, DragEventArgs e)
        {
            if (!e.Data.GetDataPresent("myFormat"))
                e.Effects = DragDropEffects.None;
        }

        private void dgPoradiOU_Drop(object sender, DragEventArgs e)
        {
            //vrací true poku jsou vkládaná data ve správném formátu
            if (e.Data.GetDataPresent("myFormat"))
            {
                //Pokud naleznu index řádku pro zvolený index
                int index = this.GetCurrentRowIndex(e.GetPosition);
                if (index < 0)
                    return;
                //Pro všechny přesouvané objekty, nalezńu jejich index a pokud to index umožňuje odeberu objekt s tímto indexem a opět ho vložím s novým indexem
                IList pousouvaneRadky = e.Data.GetData("myFormat") as IList;
                foreach (OU_ODVOZ_UKAZ row in pousouvaneRadky)
                {
                    
                    int itemIndex = dgPoradiOU.Items.IndexOf(row);
                    if (itemIndex < 0)
                        return;
                    
                    if (index == itemIndex)
                        return;

                    //NELZE použít poku požívám item source
                    //dgPoradiOU.Items.RemoveAt(itemIndex);
                    //dgPoradiOU.Items.Insert(index, row);

                    VybraneOU.RemoveAt(itemIndex);
                    VybraneOU.Insert(index, row);
                }

                dgPoradiOU.Items.Refresh();
            }
        }

        public delegate Point GetPosition(IInputElement element);

        /// <summary>
        /// Vrací index řádku podle pozice ukazatele
        /// </summary>
        /// <param name="pos"></param>
        /// <returns></returns>
        private int GetCurrentRowIndex(GetPosition pos)
        {
            int curIndex = -1;
            for (int i = 0; i < dgPoradiOU.Items.Count; i++)
            {
                DataGridRow itm = GetRowItem(i);
                
                if (itm != null && GetMouseTargetRow(itm, pos))
                {
                    curIndex = i;
                    break;
                }
            }

            
            return curIndex;
        }

        /// <summary>
        /// Zjistí zda se pozice ukazatele nachází na vybraném visuálním prvku
        /// </summary>
        /// <param name="theTarget"></param>
        /// <param name="position"></param>
        /// <returns></returns>
        private bool GetMouseTargetRow(Visual theTarget, GetPosition position)
        {
            Rect rect = VisualTreeHelper.GetDescendantBounds(theTarget);
            Point point = position((IInputElement)theTarget);
            return rect.Contains(point);
        }

        /// <summary>
        /// Vrací DataGridRow podle daného indexu, null pokud nebyl nalezen
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        private DataGridRow GetRowItem(int index)
        {
            if (dgPoradiOU.ItemContainerGenerator.Status != GeneratorStatus.ContainersGenerated)
                return null;
            
            return dgPoradiOU.ItemContainerGenerator.ContainerFromIndex(index)
                                                            as DataGridRow; ;
        }
        
        /// <summary>
        /// Scroluje automaticky datagridem pokud přetahuji objekt
        /// https://social.msdn.microsoft.com/Forums/vstudio/en-US/0c79958f-360a-459e-9aa6-486065d35df1/scrolling-while-dragging-over-a-datagrid?forum=wpf
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dgPoradiOURow_DragOver(object sender, DragEventArgs e)
        {
            DataGridRow r = (DataGridRow)sender;

            //Vyhledám ScrollViewer mého DataGridu
            DependencyObject d = dgPoradiOU;
            while (!(d is ScrollViewer))
            {
                d = VisualTreeHelper.GetChild(d, 0);
            }
            ScrollViewer scroll = d as ScrollViewer;

            Point position = dgPoradiOU.PointToScreen(Mouse.GetPosition(r));//získám pozici

            
            double topMargin = scroll.ActualHeight * .15;
            double bottomMargin = scroll.ActualHeight * .80;

            //Podle pozice pohybuji scrolem
            if (position.Y * -1 < topMargin)
                scroll.LineUp(); // <-- needs a mechanism to control the speed of the scroll.
            if (position.Y * -1 > bottomMargin)
                scroll.LineDown(); // <-- needs a mechanism to control the speed of the scroll.
        }
        #endregion Drag&Drop

        private void bHotovo_Click(object sender, RoutedEventArgs e)
        {
            this.DialogResult = true;
        }

       
    }
}
