﻿using GalaSoft.MvvmLight.CommandWpf;
using OuDot.Model;
using OuDot.Properties;
using OuDot.ServicesOudot;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;

namespace OuDot.ViewModel.Knihovnik
{
    public class VystupyDefiniceViewModel : ViewModelBase
    {
        private ULOHA_METOD_OBDOBI _obdobiMetod;

        public ULOHA_METOD_OBDOBI ObdobiMetod
        {
            get { return _obdobiMetod; }
            set
            {
                if (_obdobiMetod != value)
                {
                    _obdobiMetod = value;
                    OnPropertyChanged();
                }
            }
        }

        //ZOBRAZUJI VSECHNY OU
        //private ObservableCollection<OU_ODVOZ_UKAZ> _hotoveOU;

        //public ObservableCollection<OU_ODVOZ_UKAZ> HotoveOU

        //{
        //    get { return _hotoveOU; }
        //    set
        //    {
        //        if (_hotoveOU != value)
        //        {
        //            _hotoveOU = value;
        //            OnPropertyChanged();
        //        }
        //    }
        //}

        //public CollectionViewSource ViewSourceOU { get; set; }

        private bool _euVystup;

        public bool EuVystup

        {
            get { return _euVystup; }
            set
            {
                if (_euVystup != value)
                {
                    _euVystup = value;
                    OnPropertyChanged();
                }
            }
        }


        private string _nazevView;

        public string NazevView

        {
            get { return _nazevView; }
            set
            {
                if (_nazevView != value)
                {
                    _nazevView = value;
                    OnPropertyChanged();
                }
            }
        }


        public List<OU_ODVOZ_UKAZ> VybraneOU { get; set; }

        
        public RelayCommand VytvorVystupCommand { get; private set; }
        public RelayCommand<CheckBox> OznacVseVybraneCommand { get; private set; }

        public VystupyDefiniceViewModel()
        {
            if (DesignerProperties.GetIsInDesignMode(new System.Windows.DependencyObject()))
            {

                // Code runs in Blend --> create design time data.
            }
            else
            {
                // Code runs "for real"
                
                VytvorVystupCommand = new RelayCommand(() => VytvorVystup());
                OznacVseVybraneCommand = new RelayCommand<CheckBox>((o) => OznacVseVybrane(o));

                VybraneOU = new List<Model.OU_ODVOZ_UKAZ>();
                ObdobiMetod = DBContext.Instance.Entity.ULOHA_METOD_OBDOBI.Find(Settings.Default.IdMetodObdobi);
                //HotoveOU = new ObservableCollection<OU_ODVOZ_UKAZ>(ObdobiMetod.VsechnyOdvozUkaz.Where(o => o.STAV));
                NazevView = "ZP161_OU_doplnit_V";

            }
        }

        internal void VytvorVystup()
        {
            try
            {
                //Všeschny OU, které jdou do výstupu
                //var ouVystup = HotoveOU.Where(o => o.Vyber).OrderBy(o => o.GENERACE).ThenBy(o => o.PORADI_VYPOCTU).ToList();
                var ouVystup = ObdobiMetod.VsechnyOdvozUkaz.Where(o => o.Vyber).OrderBy(o => o.GENERACE).ThenBy(o => o.PORADI_VYPOCTU).ToList();
                if (ouVystup.Count <= 0)
                {
                    throw new InvalidOperationException("Nebyly vybrány žádné odvozené ukazatele.");
                }

                //Kontrola názvu vytvářeného výpisu
                if (NazevView == null || !NazevView.StartsWith(Validace.PrefixJmenaTabulky(ObdobiMetod) + "_", StringComparison.CurrentCultureIgnoreCase) || !NazevView.EndsWith("_V", StringComparison.CurrentCultureIgnoreCase))
                {
                    throw new FormatException($"Jméno výstupu musí začínate prefixem {Validace.PrefixJmenaTabulky(ObdobiMetod)} a končit \"_V\"");
                }
                string chybaValidace = Validace.JmenaKonvenceOracle(NazevView);
                if (chybaValidace != null)
                {
                    throw new FormatException(chybaValidace);
                }


                var tabulky = ouVystup.Select(o => o.OU_TAB).Distinct().ToList();//Tabulky OU, které jdou do výstupu
                //Kontrola obsahu nazvu EU pro EU výstup
                if (EuVystup)
                {
                    if (ouVystup.Any(o => string.IsNullOrEmpty(o.NAZEV_EU)))
                    {
                        throw new InvalidOperationException("Některý z vybrané odvozené ukazatelů neobsahují EU název, který je nutný pro EU výstup.");
                    }
                }
                //Kontrola stejných PK pro tabulky
                //Vyberu vsechny PK z jednotlivých tabulek seřadím je podle jmen a oddělím je čárkami
                var nazvyPK = tabulky.Select(t => string.Join(",", t.OU_PRIMARNI_KLICE.OrderBy(p => p.JMENO).Select(p => p.JMENO).ToList())).Distinct().ToList();
                if (nazvyPK.Count() != 1)//Pokud tabulky obsahují různé PK, ne pouze jeden stejný. Vyhodím chybu
                {
                    throw new InvalidOperationException("Vybrané odvozené ukazatele nejsou z tabulek se stejnými primárními klíči");
                }

                VystupyDefinicePoradiWindow vdpw = new Knihovnik.VystupyDefinicePoradiWindow(ouVystup);
                if (vdpw.ShowDialog() != true)
                {
                    throw new InvalidOperationException("Akce byla zrušena.");
                }
                ouVystup = vdpw.VybraneOU;
                

                //Ke každé tabulce přiřadím zástupný znak, podle jeho indexu vyberu písmenko z abecedy
                char[] alpha = "ABCDEFGHIJKLMNOPQRSTUVWXYZ".ToCharArray();
                Dictionary<char, OU_TAB> tabulkySlovnik = tabulky.ToDictionary(t => alpha[tabulky.IndexOf(t)], t => t);

                //Generování výstupu
                Services.VytvorOraView(NazevView, ouVystup, EuVystup, tabulkySlovnik, nazvyPK[0], ObdobiMetod);
                MessageBox.Show($"Vytvoření view s názvem {NazevView} proběho úspěšně.", "Vytvoření Ora View", MessageBoxButton.OK, MessageBoxImage.Information);
            }
            catch (Exception ex)
            {
                string zprava = string.Format("Nelze vytvořit výstup.{0}{0}Chyba: {1}", Environment.NewLine, ex.Message);
                MessageBox.Show(zprava, "Vytvoření Ora View", MessageBoxButton.OK, MessageBoxImage.Error);
            }
            

        }

        internal void OznacVseVybrane(CheckBox cbOznacVse)
        {
            var view = CollectionViewSource.GetDefaultView(ObdobiMetod.VsechnyOdvozUkaz);
            if (cbOznacVse.IsChecked == true)
            {
                foreach (OU_ODVOZ_UKAZ ou in view)
                {
                    ou.Vyber = true;
                }
            }
            else if (cbOznacVse.IsChecked == false)
            {
                foreach (OU_ODVOZ_UKAZ ou in view)
                {
                    ou.Vyber = false;
                }
            }
            view.Refresh();
           
        }

    }
}
