﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using OuDot.Model;
using OuDot.RozsireniGrafika;
using Xceed.Wpf.AvalonDock.Layout;
using OuDot.ViewModel.PomocnaOkna;

namespace OuDot.ViewModel
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            var vm = DataContext as MainViewModel;
            vm.OtvritNoveOkno += vm_OtvritNoveOkno;
            vm.ZavritOkno += vm_ZavritOkno;

            
        }

        void vm_ZavritOkno(object sender, EventArgs e)
        {
            try
            {
                //Zavři OU okno:
                OU_ODVOZ_UKAZ odvozUkaz = sender as OU_ODVOZ_UKAZ;
                if (odvozUkaz != null)
                    ZavriOUOkno(odvozUkaz);

                //Zavři Tabulku:
                OU_TAB VybTab = sender as OU_TAB;
                if (VybTab != null)
                {
                    ZavriTabulkaOkno(VybTab);
                    foreach (var ou in VybTab.OU_ODVOZ_UKAZ)
                    {
                        ZavriOUOkno(ou);
                    }
                }

                //Zavři všechna okna documentů
                if (sender == null)
                {
                    ZavriVsechnaDocOkna();
                }
            }
            catch (Exception ex)
            {
                string zprava = string.Format("Chyba nelze zavřít okno:{0}{1}", Environment.NewLine, ex.Message);
                //MessageBox.Show(zprava, "Uzavření okna", MessageBoxButton.OK, MessageBoxImage.Error);
                throw new Exception(zprava);
            }


        }

        /// <summary>
        /// Událost otevření nového okna
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void vm_OtvritNoveOkno(object sender, EventArgs e)
        {
            try
            {
                OtevriOknoPodleSendra(sender);
            }
            catch (Exception ex)
            {

                string zprava = string.Format("Nelze otevřít nové okno.{0}Chyba:{0}{0}{1}", Environment.NewLine, ex.Message);
                MessageBox.Show(zprava, "Otevření okna", MessageBoxButton.OK, MessageBoxImage.Error);
            }

        }

        /// <summary>
        /// Otvírá nové okno podle typu zasílaného objektu
        /// </summary>
        /// <param name="sender"></param>
        public void OtevriOknoPodleSendra(object sender)
        {
            //Otevrit OU okno:
            OU_ODVOZ_UKAZ odvozUkaz = sender as OU_ODVOZ_UKAZ; //ou na ktery bylo kliknuto
            if (odvozUkaz != null)
                Rozsireni.OtevriOUOkno(odvozUkaz, dmMyDockingManager);

            //Otevrit Tabulku:
            OU_TAB VybTab = sender as OU_TAB; //tabulka na kterou bylo kliknuto
            if (VybTab != null)
                OtevriTabulkaOkno(VybTab);

            //Otevrit Tabulku:
            OuDot.ServicesOudot.ObservableRangeCollection<OuDot.ServicesOudot.ValidacniChyba> chyby = sender as OuDot.ServicesOudot.ObservableRangeCollection<OuDot.ServicesOudot.ValidacniChyba>; //tabulka na kterou bylo kliknuto
            if (chyby != null)
                OtevriChybyOkno();
        }

        ///// <summary>
        ///// Doble click v menu na jméno tabulky
        ///// </summary>
        ///// <param name="sender"></param>
        ///// <param name="e"></param>
        //private void tblTabulka_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        //{
        //    try
        //    {
        //        e.Handled = true;//Zamezí rozvynutí treeview položky při dobleclicku

        //        if (e.ClickCount == 2)//jestli je to doble click
        //        {
        //            OU_TAB VybTab = (sender as TextBlock).DataContext as OU_TAB; //tabulka na kterou bylo kliknuto

        //            OtevriTabulkaOkno(VybTab);
        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        string zprava = string.Format("Nelze otevřít okno s definicí tabulky.{0}Chyba:{0}{0}{1}", Environment.NewLine, ex.Message);
        //        MessageBox.Show(zprava, "Okno tabulky", MessageBoxButton.OK, MessageBoxImage.Error);
        //    }

        //}

        //private void TextBlock_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        //{
        //    try
        //    {
        //        if (e.ClickCount == 2)//jestli je to doble click
        //        {
        //            OU_ODVOZ_UKAZ odvozUkaz = (sender as TextBlock).DataContext as OU_ODVOZ_UKAZ; //ou na ktery bylo kliknuto
        //            OtevriOUOkno(odvozUkaz);
        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        string zprava = string.Format("Nelze otevřít odvozený ukazatel.{0}Chyba:{0}{0}{1}", Environment.NewLine, ex.Message);
        //        MessageBox.Show(zprava, "Odvozený ukazatel", MessageBoxButton.OK, MessageBoxImage.Error);
        //    }

        //}

        private void dgOdvozUkaz_MouseDobleClick(object sender, RoutedEventArgs e)
        {
            try
            {
                    OU_ODVOZ_UKAZ odvozUkaz = (sender as DataGridRow).Item as OU_ODVOZ_UKAZ; //ou na ktery bylo kliknuto
                    Rozsireni.OtevriOUOkno(odvozUkaz, dmMyDockingManager);
            }
            catch (Exception ex)
            {
                string zprava = string.Format("Nelze otevřít odvozený ukazatel.{0}Chyba:{0}{0}{1}", Environment.NewLine, ex.Message);
                MessageBox.Show(zprava, "Odvozený ukazatel", MessageBoxButton.OK, MessageBoxImage.Error);
            }

        }

        public void ZavriTabulkaOkno(OU_TAB OUTab)
        {
            //Vyhledání okna s tabulkou
            LayoutContent OknoVybTab = this.dmMyDockingManager.Layout.Descendents().OfType<LayoutDocument>().FirstOrDefault(p => (p.Content.GetType() == typeof(TabulkaDetailsWindow)) && ((p.Content as TabulkaDetailsWindow).DataContext as TabulkaDetailsViewModel).Tabulka == OUTab);

            if (OknoVybTab != null)//Zavírám okno
            {
                OknoVybTab.Close();
            }

            foreach (var OUokno in OUTab.OU_ODVOZ_UKAZ)//Uzavře veškerá okna OU patřící do uzavírané tabulky
            {
                ZavriOUOkno(OUokno);
            }
        }

        public void ZavriOUOkno(OU_ODVOZ_UKAZ OdvozUkaz)
        {
            //Vyhledání okna s OU
            LayoutContent OknoVybOU = this.dmMyDockingManager.Layout.Descendents().OfType<LayoutDocument>().FirstOrDefault(p => (p.Content.GetType() == typeof(OdvozUkazDetailsWindow)) && ((p.Content as OdvozUkazDetailsWindow).DataContext as OdvozUkazDetailsViewModel).OdvozUkaz == OdvozUkaz);

            if (OknoVybOU != null)//Zavírám okno
            {
                OknoVybOU.Close();
            }

        }

        /// <summary>
        /// Uzavře všechna documentová okna
        /// </summary>
        public void ZavriVsechnaDocOkna()
        {
            var vsechnyLD = this.dmMyDockingManager.Layout.Descendents().OfType<LayoutDocument>().ToList();
            if (vsechnyLD != null)
            {
                foreach (var ld in vsechnyLD)
                {
                    ld.Close();
                }
            }
        }

        public void OtevriTabulkaOkno(OU_TAB OUTab)
        {
            
            //Vyhledání zda okno s tabulkou již existuje
            var vsechnyLD = this.dmMyDockingManager.Layout.Descendents().OfType<LayoutDocument>();
            LayoutContent OknoVybTab = vsechnyLD.FirstOrDefault(p => (p.Content.GetType() == typeof(TabulkaDetailsWindow)) && ((p.Content as TabulkaDetailsWindow).DataContext as TabulkaDetailsViewModel).Tabulka == OUTab);

            if (OknoVybTab == null)//Vytvářím nové okno
            {
                //Vytvořím a nastavím okno pro detail tabulky
                TabulkaDetailsWindow tdw = new TabulkaDetailsWindow(OUTab);
                //(tdw.DataContext as TabulkaDetailsViewModel).Tabulka = OUTab;
                //Vložím vytvořené okno jako LeyoutDocument a s aktivním ho
                LayoutDocument ld = new LayoutDocument { Content = tdw };

                //Nabinduji titul okna s názvem tabulky
                Binding titulBinding = new Binding();
                titulBinding.Source = tdw.DataContext;
                titulBinding.Path = new PropertyPath("Tabulka.JMENO");
                BindingOperations.SetBinding(ld, LayoutDocument.TitleProperty, titulBinding);

                //Nabinduji IsBusy property:
                Binding isBusy = new Binding("IsBusy");
                isBusy.Source = this.DataContext;
                //isBusy.Path = new PropertyPath();
                isBusy.Converter = new RozsireniGrafika.InverseBooleanConverter();
                BindingOperations.SetBinding(tdw, TabulkaDetailsWindow.IsEnabledProperty, isBusy);

                //Vyhledání LayoutDocumentPane
                LayoutDocumentPane LDP = this.dmMyDockingManager.Layout.Descendents().OfType<LayoutDocumentPane>().FirstOrDefault();
                LDP.Children.Add(ld);
                ld.IsActive = true;
            }
            else//aktivuji již existující
            {
                OknoVybTab.IsActive = true;
            }
        }

        

        public void OtevriChybyOkno()
        {
            this.laChyby.IsVisible = true;

            ////Vyhledání zda okno s ou již existuje
            //var spodniDock = this.dmMyDockingManager.Layout.BottomSide.Children.FirstOrDefault();
            //if (spodniDock == null)
            //{
            //    spodniDock = new LayoutAnchorGroup();
            //    dmMyDockingManager.Layout.BottomSide.Children.Add(spodniDock);
            //}
            ////LayoutContent oknoChyby = LDP.Children.FirstOrDefault(p => p.Content.GetType() == typeof(ChybyUserControl));
            //var oknoChyby = spodniDock.Children.FirstOrDefault(p => p.Content.GetType() == typeof(ChybyUserControl));

            //if (oknoChyby == null)//Vytvářím nové okno
            //{
            //    //Vytvořím a nastavím okno pro detaili tabulky
            //    //OdvozUkazDetailsWindow oudw = new OdvozUkazDetailsWindow(OdvozUkaz);
            //    ChybyUserControl chuc = new ChybyUserControl();

            //    //Vložím vytvořené okno jako LeyoutDocument a s aktivním ho
            //    LayoutAnchorable la = new LayoutAnchorable
            //    {
            //        Content = chuc,
            //        Title = "Chyby"
            //    };

            //    spodniDock.Children.Add(la);

            //    la.IsActive = true;

            //    la.ToggleAutoHide();//Zakotví okno musí následovat až po aktivaci okna

            //    //nastaví výšku okna musí následovat až po zakotvení jinak není možné získat LayoutAnchorablePane 
            //    var root = (LayoutAnchorablePane)la.Parent;
            //    root.DockHeight = new GridLength(100);

            //}
            //else//aktivuji již existující
            //{
            //    oknoChyby.IsActive = true;
            //}
        }

        private void laChyby_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            this.laChyby.IsVisible = false;
            e.Cancel = true;
        }


        /// <summary>
        /// Hack na upevnění okna s chybou
        /// http://stackoverflow.com/questions/17972491/how-to-create-a-bottom-panel-without-auto-hide-using-avalondock-2-0
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dmMyDockingManager_Loaded(object sender, RoutedEventArgs e)
        {
            this.laChyby.ToggleAutoHide();//Zakotví okno musí následovat až po aktivaci okna

            //nastaví výšku okna musí následovat až po zakotvení jinak není možné získat LayoutAnchorablePane 
            var root = (LayoutAnchorablePane)this.laChyby.Parent;
            root.DockHeight = new GridLength(150);


            this.laChyby.IsVisible = false;
            //System.Collections.IEnumerable
            //Xceed.Wpf.AvalonDock.DockingManager.DocumentsSourceProperty


        }
        /// <summary>
        /// Kliknutí na otevři okno s chybami v menu
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItem_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.OtevriChybyOkno();
            }
            catch (Exception ex)
            {
                string zprava = string.Format("Nelze otevřít okno s chybami.{0}Chyba:{0}{0}{1}", Environment.NewLine, ex.Message);
                MessageBox.Show(zprava, "Validační chyby", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

       /// <summary>
       /// Kliknutí na tlačítko Definice u tabulky
       /// </summary>
       /// <param name="sender"></param>
       /// <param name="e"></param>
        private void bDefiniceTabulky_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                OU_TAB VybTab = cbTabulky.SelectedItem as OU_TAB; //vybraná tabulka
                OtevriTabulkaOkno(VybTab);
            }
            catch (Exception ex)
            {
                string zprava = string.Format("Nelze otevřít okno s definicí tabulky.{0}Chyba:{0}{0}{1}", Environment.NewLine, ex.Message);
                MessageBox.Show(zprava, "Okno tabulky", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        /// <summary>
        /// Událost na tlačítko přidej pro všechny typy zdrojů
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bPriradZdroj_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                var vsechnyLD = this.dmMyDockingManager.Layout.Descendents().OfType<LayoutDocument>();
                var OknaVybOU = vsechnyLD.Where(p => (p.Content.GetType() == typeof(OdvozUkazDetailsWindow)) && (p.IsSelected));
                LayoutContent OknoVybOU = OknaVybOU.FirstOrDefault();
                if (OknaVybOU.Count() > 1)
                {
                    //Vyselektuju kolekci aktivních OU
                    var aktivniOU = OknaVybOU.Select(o => ((o.Content as OdvozUkazDetailsWindow).DataContext as OdvozUkazDetailsViewModel).OdvozUkaz).ToList();
                    //Zobrazím je uživateli k výběru jednoho
                    VyberOUWindow vouw = new PomocnaOkna.VyberOUWindow(aktivniOU);
                    var result = vouw.ShowDialog();
                    if (result != true)
                    {
                        throw new OperationCanceledException("Přiřazení zrušeno.");
                    }
                    //Nastavým okno s vybraným OU
                    OU_ODVOZ_UKAZ vybranyOU = vouw.dgAktivniOU.SelectedItem as OU_ODVOZ_UKAZ;
                    OknoVybOU = OknaVybOU.FirstOrDefault(o => ((o.Content as OdvozUkazDetailsWindow).DataContext as OdvozUkazDetailsViewModel).OdvozUkaz == vybranyOU);
                }
                

                if (OknoVybOU != null)
                {
                    OdvozUkazDetailsViewModel oudvm = (OknoVybOU.Content as OdvozUkazDetailsWindow).DataContext as OdvozUkazDetailsViewModel;
                    oudvm.PridejZdroj((sender as Button).DataContext);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Zdroj nelze přiřadit. Chyba: {ex.Message}", "Přiřaď zdroj", MessageBoxButton.OK, MessageBoxImage.Error);
            }
            
        }

        /// <summary>
        /// Událost MouseDoubleClick pro všechny typy zdrojů, otevře okno s příslušným zdrojem
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dgZdrojeVyber_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            try
            {
                BDBC_SLOUPCE sloupec = (sender as DataGridRow).Item as BDBC_SLOUPCE;
                if (sloupec != null)
                {
                    Rozsireni.ZobrazCapiMetaData(sloupec, this.dmMyDockingManager);
                }


                OU_ODVOZ_UKAZ ou = (sender as DataGridRow).Item as OU_ODVOZ_UKAZ;
                if (ou != null)
                    Rozsireni.OtevriOUOkno(ou, this.dmMyDockingManager);
            }
            catch (Exception ex)
            {
                string zprava = string.Format("Nelze otevřít detail zdroje.{0}Chyba:{0}{0}{1}", Environment.NewLine, ex.Message);
                MessageBox.Show(zprava, "Vyběr zdroje", MessageBoxButton.OK, MessageBoxImage.Error);
            }



        }
    }
}
