﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using GalaSoft.MvvmLight.CommandWpf;
using OuDot.Model;
using OuDot.Properties;
using OuDot.ServicesOudot;

namespace OuDot.ViewModel
{
    public class NastaveniViewModel : ViewModelBase
    {

        private ObservableCollection<ULOHA> _ulohyTeren;

        public ObservableCollection<ULOHA> UlohyTeren
        {
            get { return _ulohyTeren; }
            set
            {
                if (_ulohyTeren != value)
                {
                    _ulohyTeren = value;
                    OnPropertyChanged();
                }
            }
        }


        private ULOHA _ulohaTeren;

        public ULOHA UlohaTeren
        {
            get { return _ulohaTeren; }
            set
            {
                if (_ulohaTeren != value)
                {
                    _ulohaTeren = value;
                    OnPropertyChanged();
                }
            }
        }


        private ULOHA_OBDOBI _obdobiTeren;

        public ULOHA_OBDOBI ObdobiTeren
        {
            get { return _obdobiTeren; }
            set
            {
                if (_obdobiTeren != value)
                {
                    _obdobiTeren = value;
                    OnPropertyChanged();
                }
            }
        }



        private ObservableCollection<ULOHA_METOD> _ulohyMetod;

        public ObservableCollection<ULOHA_METOD> UlohyMetod
        {
            get { return _ulohyMetod; }
            set
            {
                if (_ulohyMetod != value)
                {
                    _ulohyMetod = value;
                    OnPropertyChanged();
                }
            }
        }


        private ULOHA_METOD _ulohaMetod;

        public ULOHA_METOD UlohaMetod
        {
            get { return _ulohaMetod; }
            set
            {
                if (_ulohaMetod != value)
                {
                    _ulohaMetod = value;
                    OnPropertyChanged();
                }
            }
        }


        private ULOHA_METOD_OBDOBI _obdobiMetod;

        public ULOHA_METOD_OBDOBI ObdobiMetod
        {
            get { return _obdobiMetod; }
            set
            {
                if (_obdobiMetod != value)
                {
                    _obdobiMetod = value;
                    OnPropertyChanged();
                }
            }
        }


        private ULOHA_METOD_OBDOBI _obdobiMetodPrevod;

        public ULOHA_METOD_OBDOBI ObdobiMetodPrevod
        {
            get { return _obdobiMetodPrevod; }
            set
            {
                if (_obdobiMetodPrevod != value)
                {
                    _obdobiMetodPrevod = value;
                    OnPropertyChanged();
                }
            }
        }


        public RelayCommand SaveCommand { get; private set; }
        public RelayCommand PrevodDefinicCommand { get; private set; }

        public NastaveniViewModel()
        {
            if (DesignerProperties.GetIsInDesignMode(new System.Windows.DependencyObject()))
            {

                // Code runs in Blend --> create design time data.





            }
            else
            {
                // Code runs "for real"
                UlohyTeren = new ObservableCollection<ULOHA>(DBContext.Instance.Entity.ULOHA);
                ObdobiTeren = DBContext.Instance.Entity.ULOHA_OBDOBI.Find(Settings.Default.IdTerenUlohaObd);
                if (ObdobiTeren != null)
                    UlohaTeren = ObdobiTeren.ULOHA;

                UlohyMetod = new ObservableCollection<ULOHA_METOD>(DBContext.Instance.Entity.ULOHA_METOD);
                ObdobiMetod = DBContext.Instance.Entity.ULOHA_METOD_OBDOBI.Find(Settings.Default.IdMetodObdobi);
                if (ObdobiMetod != null)
                    UlohaMetod = ObdobiMetod.ULOHA_METOD;

                SaveCommand = new RelayCommand(() => Save());
                PrevodDefinicCommand = new RelayCommand(() => PrevodDefinic());

            }



        }

        private void PrevodDefinic()
        {
            try
            {
                //Ošetření:
                if (ObdobiMetod == null)
                {
                    throw new ArgumentNullException("Není vybrané žádné období pro metodickou úlohu.");
                }
                if (ObdobiMetodPrevod == null)
                {
                    throw new ArgumentNullException("Není vybrané žádné nové období pro převod.");
                }
                if (ObdobiMetod == ObdobiMetodPrevod)
                {
                    throw new InvalidOperationException("Převod do stejného období.");
                }
                if (ObdobiMetodPrevod.OU_TAB.Count > 0)
                {
                    throw new NotImplementedException("Převod pouze do prázdného období.");
                }


                int? maxGenerace = ObdobiMetod.OU_TAB.SelectMany(t => t.OU_ODVOZ_UKAZ).Max(o => o.GENERACE);//Nejvyšší generace
                string novyPrefixObdobi = Validace.PrefixJmenaTabulky(ObdobiMetodPrevod);//získám nový prefix období
                string staryPrefixObdobi = Validace.PrefixJmenaTabulky(ObdobiMetod);//získám starý prefix období
                string novyPrefixObdTeren;
                string staryPrefixObdTeren;

                for (int g = 0; g <= maxGenerace; g++)
                {
                    //seřazení tabulkek podle pořadí:
                    var tabSerazene = ObdobiMetod.OU_TAB.OrderBy(t => t.PORADI);
                    foreach (OU_TAB tabVzor in tabSerazene)
                    {
                        //Nastavým si období terení pro klonovanou tabulku
                        var obdTeren = tabVzor.BDBCTABULKY.ULOHA_OBDOBI.ULOHA.ULOHA_OBDOBI.Where(o => o.OBDOBI == ObdobiMetodPrevod.OBDOBI).FirstOrDefault();
                        novyPrefixObdTeren = Validace.PrefixJmenaTabulky(obdTeren);
                        staryPrefixObdTeren = Validace.PrefixJmenaTabulky(tabVzor.BDBCTABULKY.ULOHA_OBDOBI);

                        if (!ObdobiMetodPrevod.OU_TAB.Any(t => t.JMENO.Substring(5) == tabVzor.JMENO.Substring(5)))//klonuju tabulku pouze jednou, NE pro každou generaci
                        {
                            //Klonuje pouze samotný objekt ne navazane vazby na jiné objekty nebo kolekce
                            OU_TAB tabKlon = (OU_TAB)DBContext.Instance.Entity.Entry(tabVzor).CurrentValues.ToObject();

                            tabKlon.SQL_OPRAVA_DAT = null;//Vymažu opravný kód
                            tabKlon.ULOHA_METOD_OBDOBI = ObdobiMetodPrevod;//Změna na nové období
                            tabKlon.JMENO = Regex.Replace(tabKlon.JMENO, staryPrefixObdobi, novyPrefixObdobi, RegexOptions.IgnoreCase); //Změní starý prefix tabulky za nový
                            //Vyhledam navázanou BDBCTabulku pro nove Tereni obdobi a vlozim ji k nakolonovane tabulce
                            tabKlon.BDBCTABULKY = obdTeren.BDBCTABULKY.Where(t => t.JMENO.Substring(5) == tabVzor.BDBCTABULKY.JMENO.Substring(5)).FirstOrDefault();

                            //Nastaveni PK podle nove tabulky BDBC
                            //Seřadit podel pořadí
                            var serazenePK = tabKlon.BDBCTABULKY.BDBC_PRIMARNI_KLICE.OrderBy(p => p.PORADI);
                            foreach (BDBC_PRIMARNI_KLICE bdbcPK in serazenePK)
                            {
                                OU_PRIMARNI_KLICE newPK = new OU_PRIMARNI_KLICE();
                                newPK.VlozNovyPK(bdbcPK.JMENO, bdbcPK.PORADI, tabKlon);
                            }

                            //SQL kód zamění prefixy zdrojových tabulek na nové období
                            tabKlon.SQL_GENEROVANI_PK = Regex.Replace(tabKlon.SQL_GENEROVANI_PK, staryPrefixObdTeren, novyPrefixObdTeren, RegexOptions.IgnoreCase);//záměna teréních úloh
                            tabKlon.SQL_GENEROVANI_PK = Regex.Replace(tabKlon.SQL_GENEROVANI_PK, staryPrefixObdobi, novyPrefixObdobi, RegexOptions.IgnoreCase);//záměna metodických úloh

                            //Vložím nakolonovanou tabulku do kontextu
                            DBContext.Instance.Entity.OU_TAB.Add(tabKlon);

                        }

                        //KLONOVÁNÍ OU:
                        //Sařadím OU v dané generaci podle pořadí výpočtu tabulek a pořadí výpočtu OU v dané generaci
                        var serazeneOU = tabVzor.OU_ODVOZ_UKAZ.Where(o => o.GENERACE == g).OrderBy(t => t.OU_TAB.PORADI).ThenBy(o => o.PORADI_VYPOCTU);

                        foreach (OU_ODVOZ_UKAZ ouVzorovy in serazeneOU)
                        {
                            //Definice klonování:

                            //Klonuje pouze samotný objekt ne navazane vazby na jiné objekty nebo kolekce
                            OU_ODVOZ_UKAZ ouKlon = (OU_ODVOZ_UKAZ)DBContext.Instance.Entity.Entry(ouVzorovy).CurrentValues.ToObject();

                            ouKlon.OPRAVNY_KOD = null;//Vymažu opravný kód
                            //Naleznu klonovanou tabulku podle pořadí (jména nesouhlasí kvůli prefixu období)
                            var tabKlon_vyhled = ObdobiMetodPrevod.OU_TAB.Where(t => t.PORADI == ouVzorovy.OU_TAB.PORADI).FirstOrDefault();
                            if (tabKlon_vyhled == null)
                            {
                                throw new MissingMemberException($"Nebyla nalezena tabulka {ouVzorovy.OU_TAB.JMENO}.");
                            }
                            ouKlon.OU_TAB = tabKlon_vyhled;//Vložím do tabulky

                            //2.Klonování kategorii a překódování:
                            foreach (var vzorovaCat in ouVzorovy.OU_CATEGORY)
                            {
                                OU_CATEGORY cloneCat = (OU_CATEGORY)DBContext.Instance.Entity.Entry(vzorovaCat).CurrentValues.ToObject();
                                cloneCat.OU_ODVOZ_UKAZ = ouKlon;
                                DBContext.Instance.Entity.OU_CATEGORY.Add(cloneCat);

                                //Klonování překódování
                                foreach (var vzorPrekod in vzorovaCat.OU_PREKODOVANI)
                                {
                                    OU_PREKODOVANI clonePrekod = (OU_PREKODOVANI)DBContext.Instance.Entity.Entry(vzorPrekod).CurrentValues.ToObject();
                                    clonePrekod.OU_CATEGORY = cloneCat;
                                    DBContext.Instance.Entity.OU_PREKODOVANI.Add(clonePrekod);
                                }
                            }

                            //3.Klonování všech typů zdrojů:
                            bool chybaNalezeniZdroje = false;
                            foreach (var bdbc_sloupec in ouVzorovy.BDBC_SLOUPCE)//Přiřazení BDBC_SLOUPCE
                            {
                                //Pokud je zdroj z jiného období než převáděného, nepřevádím ho
                                if (bdbc_sloupec.BDBCPRIRAZENE_FIELDY.BDBCTABULKY.ULOHA_OBDOBI.OBDOBI.ToUpper() != ObdobiMetod.OBDOBI.ToUpper())
                                {
                                    chybaNalezeniZdroje = true;
                                    ouKlon.POZNAMKA = ouKlon.POZNAMKA + Environment.NewLine +
                                        $"Zdroj {bdbc_sloupec.JMENO} z tabulky {bdbc_sloupec.BDBCPRIRAZENE_FIELDY.BDBCTABULKY.JMENO} pochází z jiného období než převáděného {ObdobiMetod.OBDOBI}. Nelze vyhledat správný zdroj pro nové období {ObdobiMetodPrevod.OBDOBI}." + Environment.NewLine;
                                    continue;
                                }

                                //Vyhledam v novém období podle jmena tabulky, fieldu a názvu sloupce, nový zdrojový bdbc_sloupec
                                var novBdbcSloupec = obdTeren.BDBCTABULKY.Where(t => t.JMENO.Substring(5).ToUpper() == bdbc_sloupec.BDBCPRIRAZENE_FIELDY.BDBCTABULKY.JMENO.Substring(5).ToUpper()).FirstOrDefault()?. //Z názvů tabulek musím odstanit prefix obsahující období
                                                  BDBCPRIRAZENE_FIELDY.Where(f => f.KRATKE_JMENO.ToUpper() == bdbc_sloupec.BDBCPRIRAZENE_FIELDY.KRATKE_JMENO.ToUpper()).FirstOrDefault()?.
                                                  BDBC_SLOUPCE.Where(s => s.JMENO.ToUpper() == bdbc_sloupec.JMENO.ToUpper()).FirstOrDefault();

                                //Pokud zdroj najdu přiřadím ho k novému OU, pokud ne nastavím chybu
                                if (novBdbcSloupec != null)
                                {
                                    ouKlon.BDBC_SLOUPCE.Add(novBdbcSloupec);
                                }
                                else
                                {
                                    chybaNalezeniZdroje = true;
                                    ouKlon.POZNAMKA = ouKlon.POZNAMKA + Environment.NewLine +
                                        $"Zdroj {bdbc_sloupec.JMENO} z tabulky {bdbc_sloupec.BDBCPRIRAZENE_FIELDY.BDBCTABULKY.JMENO} nebyl nalezen v novém obdob {ObdobiMetodPrevod.OBDOBI}." + Environment.NewLine;
                                }
                                
                            }
                            foreach (var old_bdbc_pk in ouVzorovy.BDBC_PRIMARNI_KLICE)//Přiřazení BDBC_PRIMARNI_KLICE
                            {
                                //Pokud je zdroj z jiného období než převáděného, nepřevádím ho
                                if (old_bdbc_pk.BDBCTABULKY.ULOHA_OBDOBI.OBDOBI.ToUpper() != ObdobiMetod.OBDOBI.ToUpper())
                                {
                                    chybaNalezeniZdroje = true;
                                    ouKlon.POZNAMKA = ouKlon.POZNAMKA + Environment.NewLine +
                                        $"Zdroj {old_bdbc_pk.JMENO} z tabulky {old_bdbc_pk.BDBCTABULKY.JMENO} pochází z jiného období než převáděného {ObdobiMetod.OBDOBI}. Nelze vyhledat správný zdroj pro nové období {ObdobiMetodPrevod.OBDOBI}." + Environment.NewLine;
                                    continue;
                                }

                                //Najdu primární klíč z bdbc tabulek v novém období podle, jména tabulky do které patří a jeho názvu
                                var nov_bdbc_pk = obdTeren.BDBCTABULKY.Where(t => t.JMENO.Substring(5).ToUpper() == old_bdbc_pk.BDBCTABULKY.JMENO.Substring(5).ToUpper()).FirstOrDefault()?. //Z názvů tabulek musím odstanit prefix obsahující období
                                                    BDBC_PRIMARNI_KLICE.Where(k => k.JMENO.ToUpper() == old_bdbc_pk.JMENO.ToUpper()).FirstOrDefault();

                                //Pokud zdroj najdu přiřadím ho k novému OU, pokud ne nastavím chybu
                                if (nov_bdbc_pk != null)
                                {
                                    ouKlon.BDBC_PRIMARNI_KLICE.Add(nov_bdbc_pk);
                                }
                                else
                                {
                                    chybaNalezeniZdroje = true;
                                    ouKlon.POZNAMKA = ouKlon.POZNAMKA + Environment.NewLine +
                                        $"Zdroj {old_bdbc_pk.JMENO} z tabulky {old_bdbc_pk.BDBCTABULKY.JMENO} nebyl nalezen v novém obdob {ObdobiMetodPrevod.OBDOBI}." + Environment.NewLine;
                                }
                                    

                            }
                            foreach (var zdroj_ou in ouVzorovy.OU_ODVOZ_UKAZ1)//Přiřazení zdrojových OU
                            {
                                //Pokud je zdroj z jiného období než převáděného, nepřevádím ho
                                if (zdroj_ou.OU_TAB.ULOHA_METOD_OBDOBI != ObdobiMetod)
                                {
                                    chybaNalezeniZdroje = true;
                                    ouKlon.POZNAMKA = ouKlon.POZNAMKA + Environment.NewLine +
                                        $"Zdrojový OU {zdroj_ou.NAZEV} z tabulky {zdroj_ou.OU_TAB.JMENO} pochází z jiného období než převáděného {ObdobiMetod.OBDOBI}. Nelze vyhledat správný zdroj pro nové období {ObdobiMetodPrevod.OBDOBI}." + Environment.NewLine;
                                    continue;
                                }

                                //Vyhledám zdroj v novém období
                                var nov_zdroj_ou = ObdobiMetodPrevod.OU_TAB.Where(t => t.PORADI == zdroj_ou.OU_TAB.PORADI).FirstOrDefault()?. //Vyhledám podle pořadí tabulky (jméno obsahuje prefix s obdobím)
                                                    OU_ODVOZ_UKAZ.Where(o => o.NAZEV.ToUpper() == zdroj_ou.NAZEV.ToUpper()).FirstOrDefault();

                                //Pokud zdroj najdu přiřadím ho k novému OU, pokud ne nastavím chybu
                                if (nov_zdroj_ou != null)
                                {
                                    ouKlon.OU_ODVOZ_UKAZ1.Add(nov_zdroj_ou);
                                }
                                else
                                {
                                    chybaNalezeniZdroje = true;
                                    ouKlon.POZNAMKA = ouKlon.POZNAMKA + Environment.NewLine +
                                        $"Zdrojový OU {zdroj_ou.NAZEV} z tabulky {zdroj_ou.OU_TAB.JMENO} nebyl nalezen v novém obdob {ObdobiMetodPrevod.OBDOBI}." + Environment.NewLine;
                                }
                                  
                               
                            }

                            foreach (var old_ext in ouVzorovy.EXT_SLOUPCE)//Přiřazení EXT_SLOUPCE
                            {
                                //Pokud externí zdroj stále platí vložím ho pokud ne nastavím chybu. Řídím se podle data spuštění klonování
                                if (old_ext.EXT_TABULKY.PLATNOST_DO == null || (old_ext.EXT_TABULKY.PLATNOST_OD <= DateTime.Today &&  DateTime.Today <= old_ext.EXT_TABULKY.PLATNOST_DO))
                                {
                                    ouKlon.EXT_SLOUPCE.Add(old_ext);
                                }
                                else
                                {
                                    chybaNalezeniZdroje = true;
                                    ouKlon.POZNAMKA = ouKlon.POZNAMKA + Environment.NewLine +
                                        $"Externí zdroj {old_ext.JMENO} z tabulky {old_ext.EXT_TABULKY.JMENO_ID} není platný v čase převodu. Platnost externího zdroje je nastavena od {old_ext.EXT_TABULKY.PLATNOST_OD} do {old_ext.EXT_TABULKY.PLATNOST_DO}" + Environment.NewLine;
                                    
                                }
                                
                            }

                            foreach (var cisel in ouVzorovy.CISEL_SEZNAM)
                            {
                                throw new NotImplementedException("Převedení číselníkohového zdroje není implementováno");
                            }

                            //Pokud je funkce vlastní zapíšu do poznámky
                            if (ouVzorovy.ID_FUNKCE == 1)
                            {
                                ouKlon.POZNAMKA = ouKlon.POZNAMKA + Environment.NewLine +
                                        $"Odvozený ukazatel používá funkci Vlastní. Je potřeba zkontrolovat SQL pro nové období {ObdobiMetodPrevod.OBDOBI}." + Environment.NewLine;
                            }

                            //Pokud je OU nastaven na funkci Vlastní (id=1) nebo nastala chyba při hledání zdroje.
                            if (ouVzorovy.ID_FUNKCE == 1 || chybaNalezeniZdroje == true)
                            {
                                ouKlon.ID_FUNKCE = 1;//Nastavím funkci na Vlastní
                                ouKlon.STAV = false;//stav nového OU nastavím na Nehotový
                                if (ouVzorovy.SQL_DOTAZ != null)//Pokud má vyplněný nějaký SQL kód převedu prefixy období na nové
                                {
                                    ouKlon.SQL_DOTAZ = Regex.Replace(ouKlon.SQL_DOTAZ, staryPrefixObdTeren, novyPrefixObdTeren, RegexOptions.IgnoreCase);//záměna teréních úloh
                                    ouKlon.SQL_DOTAZ = Regex.Replace(ouKlon.SQL_DOTAZ, staryPrefixObdobi, novyPrefixObdobi, RegexOptions.IgnoreCase);//záměna metodických úloh
                                }

                            }
                            else//Pokud je vyplněna jiná funkce než vlastní a všechny zdroje byli nalezeny
                            {
                                //Nejdříve převedu parametry funkcí
                                foreach (var pramVzor in ouVzorovy.OU_PARAM_FCE)
                                {
                                    OU_PARAM_FCE cloneParam = (OU_PARAM_FCE)DBContext.Instance.Entity.Entry(pramVzor).CurrentValues.ToObject();
                                    cloneParam.OU_ODVOZ_UKAZ = ouKlon;
                                    DBContext.Instance.Entity.OU_PARAM_FCE.Add(cloneParam);
                                }
                                //Pokusím se vygenerovat nový kód pro vybranou funkci
                                try
                                {
                                    ouKlon.SQL_DOTAZ = GenerovaniSQL.GenerujSQL(ouKlon, ouVzorovy.OU_FUNKCE);//ouKlon.ZmenaFunkce(ouVzorovy.OU_FUNKCE);
                                    ouKlon.STAV = true;
                                }
                                catch (Exception ex)
                                {
                                    //Pokud se to nepodaří nastavím funkci na Vlastní a Stav na Nehotový
                                    ouKlon.ID_FUNKCE = 1;
                                    ouKlon.STAV = false;//stav nového OU nastavím flase
                                    //Vymažu parametry funkcí
                                    DBContext.Instance.Entity.OU_PARAM_FCE.RemoveRange(ouKlon.OU_PARAM_FCE);

                                    ouKlon.POZNAMKA = ouKlon.POZNAMKA + Environment.NewLine +
                                       $"Nezdařilo se vygenerovat funkci {ouVzorovy.OU_FUNKCE.NAZEV}. Nastala chyba {ex.Message}" + Environment.NewLine;
                                }

                            }

                            DBContext.Instance.Entity.OU_ODVOZ_UKAZ.Add(ouKlon);


                        }
                    }
                }
            }
            catch (Exception ex)
            {
                string zprava = string.Format("Nelze převést definice.{0}{0}Chyba:    {1}", Environment.NewLine, ex.Message);
                MessageBox.Show(zprava, "Nastavení", MessageBoxButton.OK, MessageBoxImage.Error);

                //Zruším všechny neuložené změny
                DBContext.Instance.RejectChanges();
            }
        }

        internal void Save()
        {
            try
            {
                if (ObdobiTeren == null || ObdobiMetod == null)
                {
                    throw new ArgumentNullException("Prázdná úloha, období.");
                }

                Settings.Default.IdTerenUlohaObd = ObdobiTeren.ID;
                Settings.Default.IdMetodObdobi = ObdobiMetod.ID;
                Settings.Default.Save();
            }
            catch (Exception ex)
            {
                string zprava = string.Format("Změny nebyly uloženy.{0}{0}Chyba:    {1}", Environment.NewLine, ex.Message);
                MessageBox.Show(zprava, "Nastavení", MessageBoxButton.OK, MessageBoxImage.Error);
            }

        }
    }
}
