﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using GalaSoft.MvvmLight.CommandWpf;
using OuDot.Model;
using OuDot.ServicesOudot;

namespace OuDot.ViewModel
{
    public class PrekodovaniViewModel : ViewModelBase
    {

        private OU_CATEGORY _kategorie;

        public OU_CATEGORY Kategorie
        {
            get { return _kategorie; }
            set
            {
                if (_kategorie != value)
                {
                    _kategorie = value;
                    OnPropertyChanged();
                }
            }
        }


        private TypPrekodovani _druhPrekodovani;

        public TypPrekodovani DruhPrekodovani
        {
            get { return _druhPrekodovani; }
            set
            {
                if (_druhPrekodovani != value)
                {
                    _druhPrekodovani = value;
                    OnPropertyChanged();
                }
            }
        }


        private decimal _od;

        public decimal Od
        {
            get { return _od; }
            set
            {
                if (_od != value)
                {
                    _od = value;
                    OnPropertyChanged();
                }
            }
        }


        private decimal _do;

        public decimal Do
        {
            get { return _do; }
            set
            {
                if (_do != value)
                {
                    _do = value;
                    OnPropertyChanged();
                }
            }
        }


        private FieldStatus _status;

        public FieldStatus Status
        {
            get { return _status; }
            set
            {
                if (_status != value)
                {
                    _status = value;
                    OnPropertyChanged();
                }
            }
        }


        public RelayCommand VlozitCommand { get; private set; }
        public RelayCommand<object> OdeberCommand { get; private set; }

        public PrekodovaniViewModel(OU_CATEGORY NoveKategorie)
        {
            if (!DesignerProperties.GetIsInDesignMode(new System.Windows.DependencyObject()))
            {
                // Code runs in Blend --> create design time data.
                Kategorie = NoveKategorie;

                
            }
            else
            {
                // Code runs "for real"
            }

            VlozitCommand = new RelayCommand(() => Vlozit());
            OdeberCommand = new RelayCommand<object>((p) => Odeber(p));
        }

        public void Odeber(object PrekodovaniSmazat)
        {
            try
            {
                OU_PREKODOVANI prekodSmazat = PrekodovaniSmazat as OU_PREKODOVANI;
                DBContext.Instance.Entity.OU_PREKODOVANI.Remove(prekodSmazat);
            }
            catch (Exception ex)
            {
                string zprava = string.Format("Chyba:{0}{1}", Environment.NewLine, ex.Message);
                MessageBox.Show(zprava, "Smazat překódování", MessageBoxButton.OK, MessageBoxImage.Error);
            }  
            
        }

        public void Vlozit()
        {
            try
            {
                KontrolaNovehoPrekodovani(DruhPrekodovani);

                switch (DruhPrekodovani)
                {
                    case TypPrekodovani.NeRovno:
                    case TypPrekodovani.Vetsi:
                    case TypPrekodovani.Mensi:
                    case TypPrekodovani.Konstanta://Pokud je druh Konstanta mohu zadávat pouze hodnotu Od
                        Kategorie.VlozitPrekodovani(DruhPrekodovani, Od, null, null);
                        break;
                    case TypPrekodovani.Interval://Pokud je druh Interval mohu zadávat Od i Do
                        Kategorie.VlozitPrekodovani(DruhPrekodovani, Od, Do, null);
                        break;
                    case TypPrekodovani.Status://Pokud je druh Status mohu zadávat pouze hodnotu Status
                        Kategorie.VlozitPrekodovani(DruhPrekodovani, null, null, Status);
                        break;
                    case TypPrekodovani.Null:
                    case TypPrekodovani.NeNull:
                        Kategorie.VlozitPrekodovani(DruhPrekodovani, null, null, null);
                        break;
                    default:
                        throw new Exception("Neznámí typ překódování");
                }
            }
            catch (Exception ex)
            {
                string zprava = string.Format("Chyba:{0}{1}", Environment.NewLine, ex.Message);
                MessageBox.Show(zprava, "Vlozit překódování", MessageBoxButton.OK, MessageBoxImage.Error);
            }
            
        }

        public void KontrolaNovehoPrekodovani(TypPrekodovani ZpusobPrekodovani)
        {
            var vsechnyPrekodKategorii = Kategorie.OU_ODVOZ_UKAZ.OU_CATEGORY.SelectMany(k => k.OU_PREKODOVANI);
            switch (ZpusobPrekodovani)
            {
                case TypPrekodovani.Vetsi:
                    if (vsechnyPrekodKategorii.Any(p => this.Od < p.OD || this.Od < p.DO))
                    {
                        throw new Exception("Překódovní obsahuje již dříve definované hodnoty.");
                    }
                    break;
                case TypPrekodovani.Mensi:
                    var ouPrekodovaniOd = vsechnyPrekodKategorii.Where(p => p.OD != null);
                    foreach (var p in ouPrekodovaniOd)
                    {
                        if (this.Od > p.OD)//(this.Od >= p.OD)
                        {
                            throw new Exception("Překódovní obsahuje již dříve definované hodnoty.");
                        }
                    }
                    
                    break;
                case TypPrekodovani.Konstanta://Pokud je druh Konstanta mohu zadávat pouze hodnotu Od
                    if(vsechnyPrekodKategorii.Any(p => p.OD == this.Od && p.ZPUSOB == TypPrekodovani.Konstanta.ToString()))
                    {
                        throw new Exception("Překódovní této hodnoty již odvozený ukazatel obsahuje.");
                    }

                    
                    var ouPrekodovani = vsechnyPrekodKategorii.Where(p => p.DO != null);
                    foreach (var p in ouPrekodovani)
                    {
                        if ((decimal.Compare((decimal)p.OD, this.Od) <= 0) && (decimal.Compare(this.Od, (decimal)p.DO) <= 0))//(p.OD <= this.Od <= p.DO)
                        {
                            throw new Exception("Překódovní této hodnoty je již obsaženo v intervalu jiního překódování odvozeného ukazatele.");
                        }
                    }

                    if (vsechnyPrekodKategorii.Where(p => p.ZPUSOB == TypPrekodovani.Vetsi.ToString()).Any(p => this.Od > p.OD))
                    {
                        throw new Exception("Překódovní obsahuje již dříve definované hodnoty.");
                    }
                    if (vsechnyPrekodKategorii.Where(p => p.ZPUSOB == TypPrekodovani.Mensi.ToString()).Any(p => this.Od < p.OD))
                    {
                        throw new Exception("Překódovní obsahuje již dříve definované hodnoty.");
                    }
                    break;
                case TypPrekodovani.Interval://Pokud je druh Interval mohu zadávat Od i Do

                    if (decimal.Compare(this.Od, this.Do) >= 0)//(this.Od >= this.Do)
                    {
                        throw new Exception("Od musí být menší než Do.");
                    }

                    var ouPrekodovaniKonstanty = vsechnyPrekodKategorii.Where(p => p.OD != null);
                    foreach (var p in ouPrekodovaniKonstanty)
                    {
                        if ((decimal.Compare(this.Od,(decimal)p.OD) <= 0) && (decimal.Compare((decimal)p.OD, this.Do) <= 0))//(this.Od <= p.OD <= this.Do)
                        {
                            throw new Exception("Interval obsahuje již dříve definované hodnoty.");
                        }
                    }

                    ouPrekodovaniKonstanty = vsechnyPrekodKategorii.Where(p => p.DO != null);
                    foreach (var p in ouPrekodovaniKonstanty)
                    {
                        if ((decimal.Compare(this.Od, (decimal)p.DO) <= 0) && (decimal.Compare((decimal)p.DO, this.Do) <= 0))//(this.Od <= p.DO <= this.Do)
                        {
                            throw new Exception("Interval obsahuje již dříve definované hodnoty.");
                        }
                    }

                    if (vsechnyPrekodKategorii.Where(p => p.ZPUSOB == TypPrekodovani.Vetsi.ToString()).Any(p => this.Do > p.OD))
                    {
                        throw new Exception("Překódovní obsahuje již dříve definované hodnoty.");
                    }
                    if (vsechnyPrekodKategorii.Where(p => p.ZPUSOB == TypPrekodovani.Mensi.ToString()).Any(p => this.Od < p.OD))
                    {
                        throw new Exception("Překódovní obsahuje již dříve definované hodnoty.");
                    }

                    break;
                case TypPrekodovani.Status://Pokud je druh Status mohu zadávat pouze hodnotu Status
                    if (vsechnyPrekodKategorii.Any(p => p.STATUS == (int)this.Status))
                    {
                        throw new Exception("Status je již obsažena v nějaké z kategorií.");
                    }
                    break;
                case TypPrekodovani.Null:
                    if (vsechnyPrekodKategorii.Any(p => p.ZPUSOB == TypPrekodovani.Null.ToString()))
                    {
                        throw new Exception("Překódování prázdné hodnoty již bylo definováno pro odvozený ukazatel.");
                    }
                    break;
                case TypPrekodovani.NeNull:
                    if (vsechnyPrekodKategorii.Any(p => p.ZPUSOB == TypPrekodovani.NeNull.ToString()))
                    {
                        throw new Exception("Překódování ne prázdné hodnoty již bylo definováno pro odvozený ukazatel.");
                    }
                    break;
                case TypPrekodovani.NeRovno:
                    var prekodZJinychKat = vsechnyPrekodKategorii.Where(c => c.OU_CATEGORY != Kategorie).ToList();
                    if (prekodZJinychKat != null && prekodZJinychKat.Any(p => p.ZPUSOB == TypPrekodovani.NeRovno.ToString()))//Více nerovnáse může být pouze u jedné kategorie
                    {
                        throw new Exception("Nerovná se lze definovat pouze u jedné kategorii.");
                    }
                    if (vsechnyPrekodKategorii.Any(p => p.OD == this.Od && p.ZPUSOB == TypPrekodovani.NeRovno.ToString()))//nemohu překódovat stejné hodnoty
                    {
                        throw new Exception("Překódovní této hodnoty již odvozený ukazatel obsahuje.");
                    }
                    
                    break;
                default:
                    throw new Exception("Neznámí typ překódování");
            }
        }
    }
}
