﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using GalaSoft.MvvmLight.CommandWpf;
using OuDot.Model;
using OuDot.Properties;
using OuDot.ServicesOudot;

namespace OuDot.ViewModel
{
    public class TabulkaDetailsViewModel : ViewModelBase
    {

        private OU_TAB _tabulka;
        
        public OU_TAB Tabulka
        {
            get { return _tabulka; }
            set
            {
                if (_tabulka != value)
                {
                    _tabulka = value;
                    OnPropertyChanged();
                }
            }
        }


        private ICollectionView _prirazeneOU;
        /// <summary>
        /// Binduje se na datagrid, používá se jiný View interface observablecollection, aby se neaplikovali sortovací nebo filtrovací pravidla z původní kolekce a zároveň bylo vidět online přidání nového prvku
        /// </summary>
        public ICollectionView PrirazeneOU
        {
            get { return _prirazeneOU; }
            set
            {
                if (_prirazeneOU != value)
                {
                    _prirazeneOU = value;
                    OnPropertyChanged();
                }
            }
        }


        //private ObservableCollection<ULOHA> _ulohyTeren;

        //public ObservableCollection<ULOHA> UlohyTeren
        //{
        //    get { return _ulohyTeren; }
        //    set
        //    {
        //        if (_ulohyTeren != value)
        //        {
        //            _ulohyTeren = value;
        //            OnPropertyChanged();
        //        }
        //    }
        //}

        //private ULOHA _ulohaTeren;

        //public ULOHA UlohaTeren
        //{
        //    get { return _ulohaTeren; }
        //    set
        //    {
        //        if (_ulohaTeren != value)
        //        {
        //            _ulohaTeren = value;
        //            OnPropertyChanged();
        //        }
        //    }
        //}

        private ULOHA_OBDOBI _obdobiTeren;

        public ULOHA_OBDOBI ObdobiTeren
        {
            get { return _obdobiTeren; }
            set
            {
                if (_obdobiTeren != value)
                {
                    _obdobiTeren = value;
                    OnPropertyChanged();
                }
            }
        }


        private BDBCTABULKY _tabTeren;

        public BDBCTABULKY TabTeren
        {
            get { return _tabTeren; }
            set
            {
                if (_tabTeren != value)
                {
                    _tabTeren = value;
                    OnPropertyChanged();
                }
            }
        }


        private bool _povolitUpravy;
        /// <summary>
        /// Povoluje upravovat vybraná pole (Jméno)
        /// </summary>
        public bool PovolitUpravy
        {
            get { return _povolitUpravy; }
            set
            {
                if (_povolitUpravy != value)
                {
                    _povolitUpravy = value;
                    OnPropertyChanged();
                }
            }
        }



        public RelayCommand NastavitPKCommand { get; private set; }
        public RelayCommand SmazatPKCommand { get; private set; }
        public RelayCommand GenerujSQLCommand { get; private set; }
        public RelayCommand SmazatTabulkuCommand { get; private set; }

        public TabulkaDetailsViewModel(OU_TAB VybranaTabulka)
        {
            if (!DesignerProperties.GetIsInDesignMode(new System.Windows.DependencyObject()))
            {
                // Code runs in Blend --> create design time data.
                Tabulka = VybranaTabulka;
                
                //Pokud obsahuje OU nepovilej upravy
                Tabulka.OU_ODVOZ_UKAZ.CollectionChanged += OU_ODVOZ_UKAZ_CollectionChanged;
                PovolitUpravy = (Tabulka.OU_ODVOZ_UKAZ == null || Tabulka.OU_ODVOZ_UKAZ.Count == 0);

                //UlohyTeren = new ObservableCollection<ULOHA>(DBContext.Instance.Entity.ULOHA);
                ObdobiTeren = DBContext.Instance.Entity.ULOHA_OBDOBI.Find(Settings.Default.IdTerenUlohaObd);
                TabTeren = Tabulka.BDBCTABULKY;

                //seřazení primárních klíčů podle pořadí
                Tabulka.OU_PRIMARNI_KLICE = new ObservableCollection<OU_PRIMARNI_KLICE>(Tabulka.OU_PRIMARNI_KLICE.OrderBy(p => p.PORADI));

                PrirazeneOU = new System.Windows.Data.ListCollectionView(Tabulka.OU_ODVOZ_UKAZ);
            }
            else
            {
                // Code runs "for real"
            }

            NastavitPKCommand = new RelayCommand(() => NastavitPK());
            SmazatPKCommand = new RelayCommand(() => SmazatPK());
            GenerujSQLCommand = new RelayCommand(() => GenerujSQL());
            SmazatTabulkuCommand = new RelayCommand(() => SmazatTabulku());
        }

        
        /// <summary>
        /// Událost na změnu kolekce OU v tabulce
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void OU_ODVOZ_UKAZ_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            var kolekceOU = sender as ObservableCollection<OU_ODVOZ_UKAZ>;

            PovolitUpravy = (kolekceOU == null || kolekceOU.Count == 0);
            
        }

        internal void GenerujSQL()
        {
            try
            {
                Tabulka.GenerujSQLZalozeniPK();
            }
            catch (Exception ex)
            {
                Tabulka.SQL_VLASTNI_KOD = true;
                string zprava = string.Format("Chyba generování sql:{0}{1}", Environment.NewLine, ex.Message);
                MessageBox.Show(zprava, "SQL založení primárních klíčů", MessageBoxButton.OK, MessageBoxImage.Error);
            }
            finally
            {
                OnPropertyChanged("Tabulka");
            }
            
        }

        internal void NastavitPK()
        {
            try
            {
                if (Tabulka.OU_PRIMARNI_KLICE.Any())
                {
                    throw new Exception("Tabulka již primární klíče obsahuje.");
                }

                if (Tabulka.OU_ODVOZ_UKAZ.Any())
                {
                    throw new Exception("Tabulka obsahuje odvozené ukazatele.");
                }

                if (TabTeren == null)
                {
                    throw new Exception("Nebyla vybrána tabulka.");
                }
                //Seřadit podel pořadí
                var serazenePK = TabTeren.BDBC_PRIMARNI_KLICE.OrderBy(p=>p.PORADI);
                Tabulka.BDBCTABULKY = TabTeren;
                foreach (BDBC_PRIMARNI_KLICE bdbcPK in serazenePK)
                {
                    
                    OU_PRIMARNI_KLICE newPK = new OU_PRIMARNI_KLICE();
                    newPK.VlozNovyPK(bdbcPK.JMENO, bdbcPK.PORADI, Tabulka);
                }

                GenerujSQL();
            }
            catch (Exception ex)
            {
                string zprava = string.Format("Chyba nastavení primárních klíčů:{0}{1}", Environment.NewLine, ex.Message);
                MessageBox.Show(zprava, "Nastavení primárního klíče", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }


        internal void SmazatPK()
        {
            try
            {
                if (Tabulka.OU_ODVOZ_UKAZ.Any())
                {
                    throw new Exception("Tabulka obsahuje odvozené ukazatele.");
                }
                else
                {
                    while (Tabulka.OU_PRIMARNI_KLICE.Count > 0)
                    {
                        DBContext.Instance.Entity.OU_PRIMARNI_KLICE.Remove(Tabulka.OU_PRIMARNI_KLICE[0]);
                        Tabulka.BDBCTABULKY = null;
                    }
                    Tabulka.SQL_GENEROVANI_PK = null;
                }
            }
            catch (Exception ex)
            {
                string zprava = string.Format("Chyba nastavení primárních klíčů:{0}{1}", Environment.NewLine, ex.Message);
                MessageBox.Show(zprava, "Nastavení primárního klíče", MessageBoxButton.OK, MessageBoxImage.Error);
            }
            
        }

        /// <summary>
        /// Maže objekt OU_TAB
        /// </summary>
        /// <param name="Tab"></param>
        internal void SmazatTabulku()
        {
            try
            {
                MessageBoxResult mbr = MessageBox.Show("Opravdu si přejete smazat tabulku? Budou ztracena veškerá data i odvozené ukazatele, které obsahuje.", "Smazat tabulku", MessageBoxButton.YesNo, MessageBoxImage.Warning);
                if (mbr != MessageBoxResult.Yes)
                    return;
                //OU_TAB tabVymaz = Tab as OU_TAB;
                if (Tabulka == null)
                    throw new Exception("Neznámý typ.");

                var mw = Application.Current.MainWindow as MainWindow;

                mw.ZavriTabulkaOkno(Tabulka);

                //ZavritOkno.Invoke(Tabulka, null);
                DBContext.Instance.Entity.OU_TAB.Remove(Tabulka);

            }
            catch (Exception ex)
            {
                string zprava = string.Format("Chyba při mazání tabulky:{0}{1}", Environment.NewLine, ex.Message);
                MessageBox.Show(zprava, "Smazat tabulku", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }
    }
}
